<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 06f4047033e8 $
 * $Author: Lukas Imhof $
 * $Date: 2020-06-03 18:46 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_backup_object extends we_contents_object{
	protected $isAddFieldNoSave = false;
	protected $isModifyFieldNoSave = false;
	protected $isDropFieldNoSave = false;
	protected $isForceDropOnSave = false;

	function saveToDB(): bool{
		$this->wasUpdate = $this->ID > 0;

		$this->savePersistentSlotsToDB();
		$ctable = OBJECT_X_TABLE . intval($this->ID);

		if(!$this->wasUpdate){
			$qarr = we_contents_object::DEFAULT_FIELDS;
			$indexe = we_contents_object::DEFAULT_INDEXES;

			$noFields = ['WorkspaceFlag', 'elements', 'WE_CSS_FOR_CLASS'];
			foreach($this->DefaultValues as $key => $value){
				'@phan-var string $key';
				if(!in_array($key, $noFields, true)){
					$arr = explode('_', $key);
					$len = (int) ($value['length'] ?? $this->getElementI($key . 'length'));
					$type = self::switchtypes2($arr[0], $len);
					if(!empty($type)){
						$qarr[$key] = $type;
//add index for complex queries
						if($arr[0] === we_contents_object::TYPE_OBJECT){
							$indexe[] = 'KEY ' . $key . ' (' . $key . ')';
						}
					}
				}
			}

			$this->DB_WE->delTable($ctable);
			$this->DB_WE->addTable($ctable, $qarr, $indexe);


// folder in object schreiben
			if(!($this->OldPath && ($this->OldPath != $this->Path))){
				$fold = new we_folder_objectFile();
				$fold->initByPath($this->getPath(), OBJECT_FILES_TABLE);
			}
		} else {
			$tableInfo = $this->DB_WE->metadata($ctable, we_database_base::META_FULL);
			'@phan-var array<string,array{table:string,name:string,type:string,len:int,flags:int}> $tableInfo';

			$add = $drop = $alter = $addKey = [];
			'@phan-var array<string,string> $drop';
			'@phan-var array<string,string> $addKey';
			'@phan-var array<string,string> $add';
			'@phan-var array<string,string> $alter';

			foreach($this->DefaultValues as $fieldname => $value){
				'@phan-var string $fieldname';
				$arr = explode('_', $fieldname);
				if(!isset($arr[1])){
					continue;
				}

				$fieldtype = $this->getFieldType($arr[0]);
				$len = (int) ($value['length'] ?? 0);
				$type = self::switchtypes2($arr[0], $len);
				$isObject = ($arr[0] === we_contents_object::TYPE_OBJECT);

				if(isset($tableInfo['meta'][$fieldname])){
					$props = $tableInfo[$tableInfo['meta'][$fieldname]];
// the field exists
					if(!empty($fieldtype) && (strtolower($fieldtype) == strtolower($props['type'])) && $len != $props['len']){
						$alter[$fieldname] = $type;
					}
				} elseif(!empty($type)){
					$add[$fieldname] = $type;
					if($isObject){
						$addKey[$fieldname . '_key'] = ' INDEX (`' . $fieldname . '`)';
					}
				}
			}

			if(isset($tableInfo['meta'])){
				foreach(array_keys($tableInfo['meta']) as $key){
					if(!isset($this->DefaultValues[$key]) && substr((string) $key, 0, 3) != 'OF_' && $key != 'ID'){
						$drop[$key] = $key;
					}
				}
			}

//FIXME: deactivated for #9899 - some elements are not present (e.g. object-references) & will be deleted therefore
//With $this->isForceDropOnSave drops can be activated
			if($this->isForceDropOnSave){
				foreach($drop as $value){
					$this->DB_WE->delCol($ctable, (string) $value);
				}
			}

			foreach($alter as $key => $value){
				$this->DB_WE->changeColType($ctable, (string) $key, $value);
			}

			foreach($add as $key => $value){
				$this->DB_WE->addCol($ctable, (string) $key, $value);
			}
			foreach($addKey as $value){
				$this->DB_WE->addKey($ctable, $value);
			}
		}

		$this->elements = [];
		$this->getContentData($ctable);
		return true;
	}

	function getFieldType(string $type): string{
		switch($type){
			case we_contents_object::TYPE_COUNTRY:
			case we_contents_object::TYPE_LANGUAGE:
			case we_contents_object::TYPE_SELECT:
			case we_contents_object::TYPE_LINK:
				return 'meta';
			case we_contents_object::TYPE_HREF:
			case we_contents_object::TYPE_INPUT:
				return 'string';
			case we_contents_object::TYPE_FLOAT:
				return 'real';
			case we_contents_object::TYPE_IMG:
			case we_contents_object::TYPE_FLASHMOVIE:
			case we_contents_object::TYPE_BINARY:
			case we_contents_object::TYPE_OBJECT:
			case we_contents_object::TYPE_DATE:
			case we_contents_object::TYPE_CHECKBOX:
			case we_contents_object::TYPE_INT:
				return 'int';
			case we_contents_object::TYPE_TEXT:
				return 'longtext';
		}
		return '';
	}

	private static function switchtypes2(string $type, int $len = 0): string{
		switch($type){
			case we_contents_object::TYPE_SELECT:
				return ' VARCHAR(' . (($len > 0 && ($len < 256)) ? $len : 255) . ') NOT NULL ';
			case we_contents_object::TYPE_DATE:
				return ' INT unsigned NOT NULL ';
			case we_contents_object::TYPE_INPUT:
				return " VARCHAR(" . (($len > 0 && ($len < 4096)) ? $len : 255) . ') NOT NULL ';
			case we_contents_object::TYPE_COUNTRY:
			case we_contents_object::TYPE_LANGUAGE:
				return ' CHAR(2) NOT NULL ';
			case we_contents_object::TYPE_LINK:
			case we_contents_object::TYPE_HREF:
				return ' TEXT NOT NULL ';
			case we_contents_object::TYPE_TEXT:
				return ' LONGTEXT NOT NULL ';
			case we_contents_object::TYPE_IMG:
			case we_contents_object::TYPE_FLASHMOVIE:
			case we_contents_object::TYPE_BINARY:
				return ' INT unsigned DEFAULT "0" NOT NULL ';
			case we_contents_object::TYPE_CHECKBOX:
				return ' tinyint unsigned DEFAULT "0" NOT NULL';
			case we_contents_object::TYPE_INT:
				return ' INT(' . (($len > 0 && ($len < 256)) ? $len : 11) . ') DEFAULT NULL ';
			case we_contents_object::TYPE_FLOAT:
				return ' DOUBLE DEFAULT NULL ';
			case we_contents_object::TYPE_OBJECT:
				return ' INT unsigned DEFAULT "0" NOT NULL ';
			case we_contents_objectBase::TYPE_MULTIOBJECT:
				return ' TEXT NOT NULL ';
			case we_contents_object::TYPE_SHOPVAT:
				return ' TEXT NOT NULL';
		}
		return '';
	}

	function isFieldExists(string $name, string $type = ''): bool{
		foreach(array_keys($this->DefaultValues) as $fieldname){
			'@phan-var string $fieldname';
			$arr = explode('_', $fieldname);
			if(!isset($arr[0])){
				continue;
			}
			$fieldtype = $arr[0];
			unset($arr[0]);
			$fieldname = implode('_', $arr);
			if($type === ''){
				if($fieldname == $name){
					return true;
				}
			} elseif($fieldname === $name && $fieldtype === $type){
				return true;
			}
		}
		return false;
	}

	/* setter for for property isAddFieldNoSave which allows to construct Classes from within Apps */

	function setIsAddFieldNoSave(bool $isAddFieldNoSave): void{
		$this->isAddFieldNoSave = $isAddFieldNoSave;
	}

	/* getter for for property isAddFieldNoSave which allows to construct Classes from within Apps */

	function getIsAddFieldNoSave(): bool{
		return $this->isAddFieldNoSave;
	}

	/* setter for for property isModifyFieldNoSave which allows to construct Classes from within Apps */

	function setIsModifyFieldNoSave(bool $isModifyFieldNoSave): void{
		$this->isModifyFieldNoSave = $isModifyFieldNoSave;
	}

	/* getter for property isModifyFieldNoSave which allows to construct Classes from within Apps */

	function getIsModifyFieldNoSave(): bool{
		return $this->isModifyFieldNoSave;
	}

	/* setter for property isDropFieldNoSave which allows to construct Classes from within Apps */

	function setIsDropFieldNoSave(bool $isDropFieldNoSave): void{
		$this->isDropFieldNoSave = $isDropFieldNoSave;
	}

	/* getter for property isDropFieldNoSave which allows to construct Classes from within Apps */

	function getIsDropFieldNoSave(): bool{
		return $this->isDropFieldNoSave;
	}

	/* setter for property isForceDropOnSave which allows to construct Classes from within Apps */

	function setIsForceDropOnSave(bool $isForceDropOnSave): void{
		$this->isForceDropOnSave = $isForceDropOnSave;
	}

	/* getter for property isForceDropOnSave which allows to construct Classes from within Apps */

	function getIsForceDropOnSave(): bool{
		return $this->isForceDropOnSave;
	}

}
