<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: f389fd111aab $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-17 01:17 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* * Class for external JS commands
 * dynamic generated content is handled as a command dynamic paramters can be passed.
 * we_processCmd.js is responsible for the correct commands to be executed
 */
class we_base_jsCmd{
	/** @var string[] */
	private $cmds = [];

	/** @var string[] */
	private $cmdData = [];

	/** @var int */
	private static $count = 0;

	/** @var string[] for debug only */
	private static $traces = [];

	/** @var we_base_jsCmd|null */
	private static $active = null;

	public function __construct(){
		self::$traces[] = we_base_errorHandler::getBacktrace(['getBacktrace'])[0];
		self::$count++;
		self::$active = $this;
	}

	public function __destruct(){
		if($this->cmds){
			t_e('unprinted commands', $this->cmds, self::$traces);
		}
	}

	public function clear(): void{
		$this->cmds = [];
	}

	public function addCmd(string $cmd, /* mixed */ $data = [], ...$moreArgs): void{
		if(empty($moreArgs)){
			$this->cmds[] = $cmd;
			$this->cmdData[] = $data ? setDynamicVar($data) : '';
		} else {
			//all data is called as we_cmd(cmd,p1,p2,p3)
			$this->cmds[] = 'we_cmd';
			$this->cmdData[] = setDynamicVar(func_get_args());
		}
	}

	public function addMsg(string $message, int $priority): void{
		$this->cmds[] = 'msg';
		$this->cmdData[] = setDynamicVar(['msg' => $message, 'prio' => $priority]);
	}

	public function getCmds(): string{
		self::$active = null;
		if(empty($this->cmds)){
			self::$count--;
			return '';
		}
		$attrs = ['id' => 'loadVarCmd', 'data-cmds' => implode(',', $this->cmds)];
		foreach($this->cmdData as $pos => $cur){
			if($cur){
				$attrs['data-cmd' . $pos] = $cur;
			}
		}

		$this->cmds = $this->cmdData = [];
		self::$traces[] = we_base_errorHandler::getBacktrace(['getBacktrace'])[0];
		if(self::$count > 1){
			t_e('possible JS error will arrise', self::$traces);
		}

		return we_html_element::jsScript(JS_DIR . 'we_processCmd.js', '', $attrs);
	}

	public function isEmpty(): bool{
		return empty($this->cmds);
	}

}
