<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 11c39a0cfd73 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-05 19:50 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/**
 * General Definition of Customer's Prefers
 *
 */
define('DATE_ONLY_FORMAT', 'Y-m-d');

class we_customer_settings{
	/** @var we_database_base */
	private $db;

	/** @var array<string,string> */
	public $properties = [];

	private const CHANGED_FIELD_TYPES = [
		'dateTime' => 'varchar(24)',
		'country' => 'varchar(4)',
		'language' => 'varchar(2)',
		'select' => 'varchar(200)',
	];
	const FIELD_TYPES = [
		'input' => 'varchar(255)',
		'number' => 'int(11)',
		'select' => 'enum',
		'multiselect' => 'set',
		'textarea' => 'text',
		'dateTime' => 'datetime',
		'date' => 'date',
		'password' => 'varchar(32)',
		'img' => 'int unsigned',
		'country' => "enum('AF','AX','AL','DZ','AS','AD','AO','AI','AQ','AG','AR','AM','AW','AU','AT','AZ','BS','BH','BD','BB','BY','BE','BZ','BJ','BM','BT','BO','BQ','BA','BW','BV','BR','IO','BN','BG','BF','BI','KH','CM','CA','CV','KY','CF','TD','CL','CN','CX','CC','CO','KM','CG','CD','CK','CR','CI','HR','CU','CW','CY','CZ','DK','DJ','DM','DO','EC','EG','SV','GQ','ER','EE','ET','FK','FO','FJ','FI','FR','GF','PF','TF','GA','GM','GE','DE','GH','GI','GR','GL','GD','GP','GU','GT','GG','GN','GW','GY','HT','HM','VA','HN','HK','HU','IS','IN','ID','IR','IQ','IE','IM','IL','IT','JM','JP','JE','JO','KZ','KE','KI','KP','KR','KW','KG','LA','LV','LB','LS','LR','LY','LI','LT','LU','MO','MK','MG','MW','MY','MV','ML','MT','MH','MQ','MR','MU','YT','MX','FM','MD','MC','MN','ME','MS','MA','MZ','MM','NA','NR','NP','NL','NC','NZ','NI','NE','NG','NU','NF','MP','NO','OM','PK','PW','PS','PA','PG','PY','PE','PH','PN','PL','PT','PR','QA','RE','RO','RU','RW','BL','SH','KN','LC','MT','PM','VC','WS','SM','ST','SA','SN','SR','SC','SL','SG','SX','SK','SI','SB','SO','ZA','GS','ES','LK','SD','SR','SJ','SZ','SE','CH','SY','TW','TJ','TZ','TH','TL','TG','TK','TO','TT','TN','TR','TM','TC','TV','UG','UA','AE','GB','US','UM','UY','UZ','VU','VA','VE','VN','VG','VI','WF','EH','YE','YU','ZM','ZW')",
		'language' => "enum('ab','af','an','ar','as','az','be','bg','bn','bo','br','bs','ca','ce','co','cs','cu','cy','da','de','el','en','eo','es','et','eu','fa','fi','fj','fo','fr','fy','ga','gd','gl','gv','he','hi','hr','ht','hu','hy','id','is','it','ja','jv','ka','kg','ko','ku','kw','ky','la','lb','li','ln','lt','lv','mg','mk','mn','mo','ms','mt','my','nb','ne','nl','nn','no','oc','pl','pt','rm','ro','ru','sc','se','sk','sl','so','sq','sr','sv','sw','tk','tr','ty','uk','ur','uz','vi','vo','yi','zh')",
	];
	private const SPECIAL_FIELD_TYPES = [
		'select' => 'enum',
		'multiselect' => 'set',
	];
	const ORDER_TABLE = [
		'ASC' => 'ASC',
		'DESC' => 'DESC'
	];
	const FUNCTION_TABLE = [
		'FIRSTCHAR' => 'UPPER(SUBSTRING(%s,1,1))',
		/* 'ALPH2' => 'UPPER(SUBSTRING(%s,1,2))',
		  'ALPH3' => 'UPPER(SUBSTRING(%s,1,3))', */
		'MINUTE' => 'DATE_FORMAT(%s,\'%%i\')',
		'HOUR' => 'DATE_FORMAT(%s,\'%%H\')',
		'DAY' => 'DAYOFMONTH(%s)',
		'MONTH' => 'MONTH(%s)',
		'YEAR' => 'YEAR(%s)',
		'DAYOFWEEK' => 'DAYOFWEEK(%s)',
		'DAYOFMONTH' => 'DAYOFMONTH(%s)',
		'DAYOFYEAR' => 'DAYOFYEAR(%s)',
		'DAYNAME' => 'DAYNAME(%s)',
		'MONTHNAME' => 'MONTHNAME(%s)',
		'QUARTER' => 'QUARTER(%s)'
	];
	private const TYPE_FUNCTION = [
		'FIRSTCHAR' => 'input,select,textarea,password',
		'MINUTE' => 'date',
		'HOUR' => 'date',
		'DAY' => 'date',
		'MONTH' => 'date',
		'YEAR' => 'date',
		'DAYOFWEEK' => 'date',
		'DAYOFMONTH' => 'date',
		'DAYOFYEAR' => 'date',
		'DAYNAME' => 'date',
		'MONTHNAME' => 'date',
		'QUARTER' => 'date'
	];

	/** @var int */
	public const MaxSearchResults = 99999;

	/** @var array<string,string> */
	private $PropertyTitle = [];

	/** @var array<string,array{type:string,default:string,encrypt:bool}> */
	public $FieldAdds = [];

	/** @var array<string,array<int,array{branch:string,field:string,function:string,order:string}>> */
	public $SortView = [];

	/** @var array<string,string> */
	private $Prefs = [
		'treetext_format' => '#Username (#Forename #Surname)',
		'start_year' => '1900',
		'default_sort_view' => '',
		'default_order' => '',
	];

	/** @var string */
	public $treeTextFormatSQL = '';

	/** @var string[] */
	public $formatFields = [];

	function __construct(){
		$this->db = new DB_WE();

		$this->PropertyTitle = [
			'Username' => g_l('modules_customer', '[username]'),
			'Password' => g_l('modules_customer', '[password]'),
			'Forename' => g_l('modules_customer', '[Forname]'),
			'Surname' => g_l('modules_customer', '[Surname]'),
			'LoginDenied' => g_l('modules_customer', '[login]'),
			'AutoLoginDenied' => g_l('modules_customer', '[autologin]'),
			'AutoLogin' => g_l('modules_customer', '[autologin]'),
			'MemberSince_Date' => g_l('modules_customer', '[MemeberSince]'),
			'LastLogin_Date' => g_l('modules_customer', '[LastLogin]'),
			'LastAccess_Date' => g_l('modules_customer', '[LastAccess]'),
			'ID' => 'ID',
		];
	}

	function load(): void{
		$modified = false;
		$this->properties = $this->db->getAllFirstq('SELECT pref_name,pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="customer"', false);
		$this->SortView = (empty($this->properties['SortView']) ? [] : we_unserialize((string) $this->properties['SortView']));

		if(!empty($this->properties['FieldAdds'])){
			$this->FieldAdds = we_unserialize((string) $this->properties['FieldAdds']);
			//check if all fields are set
			$fields = we_customer_customer::getFieldset();
			foreach($fields as $name){
				$name = (string) $name;
				$tmp = we_customer_customer::getFieldDbProperties($name);
				$old = $this->FieldAdds[$name]['type'] ?? '';
				$this->FieldAdds[$name]['default'] = $tmp['Default'] ?: '';

				switch($old){
					case 'input':
					default:
						$this->FieldAdds[$name]['type'] = $this->getOldFieldType($tmp['Type'], $this->FieldAdds[$name]['default']);
						$modified = $modified || ($old != $this->FieldAdds[$name]['type']);
						break;
					case 'img':
					case 'number':
						$this->FieldAdds[$name]['default'] = '0';
				}
				if(!isset($this->FieldAdds[$name]['encrypt'])){
					$modified = true;
					$this->FieldAdds[$name]['encrypt'] = false;
				}
			}
		}

		if(!empty($this->properties['Prefs'])){
			$this->Prefs = array_merge($this->Prefs, we_unserialize((string) $this->properties['Prefs']));
		}

		$this->treeTextFormatSQL = 'COALESCE(NULLIF(TRIM(CONCAT("' . $this->Prefs['treetext_format'] . '")),""),Username)';
		$field_names = DB_WE::$inst->metaFieldNames(CUSTOMER_TABLE);

		foreach($field_names as $fieldname){
			if(strpos($this->treeTextFormatSQL, '#' . $fieldname) !== false){
				$this->formatFields[] = $fieldname;
			}
			$this->treeTextFormatSQL = str_replace('#' . $fieldname, '",`' . $fieldname . '`,"', $this->treeTextFormatSQL);
		}

		if($modified){
			$this->save();
		}
	}

	function save(): bool{
		//FIXME: make Fieldadds more fields in DB
		$tmp = $this->FieldAdds;
		foreach($tmp as &$cur){
			unset($cur['default']);
		}
		$this->properties['FieldAdds'] = we_serialize($tmp, SERIALIZE_JSON);
		$this->properties['SortView'] = we_serialize($this->SortView, SERIALIZE_JSON);
		$this->properties['Prefs'] = we_serialize($this->Prefs, SERIALIZE_JSON);
		foreach($this->properties as $key => $value){
			'@phan-var string $value';
			$this->db->query('REPLACE INTO ' . SETTINGS_TABLE . ' SET tool="customer",pref_value="' . $this->db->escape($value) . '",pref_name="' . ((string) $key) . '"');
		}
		return true;
	}

	function getPref(string $pref_name): string{
		return (string) $this->properties[$pref_name];
	}

	function setPref(string $pref_name, string $pref_value): void{
		$this->properties[$pref_name] = $pref_value;
	}

	function isFunctionForField(string $function, string $field): bool{
		if(strpos($field, g_l('modules_customer', '[other]')) !== false){
			$field = str_replace(g_l('modules_customer', '[other]') . '_', '', $field);
		}
		$fieldprops = we_customer_customer::getFieldDbProperties($field);

		$fieldtype = $this->getFieldType($fieldprops['Field'], $fieldprops['Type']);
		if($fieldtype != ''){
			foreach(self::TYPE_FUNCTION as $fk => $fv){
				$tmp = explode(',', $fv);
				if(($fk == $function) && (in_array($fieldtype, $tmp))){
					return true;
				}
			}
		}
		return false;
	}

	//returns db field type
	function getDbType(string $field_type, string $fieldname, string $default): string{
		if(array_key_exists($field_type, self::SPECIAL_FIELD_TYPES)){
			if(empty($this->FieldAdds[$fieldname]['default'])){
				return self::FIELD_TYPES['input'];
			}
			return self::SPECIAL_FIELD_TYPES[$field_type] . '(\'' . implode('\',\'', explode(',', $this->FieldAdds[$fieldname]['default'])) . '\')';
		}
		switch($field_type){
			case 'date':
				return self::FIELD_TYPES[$field_type] . ' NULL DEFAULT ' . ($default ?: 'NULL');
			case 'dateTime':
				return self::FIELD_TYPES[$field_type] . ' NULL DEFAULT ' . ($default ?: 'NULL');
			default:
				return self::FIELD_TYPES[$field_type] . ' NOT NULL ' . ($default ? ' DEFAULT \'' . $default . '\'' : '');
		}
	}

	function getFieldType(string $name, string $dbType): string{
		return (string) ($this->FieldAdds[$name]['type'] ?? $this->FieldAdds['input']['type'] ?? (array_search($dbType, self::FIELD_TYPES) ?: 'input'));
	}

//returns predefined field type
	private function getOldFieldType(string $field_type, string &$default): string{
		$expl = explode('(', $field_type);
		$type = $expl[0];
		foreach(self::FIELD_TYPES as $k => $v){
			if($v == $field_type){
				return $k;
			}

			if($v == $type){
				//if this matches we have to extract fieldvalues
				$data = explode(',', rtrim($expl[1], ')'));
				foreach($data as &$d){
					$d = trim($d, '\'');
				}
				$default = implode(',', $data);
				return $k;
			}
		}
		foreach(self::CHANGED_FIELD_TYPES as $k => $v){
			if($v == $field_type){
				return $k;
			}
		}

		return 'input';
	}

	function getPropertyTitle(string $prop): string{
		return ($this->PropertyTitle[$prop] ?? $prop);
	}

	function initCustomerWithDefaults(we_customer_customer $customer): void{
		$meta = $this->db->metadata(CUSTOMER_TABLE, we_database_base::META_FULL);
		'@phan-var array<string,array{table:string,name:string,type:string,len:int,flags:int,default:mixed}> $meta';
		foreach($meta as $field => $vals){
			if(we_customer_customer::isProperty($field) || we_customer_customer::isProtected($field)){
				continue;
			}
			if(empty($vals['default'])){
				continue;
			}
			$customer->{$field} = $vals['default'];
		}
	}

	//field adds operations
	function storeFieldAdd(string $fieldName, string $addName, string $value): void{
		$this->FieldAdds[$fieldName][$addName] = $value;
	}

	function retriveFieldAdd(string $fieldName, string $addName, string $default = ''): string{
		return (string) ($this->FieldAdds[$fieldName][$addName] ?? $default);
	}

	function removeFieldAdd(string $fieldName): void{
		unset($this->FieldAdds[$fieldName]);
	}

	function renameFieldAdds(string $old, string $new): void{
		foreach(array_keys($this->FieldAdds) as $k){
			if($k == $old){
				$tmp = $this->FieldAdds[$k];
				unset($this->FieldAdds[$k]);
				$this->FieldAdds[$new] = $tmp;
			}
		}
	}

	function getSettings(string $settings): string{
		return (string) ($this->Prefs[$settings] ?? '');
	}

	function setSettings(string $settings, string $value): void{
		$this->Prefs[$settings] = $value;
	}

	/**
	 *
	 * @return array<string,string>
	 */
	function getAllSettings(): array{
		return $this->Prefs;
	}

}
