<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 39e7b777091a $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-20 15:43 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_dialog_addThumbnail extends we_dialog_base{
	/** @var int */
	private $selectedId = 0;

	/**
	 * @var we_document_image
	 */
	protected $we_doc = null;

	/** @var string */
	protected $transaction;

	public function __construct(bool $noInternals = false){
		parent::__construct(false);
		//$this->title = g_l('button', '[new_collection][value]');
		$this->initByHttp();
		$this->we_doc = we_contents_root::getSessionDocument($this->transaction);
	}

	protected function initByHttp(): void{
		$this->transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', '', 1);
	}

	private function getSelectorHTML(int &$selectedID): string{
		assert($this->we_doc instanceof we_document_image);

		$thumbnails = new we_html_select(['multiple' => 'multiple', 'name' => "Thumbnails", 'id' => "Thumbnails", 'class' => 'defaultfont', "size" => 6, 'style' => "width: 340px;",
			'onchange' => "select_thumbnails(this);"]);

		$thumbnail_counter_firsttime = true;
		foreach(DB_WE::$inst->getAllFirstq('SELECT ID,Text,Format,description FROM ' . THUMBNAILS_TABLE . ' ORDER BY Text', true, MYSQLI_ASSOC) as $id => $cur){
			if(!in_array($id, $this->we_doc->Thumbs)){
				if(we_base_imageEdit::is_imagetype_read_supported(we_base_imageEdit::GDIMAGE_TYPE[strtolower($this->we_doc->Extension)]) && we_base_imageEdit::is_imagetype_supported(trim($cur['Format']) ?: we_base_imageEdit::GDIMAGE_TYPE[strtolower($this->we_doc->Extension)])){
					$thumbnails->addOption((string) $id, $cur['Text']);
				}
				if($thumbnail_counter_firsttime){
					$selectedID = $id;
					$thumbnails->selectOption(strval($selectedID));
				}

				$thumbnail_counter_firsttime = false;
			}
		}
		return $thumbnails->getHtml();
	}

	protected function getDialogPartsHTML(): we_html_multiIconBoxContent{
		if(!($this->we_doc instanceof we_document_image)){
			exit("ERROR: Couldn't initialize we_document_image object");
		}
		$uniqid = md5(uniqid(__FILE__, true));
		$this->selectedId = 0;

		return (new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine('',$this->getSelectorHTML($this->selectedId) . '<p style="text-align:right">' . we_html_button::create_button('edit_all_thumbs', "javascript:we_cmd('we_thumbnail_edit');") . '</p>'))
				->add(new we_html_multiIconBoxContentLine('','<iframe name="showthumbs" id="showthumbs" src="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=showThumbTable&u=' . $uniqid . '&t=' . $this->transaction . '&id=' . $this->selectedId . '" style="width:340px;height:130px;"></iframe>'));
	}

	protected function getJs(): string{
		return we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_addThumb.js', '', [
				'id' => 'loadVarAdd_thumb', 'data-thumbData' => setDynamicVar([
					'transaction' => $this->transaction
		])]);
	}

	protected function getDialogButtons(): string{
		return we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:add_thumbnails();", '', "", !$this->selectedId, false), '', we_html_button::create_button(we_html_button::CANCEL, 'javascript:WE().util.dialogTop(window).doClose();'));
	}

}
