<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 251a26014e6e $
 * $Author: Lukas Imhof $
 * $Date: 2020-04-18 00:15 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class representing the model data of the search
 */
class we_doclist_model extends we_search_modelBase{
	/** @var string */
	public $whichSearch;

	/** @var int */
	public $height = 0;

	/** @var string */
	public $transaction = '';

	/*	 * @var int position to start the search	 */
	protected $searchstartDoclistSearch = 0;

	/** @var array includes the text to search for	 */
	protected $searchDoclistSearch = [];

	/** @var array includes the operators	 */
	protected $locationDoclistSearch = [];

	/** @var int folder-ids of the docsearch and the tmplsearch	 */
	protected $folderIDDoclist;

	/**
	 * @var string flag that shows what view is set in each search
	 */
	protected $setViewDoclistSearch = we_search_view::VIEW_LIST;

	/**
	 * @var int gives the number of entries in each search for one page
	 */
	protected $anzahlDoclistSearch = 10;

	/**
	 * @var string gives the order
	 */
	protected $OrderDoclistSearch = 'Text';
	protected $searchTablesDoclistSearch = [];

	/**
	 * @var array includes the searchfiels which you are searching in
	 */
	protected $searchFieldsDoclistSearch = [];

	/**
	 * @var int
	 */
	protected $TableID = 0;

	/**
	 * @var int
	 */
	protected $workspaceDoclistSearch = 0;

	/**
	 * @var bool
	 */
	protected $greenOnlyDoclistSearch = false;

	/**
	 * Default Constructor
	 * Can load or create new searchtool object depends of parameter
	 */
	function __construct(string $transaction, string $searchTable, int $folderID = 0, string $setView = we_search_view::VIEW_LIST){
		$this->transaction = $transaction;
		$this->searchTablesDoclistSearch = [$searchTable];
		$this->folderIDDoclist = $folderID;
		$this->setViewDoclistSearch = $setView;

		// fill respective props!!
		switch($searchTable){
			case TEMPLATES_TABLE:
				$this->whichSearch = we_doclist_view::DOCLIST_TEMPLATES;
				break;
			case OBJECT_FILES_TABLE:
				$this->whichSearch = we_doclist_view::DOCLIST_OBJECTFILES;
				break;
			case VFILE_TABLE:
				$this->whichSearch = we_doclist_view::DOCLIST_COLLECTION;
				break;
			case FILE_TABLE:
			default:
				$this->whichSearch = we_doclist_view::DOCLIST_DOCUMENTS;
				break;
		}
	}

	public function initByHttp(string $whichSearch = '', bool $isWeCmd = true): void{
		// IMPORTANT: this is the ONLY place where model vars are set!
		if(we_base_request::int('searchstart' . $this->whichSearch, we_base_request::NOT_VALID_I) !== we_base_request::NOT_VALID_I ||
			we_base_request::int('we_cmd', we_base_request::NOT_VALID_I, 'searchstart' . $this->whichSearch) !== we_base_request::NOT_VALID_I){
			if(isset($_REQUEST['searchFields' . $this->whichSearch])){
				$_REQUEST['we_cmd']['searchFields' . $this->whichSearch] = $_REQUEST['searchFields' . $this->whichSearch];
				$_REQUEST['we_cmd']['location' . $this->whichSearch] = $_REQUEST['location' . $this->whichSearch] ?? [];
				$_REQUEST['we_cmd']['search' . $this->whichSearch] = $_REQUEST['search' . $this->whichSearch];
			} else {
				foreach(/* $_REQUEST['we_cmd'] */we_base_request::stringList('we_cmd', []) as $k => $v){
					if(stristr((string) $k, 'searchFields' . $this->whichSearch . '[') && !stristr((string) $k, 'hidden_')){
						$_REQUEST['we_cmd']['searchFields' . $this->whichSearch][] = $v;
						continue;
					}
					if(stristr((string) $k, 'location' . $this->whichSearch . '[')){
						$_REQUEST['we_cmd']['location' . $this->whichSearch][] = $v;
						continue;
					}
					if(stristr((string) $k, 'search' . $this->whichSearch . '[')){
						$_REQUEST['we_cmd']['search' . $this->whichSearch][] = $v;
					}
				}
			}

			// FIXME: unify the different ways these params are committed
			if(isset($_REQUEST['searchstart' . $this->whichSearch])){
				$_REQUEST['we_cmd']['Order' . $this->whichSearch] = $_REQUEST['Order' . $this->whichSearch];
				$_REQUEST['we_cmd']['setView' . $this->whichSearch] = $_REQUEST['setView' . $this->whichSearch];
				$_REQUEST['we_cmd']['searchstart' . $this->whichSearch] = $_REQUEST['searchstart' . $this->whichSearch];
				$_REQUEST['we_cmd']['anzahl' . $this->whichSearch] = $_REQUEST['anzahl' . $this->whichSearch];
				$_REQUEST['we_cmd']['mode'] = $_REQUEST['mode'];

				$_REQUEST['we_cmd']['searchFields' . $this->whichSearch] = $_REQUEST['hidden_searchFields'] ?? [];
				$_REQUEST['we_cmd']['search' . $this->whichSearch] = $_REQUEST['search'] ?? [];
				$_REQUEST['we_cmd']['location' . $this->whichSearch] = $_REQUEST['location'] ?? [];

				$_REQUEST['we_cmd']['WorkspaceID' . $this->whichSearch] = $_REQUEST['WorkspaceID' . $this->whichSearch];
				$_REQUEST['we_cmd']['GreenOnly' . $this->whichSearch] = $_REQUEST['GreenOnly' . $this->whichSearch];
			}

			$this->searchFieldsDoclistSearch = we_base_request::stringList('we_cmd', $this->searchFieldsDoclistSearch, 'searchFields' . $this->whichSearch);
			$this->searchDoclistSearch = array_map('trim', we_base_request::stringList('we_cmd', $this->searchDoclistSearch, 'search' . $this->whichSearch));
			$this->locationDoclistSearch = we_base_request::stringList('we_cmd', $this->locationDoclistSearch, 'location' . $this->whichSearch);

			// FIXME: dirty...
			switch($this->whichSearch){
				case we_doclist_view::DOCLIST_OBJECTFILES:
					if(($this->searchFieldsDoclistSearch === ['`of`.ID'] &&
							$this->searchDoclistSearch === [''] &&
							$this->locationDoclistSearch === ['IS']) ||
							($this->searchFieldsDoclistSearch === ['Content'] &&
								$this->searchDoclistSearch === [''] &&
								$this->locationDoclistSearch === ['']
							)){
						$this->searchFieldsDoclistSearch = [];
						$this->searchDoclistSearch = [];
						$this->locationDoclistSearch = [];
					}
					break;
				default:
					/*
					if($this->searchFieldsDoclistSearch === ['ID'] &&
							$this->searchDoclistSearch === [''] &&
							$this->locationDoclistSearch === ['IS']){
						// searchfield's default is an empty 'Content' search (not having any impact on the result)
						$this->searchFieldsDoclistSearch = ['Content'];
						$this->searchDoclistSearch = [''];
						$this->locationDoclistSearch = [''];
					}
					 * 
					 */
			}

			$this->OrderDoclistSearch = we_base_request::string('we_cmd', $this->OrderDoclistSearch, 'Order' . $this->whichSearch);
			$this->setViewDoclistSearch = we_base_request::string('we_cmd', $this->setViewDoclistSearch, 'setView' . $this->whichSearch);
			$this->searchstartDoclistSearch = we_base_request::int('we_cmd', $this->searchstartDoclistSearch, 'searchstart' . $this->whichSearch);
			$this->anzahlDoclistSearch = we_base_request::int('we_cmd', $this->anzahlDoclistSearch, 'anzahl' . $this->whichSearch);
			$this->height = count($this->searchFieldsDoclistSearch);

			$this->workspaceDoclistSearch = we_base_request::int('we_cmd', $this->workspaceDoclistSearch, 'WorkspaceID' . $this->whichSearch);
			$this->greenOnlyDoclistSearch = we_base_request::bool('we_cmd', $this->greenOnlyDoclistSearch, 'GreenOnly' . $this->whichSearch);

			// reindex search arrays
			$this->searchDoclistSearch = array_merge($this->searchDoclistSearch);
			$this->searchFieldsDoclistSearch = array_merge($this->searchFieldsDoclistSearch);
			$this->locationDoclistSearch = array_merge($this->locationDoclistSearch);
		}

		$this->prepareModelForSearch();
	}

	public function prepareModelForSearch(): void{
		$this->currentSearchTables = $this->searchTablesDoclistSearch;
		$this->currentSearchFields = $this->searchFieldsDoclistSearch;
		$this->currentLocation = $this->locationDoclistSearch;
		$this->currentSearch = $this->searchDoclistSearch;
		$this->currentFolderID = $this->folderIDDoclist;
		$this->currentOrder = $this->OrderDoclistSearch;
		$this->currentSetView = $this->setViewDoclistSearch;
		$this->currentSearchstart = $this->searchstartDoclistSearch;
		$this->currentAnzahl = $this->anzahlDoclistSearch;
		$this->currentWorkspace = $this->getWorkspace($this->workspaceDoclistSearch);
		$this->currentGreenOnly = $this->greenOnlyDoclistSearch;
	}

	public function getWhichSearch(): string{
		return $this->whichSearch;
	}

	function clearSessionVars(): void{

	}

	public function setTableID(int $tableID = 0): void{
		$this->TableID = $tableID;
	}

	protected function getWorkspace(int $id = 0): int{
		//(we_users_util::in_workspace($this->WorkspaceID, explode(',', $this->searchclass->f('Workspaces')), FILE_TABLE)
		if(!$id){
			$userWSArray = we_users_util::get_ws(FILE_TABLE);
			return (int) (empty($userWSArray) ? 0 : $userWSArray[0]);
		}

		return $this->workspaceDoclistSearch;
	}

}
