<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6917b9add631 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-21 13:24 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/*  a class for handling flashDocuments. */

class we_document_flash extends we_document_video{
	/* Parameternames which are placed within the object-Tag */
	private const ObjectParamNames = ['align', 'border', 'id', 'height', 'hspace', 'name', 'width', 'vspace', 'only', 'style'];

	/** @var string ContentType of the Object  */
	public $ContentType = we_base_ContentTypes::FLASH;

	// is not written yet
	function initByAttribs(array $attribs): void{
		if(($sizingrel = we_weTag_tag::getAttribute('sizingrel', $attribs, 0, we_base_request::INT))){
			$orig_w = we_weTag_tag::getAttribute('width', $attribs, $this->getElementI('width'), we_base_request::UNIT);
			$orig_h = we_weTag_tag::getAttribute('height', $attribs, $this->getElementI('height'), we_base_request::UNIT);
			$attribs['width'] = round($orig_w * $sizingrel);
			$attribs['height'] = round($orig_h * $sizingrel);
		}
		$sizingbase = we_weTag_tag::getAttribute('sizingbase', $attribs, 16, we_base_request::UNIT);
		$sizingstyle = we_weTag_tag::getAttribute('sizingstyle', $attribs, false, we_base_request::STRING);
		if($sizingstyle === 'none'){
			$sizingstyle = false;
		}
		$removeAttribs = ['sizingrel', 'sizingbase', 'sizingstyle', 'xml', '_name_orig'];
		if($sizingstyle){
			$style_width = round($attribs['width'] / $sizingbase, 6);
			$style_height = round($attribs['height'] / $sizingbase, 6);
			$attribs['style'] = ($attribs['style'] ?? '') . ';width:' . $style_width . $sizingstyle . ';height:' . $style_height . $sizingstyle . ';';
			$removeAttribs[] = 'width';
			$removeAttribs[] = 'height';
		}

		$attribs = we_weTag_tag::removeAttribs($attribs, $removeAttribs);

		foreach($attribs as $a => $b){
			if($b != ''){
				$this->setElementS($a, $b, ($a === 'Pluginspage' || $a === 'Codebase' ? 'txt' : 'attrib'));
			}
		}
	}

	/** gets the HTML for including in HTML-Docs */
	function getHtml(bool $dyn = false, bool $preload = false): string{
		$data = $this->getElementS('data');
		$embedAtts = [];
		if(!($this->ID || ($data && !is_dir($data) && is_readable($data)))){
			return ($this->html = '');
		}
		$pluginspage = $this->getElementS('Pluginspage') ?: 'http://www.macromedia.com/shockwave/download/index.cgi?P1_Prod_Version=ShockwaveFlash';
		$codebase = $this->getElementS('Codebase') ?: 'http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=6,0,40,0';

		// fix. older versions of webEdition bgcolor was type txt and not attrib
		if($this->issetElement('bgcolor')){
			$this->elements['bgcolor']['type'] = 'attrib';
		}

		$src = $dyn ?
			WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_base_showBinaryDoc&we_cmd[1]=' . $this->ContentType . '&we_cmd[2]=' . we_base_WE::$transaction . '&rand=' . we_base_file::getUniqueId() :
			$this->Path;
		$attribs = $params = [];

		/*		 * ****************************************************************************
		  /* take all attribs and seperate them in attribs, params and embeds
		  /***************************************************************************** */

		//   first we deal with alt-content
		$alt = $this->getElementS('alt');
		$altContent = '';
		if($alt && ($type = we_base_WE::$we_doc->getElementType($alt))){
			$altContent = we_base_WE::$we_doc->getField(['name' => $alt], $type);
		}

		//  Normal-Version - with embed-tag
		$filter = ['type', 'alt', 'parentid', 'startid'];

		foreach($this->nextElement('attrib') as $key){
			'@phan-var string $key';
			$type = (array_key_exists($key, self::ELEMENT_KEY) ? self::ELEMENT_KEY[$key] : self::ETYPE_TEXT);
			$v = $type === self::ETYPE_NUMBER ? $this->getElementI($key) : $this->getElementS($key);
			if(in_array($key, self::ObjectParamNames, true)){ //  use as name='value'
				$attribs[$key] = $v;
			} else if(!in_array($key, $filter, true)){ //  use as <param>
				$params[$key] = $v;
			}
			if(!in_array($key, $filter, true) && $v){
				$embedAtts[$key] = $v;
			}
		}
		$attribs['classid'] = 'clsid:D27CDB6E-AE6D-11cf-96B8-444553540000';
		$attribs['codebase'] = $codebase;

		if(isset($attribs['only'])){
			return ($this->html = (string) $attribs[$attribs['only']]);
		} else if(!empty($attribs['pathonly'])){
			return ($this->html = (string) $src);
		}


		//   handle with params
		$params['movie'] = $src; //  always needed
		$params = we_weTag_tag::removeAttribs($params, ['xml', 'to', 'nameto', '_name_orig']);
		$html = '';
		foreach($params as $k => $v){
			if($v !== ''){
				$html .= we_weTag_tag::htmlTag('param', ['name' => $k, 'value' => $v, 'xml' => $this->getElementS('xml')]);
			}
		}

		$embedAtts['type'] = 'application/x-shockwave-flash';
		$embedAtts['pluginspage'] = $pluginspage;
		$embedAtts['src'] = $src;

		$html .= we_weTag_tag::htmlTag('embed', $embedAtts, '', true);

		return ($this->html = we_weTag_tag::htmlTag('object', $attribs, $html . $altContent));
	}

	function formProperties(): string{
		$row = 0;
		$content = (new we_html_table(['class' => 'default propertydualtable'], 5, 3))
			->setCol($row, 0, $this->formInput('attrib', 'width#' . self::ETYPE_NUMBER, g_l('weClass', '[width]') . ' (origwidth: ' . $this->getElementI('origwidth') . ')', 155, 0, [
					'type' => 'number', 'onchange' => "_EditorFrame.setEditorIsHot(true);"]))
			->setCol($row, 1, $this->formInput('attrib', 'height#' . self::ETYPE_NUMBER, g_l('weClass', '[height]') . ' (origheight: ' . $this->getElementI('origheight') . ')', 155, 0, [
					'type' => 'number', 'onchange' => "_EditorFrame.setEditorIsHot(true);"]))
			->setCol($row, 2, $this->formSelectFromArray('attrib', 'scale', ['' => '',
					'showall' => g_l('global', '[showall]'),
					'noborder' => g_l('global', '[noborder]'),
					'exactfit' => g_l('global', '[exactfit]')
					], g_l('weClass', '[scale]'), 1, false, ['style' => 'width:155px;', 'onchange' => '_EditorFrame.setEditorIsHot(true);']))
			->setCol(++$row, 0, $this->formInput('attrib', 'hspace', g_l('weClass', '[hspace]'), 155, 0, ['onchange' => "_EditorFrame.setEditorIsHot(true);"]))
			->setCol($row, 1, $this->formInput('attrib', 'vspace', g_l('weClass', '[vspace]'), 155, 0, ['onchange' => "_EditorFrame.setEditorIsHot(true);"]))
			->setCol($row, 2, $this->formInput('attrib', 'name', g_l('weClass', '[name]'), 155, 0, ['onchange' => "_EditorFrame.setEditorIsHot(true);"]))
			->setCol(++$row, 0, $this->formSelectFromArray('attrib', 'play', ['' => g_l('global', '[true]'), 'false' => g_l('global', '[false]')], g_l('weClass', '[play]'), 1, false, [
					'style' => 'width:155px;',
					'onchange' => '_EditorFrame.setEditorIsHot(true);']))
			->setCol($row, 1, $this->formSelectFromArray('attrib', 'quality', ['' => '', 'low' => 'low', 'high' => 'high', 'autohigh' => 'autohigh', 'autolow' => 'autolow',
					'best' => 'best'], g_l('weClass', '[quality]'), 1, false, [
					'style' => 'width:155px;',
					'onchange' => '_EditorFrame.setEditorIsHot(true);']))
			->setCol($row, 2, $this->formColor(155, 'bgcolor', 'attrib'))
			->setCol(++$row, 0, $this->formSelectFromArray('attrib', 'align', ['' => '', 'left' => g_l('global', '[left]'), 'right' => g_l('global', '[right]'),
					'top' => g_l('global', '[top]'),
					'bottom' => g_l('global', '[bottom]')], g_l('weClass', '[align]'), 1, false, ['style' => 'width:155px;', 'onchange' => '_EditorFrame.setEditorIsHot(true);']))
			->setCol($row, 1, $this->formSelectFromArray('attrib', 'salign', ['' => '', 'l' => g_l('global', '[left]'), 'r' => g_l('global', '[right]'), 't' => g_l('global', '[top]'),
					"b" => g_l('global', '[bottom]'),
					'tl' => g_l('global', '[topleft]'), "tr" => g_l('global', '[topright]'), "bl" => g_l('global', '[bottomleft]'), "br" => g_l('global', '[bottomright]')], g_l('weClass', '[salign]'), 1, false, [
					'style' => 'width:155px;',
					'onchange' => '_EditorFrame.setEditorIsHot(true);']))
			->setCol($row, 2, $this->formSelectFromArray('attrib', 'loop', ['' => g_l('global', '[true]'), "false" => g_l('global', '[false]')], g_l('weClass', '[loop]'), 1, false, [
					'style' => 'width:155px;',
					'onchange' => '_EditorFrame.setEditorIsHot(true);']))
			->setCol(++$row, 0, $this->formSelectFromArray('attrib', 'wmode', ['' => '', 'window' => 'window', 'opaque' => 'opaque', 'transparent' => 'transparent'], g_l('weClass', '[wmode]'), 1, false, [
				'style' => 'width:155px;', 'onchange' => '_EditorFrame.setEditorIsHot(true);']));
		return $content->getHtml();
	}

	function formOther(): string{
		return
			$this->formInput('txt', 'Pluginspage', 'Pluginspage', 0, 0, ['onchange' => "_EditorFrame.setEditorIsHot(true);"]) . we_html_element::br() .
			$this->formInput('txt', 'Codebase', 'Codebase', 0, 0, ['onchange' => "_EditorFrame.setEditorIsHot(true);"]);
	}

	protected function getThumbnail(int $width = 150, int $height = 100): string{
		$elemWidth = $this->getElementI('width');
		$elemHeight = $this->getElementI('height');
		$scale = $this->getElementS('scale');
		$hspace = $this->getElementI('hspace');
		$vspace = $this->getElementI('vspace');
		$name = $this->getElementS('name');
		$play = $this->getElementS('play');
		$quality = $this->getElementS('quality');
		$bgcolor = $this->getElementS('bgcolor');
		$align = $this->getElementS('align');
		$salign = $this->getElementS('salign');
		$loop = $this->getElementS('loop');
		$wmode = $this->getElementS('wmode');
		$origwidth = $this->getElementI('origwidth');
		$origheight = $this->getElementI('origheight');

		$this->setElementI('width', $width, 'attrib');
		$this->setElementI('height', $height, 'attrib');
		$this->setElementS('scale', '', 'attrib');
		$this->setElementI('hspace', 0, 'attrib');
		$this->setElementI('vspace', 0, 'attrib');
		$this->setElementS('name', '', 'attrib');
		$this->setElementS('play', 'true', 'attrib');
		$this->setElementS('quality', '', 'attrib');
		$this->setElementS('bgcolor', '', 'attrib');
		$this->setElementS('align', '', 'attrib');
		$this->setElementS('salign', '', 'attrib');
		$this->setElementS('loop', '', 'attrib');
		$this->setElementS('wmode', 'window', 'attrib');
		$this->setElementI('origwidth', 0, 'attrib');
		$this->setElementI('origheight', 0, 'attrib');

		$html = $this->getHtml(true);
		$this->setElementI('width', $elemWidth, 'attrib');
		$this->setElementI('height', $elemHeight, 'attrib');
		$this->setElementS('scale', $scale, 'attrib');
		$this->setElementI('hspace', $hspace, 'attrib');
		$this->setElementI('vspace', $vspace, 'attrib');
		$this->setElementS('name', $name, 'attrib');
		$this->setElementS('play', $play, 'attrib');
		$this->setElementS('quality', $quality, 'attrib');
		$this->setElementS('bgcolor', $bgcolor, 'attrib');
		$this->setElementS('align', $align, 'attrib');
		$this->setElementS('salign', $salign, 'attrib');
		$this->setElementS('loop', $loop, 'attrib');
		$this->setElementS('wmode', $wmode, 'attrib');
		$this->setElementI('origwidth', $origwidth, 'attrib');
		$this->setElementI('origheight', $origheight, 'attrib');

		return $html;
	}

	/**
	 * function will determine the size of any GIF, JPG, PNG.
	 * This function uses the php Function with the same name.
	 * But the php function doesn't work with some images created from some apps.
	 * So this function uses the gd lib if nothing is returned from the php function
	 *
	 * @static
	 * @public
	 * @return array
	 * @throws we_exception_image in case the size can not be determined
	 * @param string $filename complete path of the image
	 */
	private static function getimagesize(string $filename): array{
		$arr = @getimagesize($filename);

		if(is_array($arr) && (count($arr) >= 4) && $arr[0] && $arr[1]){
			return $arr;
		}
		return we_base_imageEdit::getimagesize($filename);
	}

	/**
	 * saves the data of the document
	 *
	 * @return boolean
	 * @param boolean $resave
	 */
	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		// get original width and height of the image
		$arr = $this->getOrigSize(true, true);
		$this->setElementI('origwidth', $arr[0] ?? 0, 'attrib');
		$this->setElementI('origheight', $arr[1] ?? 0, 'attrib');
		if(!$this->getElementI('width')){
			$this->setElementI('width', $this->getElementI('origwidth'), 'attrib');
		}
		if(!$this->getElementI('height')){
			$this->setElementI('height', $this->getElementI('origheight'), 'attrib');
		}

		$docChanged = $this->DocChanged; // will be reseted in parent::we_save()
		if(parent::we_save($resave, $skipHook)){
			if($docChanged){
				$this->DocChanged = true;
			}

			return true;
		}

		return false;
	}

	function we_rewrite(): bool{
		$ret = parent::we_rewrite();
		$r = $this->we_save();
		return $ret && $r;
	}

	public function uploadImportFile(array $fileVars, array $docVars): void{
		try{
			$we_size = $this->getimagesize(WE_DOCUMENT_ROOT . $fileVars['fileTemp']);
			$this->setElementI('width', $we_size[0], 'attrib');
			$this->setElementI('height', $we_size[1], 'attrib');
			$this->setElementI('origwidth', $we_size[0], 'attrib');
			$this->setElementI('origheight', $we_size[1], 'attrib');
		} catch (we_exception_image $e){

		}

		parent::uploadImportFile($fileVars, $docVars);
	}

	/**
	 * Calculates the original image size of the image.
	 * Returns an array like the PHP function getimagesize().
	 * If the array is empty the image is not uploaded or an error occured
	 *
	 * @param boolean $calculateNew
	 * @throws we_exception_image
	 * @return array
	 */
	function getOrigSize(bool $calculateNew = false, bool $useOldPath = false): array{
		$arr = [0, 0, 0, ''];
		if(!$this->DocChanged && $this->ID){
			if($this->getElementI('origwidth') && $this->getElementI('origheight') && ($calculateNew == false)){
				return [$this->getElementI('origwidth'), $this->getElementI('origheight'), 0, ''];
			} else {
				// we have to calculate the path, because maybe the document was renamed
				//$path = $this->getParentPath() . '/' . $this->Filename . $this->Extension;
				return self::getimagesize(WE_DOCUMENT_ROOT . (($useOldPath && $this->OldPath) ? $this->OldPath : $this->Path));
			}
		} else if(($tmp = $this->getElementS('data'))){
			$arr = self::getimagesize($tmp);
		}
		return $arr;
	}

	static function checkAndPrepare(string $formname, we_contents_root $doc): void{
		// check to see if there is an image to create or to change
		if(!(isset($_FILES['we_ui_' . $formname]) && is_array($_FILES['we_ui_' . $formname]) && isset($_FILES['we_ui_' . $formname]['name']) && is_array($_FILES['we_ui_' . $formname]['name']) )){
			return;
		}
		//$webuserId = isset($_SESSION['webuser']['ID']) ? $_SESSION['webuser']['ID'] : 0;

		foreach($_FILES['we_ui_' . $formname]['name'] as $videoName => $filename){
			'@phan-var string $videoName';
			'@phan-var string $filename';

			$videoDataId = we_base_request::string(we_fileupload::USERINPUT_TYPE_FLASHMOVIE . $videoName);

			if(!$videoDataId || empty($_SESSION[$videoDataId])){
				continue;
			}

			$_SESSION[$videoDataId]['doDelete'] = false;

			if(we_base_request::bool(we_fileupload::USERINPUT_TYPE_DEL . $videoName)){
				$_SESSION[$videoDataId]['doDelete'] = true;
			} elseif($filename &&
				// file is selected, check to see if it is an image
				we_base_ContentTypes::getFromFilename($filename) === we_base_ContentTypes::FLASH){
				$videoid = $doc->getElementI($videoName);

				// move document from upload location to tmp dir
				$_SESSION[$videoDataId]['serverPath'] = TEMP_PATH . we_base_file::getUniqueId();
				we_base_file::insertIntoCleanUp($_SESSION[$videoDataId]['serverPath']);
				move_uploaded_file((string) $_FILES['we_ui_' . $formname]['tmp_name'][$videoName], $_SESSION[$videoDataId]['serverPath']);

				$unique = we_base_file::getUniqueId();
				$tmp_Filename = $videoName . '_' . $unique . '_' . preg_replace('[^A-Za-z0-9._-]', '', (string) $_FILES['we_ui_' . $formname]['name'][$videoName]);

				if($videoid){
					$_SESSION[$videoDataId]['id'] = $videoid;
				}

				$_SESSION[$videoDataId]['fileName'] = preg_replace('#^(.+)\..+$#', '${1}', $tmp_Filename);
				$_SESSION[$videoDataId]['extension'] = (strpos($tmp_Filename, ".") > 0) ? preg_replace('#^.+(\..+)$#', '${1}', $tmp_Filename) : '';
				$_SESSION[$videoDataId]['text'] = $_SESSION[$videoDataId]['fileName'] . $_SESSION[$videoDataId]['extension'];
				$_SESSION[$videoDataId]['unique'] = $unique;

				$we_size = getimagesize($_SESSION[$videoDataId]['serverPath']);
				$_SESSION[$videoDataId]['imgwidth'] = $we_size[0];
				$_SESSION[$videoDataId]['imgheight'] = $we_size[1];
				$_SESSION[$videoDataId]['type'] = $_FILES['we_ui_' . $formname]['type'][$videoName];
				$_SESSION[$videoDataId]['size'] = $_FILES['we_ui_' . $formname]['size'][$videoName];
			}
		}
	}

	public function getPropertyPage(we_base_jsCmd $jsCmd): string{
		return (new we_html_multiIconBox((new we_html_multiIconBoxContent())
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[path]'), $this->formPath(), we_html_multiIconBox::PROP_PATH)))
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[document]'), $this->formIsProtected(), we_html_multiIconBox::PROP_DOC)))
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[other]'), $this->formOther(), we_html_multiIconBox::PROP_OTHER))->fold())
					, 'PropertyPage'))
				->marginLeft()
				->get();
	}

}
