<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 34a4f9a228a6 $
 * $Author: Lukas Imhof $
 * $Date: 2020-06-18 13:48 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_editor_base{
	/** @var ?we_base_jsCmd	 */
	protected $jsCmd = null;

	/** @var ?we_contents_root */
	protected $we_doc = null;

	/** @var string */
	protected $title = '';

	/** @var string */
	protected $transaction = '';

	public function __construct(?we_base_mainmodel $we_doc, ?we_base_jsCmd $jsCmd = null, string $we_transaction = ''){
		$this->jsCmd = we_base_WE::$rebuildActive ? null : ($jsCmd ?? new we_base_jsCmd());
		$this->we_doc = $we_doc;
		$this->transaction = $we_transaction;
	}

	public function getJsCmd(): we_base_jsCmd{
		if(!$this->jsCmd){
			throw new we_exception_base('jsCmd not set');
		}
		return $this->jsCmd;
	}

	public abstract function show(): string;

	/**
	 *
	 * @param string $form
	 * @param string $header
	 * @param array<string,string>  $bodyAttr
	 * @param array<string,string>|null $formAttr set to false if no form should be generated,else set form attributes
	 * @return string
	 */
	protected function getPage(string $form, string $header = '', array $bodyAttr = [], ?array $formAttr = []): string{
		return we_html_tools::getHtmlTop($this->title, $header .
				we_html_element::jsScript(JS_DIR . 'multiIconBox.js') .
				(!empty($this->we_doc) ? we_editor_script::get($this->we_doc) : '') .
				$this->jsCmd->getCmds(), we_html_element::body(array_merge(
						[
							'class' => 'weEditorBody',
							'onunload' => "doUnload()"
						], $bodyAttr), ($formAttr === null ?
					$form :
					we_html_element::form(array_merge([
						'name' => we_base_constants::WE_FORM,
						'method' => 'post',
						'onsubmit' => 'return false;'
							], $formAttr), we_contents_base::hiddenTrans() .
						$form .
						we_html_element::hidden('we_complete_request', 1)))
				)
		);
	}

}
