<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: fa0353dd1263 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-18 16:54 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_editor_functions{
	/** @var string */
	static $we_responseText = '';

	/** @var int */
	static $we_responseTextType = 0;

	/** @var bool */
	static $publish_doc = false;

	protected static function initVars(): we_contents_root{
		we_base_WE::$we_EDITOR = true;

// init document
//we_session assumes to have transaction in parameter 'we_transaction'
		$we_transaction = we_base_WE::$transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_transaction', we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', '', 1));
		if(!$we_transaction){
			//t_e('not found transaction in we_transaction nor we_cmd[1]: try we_cmd[2] instead');
			$we_transaction = we_base_WE::$transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', 'hmm', 2);
		}
		$doc = we_contents_root::getSessionDocument($we_transaction);
		//we_base_WE::setWeDoc($doc);
//	if document is locked - only Preview mode is possible. otherwise show warning.
		self::documentLocking($doc);
		return $doc;
	}

	protected static function initVarsDoc(): we_document_base{
		return self::initVars();
	}

	private static function initVarsObj(): we_contents_objectBase{
		return self::initVars();
	}

	public static function processObjectCommonCmd(string $cmd0): void{
		$we_doc = self::initVarsObj();
		switch($cmd0){
			case 'object_add_workspace':
				$we_doc->add_workspace(we_base_request::int('we_cmd', -1, 1));
				break;
			case 'object_del_workspace':
				$we_doc->del_workspace(we_base_request::int('we_cmd', -1, 1));
				break;
		}
		self::includeEditorDefault($we_doc, we_base_WE::$transaction, null);
	}

	public static function processObjectFileCmd(string $cmd0): void{
		$we_doc = self::initVarsObj();
		assert($we_doc instanceof we_contents_objectFile);
		switch($cmd0){
			case 'we_object_class_getClassWs':
				$we_doc->ws_from_class();
				break;
			case 'restore_defaults':
				$we_doc->restoreDefaults();
				break;
		}
		self::includeEditorDefault($we_doc, we_base_WE::$transaction, null);
	}

	public static function processObjectCmd(string $cmd0): void{
		$we_doc = self::initVarsObj();
		assert($we_doc instanceof we_contents_object);
		switch($cmd0){
			case 'object_add_workspace':
				$we_doc->add_workspace(we_base_request::int('we_cmd', -1, 1));
				break;
			case 'object_del_workspace':
				$we_doc->del_workspace(we_base_request::int('we_cmd', -1, 1));
				break;
			case 'object_changeTempl_ob':
				$we_doc->changeTempl_ob(we_base_request::int('we_cmd', 0, 1), we_base_request::int('we_cmd', 0, 2));
				break;
			case 'we_object_class_cssAdd':
				$we_doc->add_css(we_base_request::intList('we_cmd', [], 1));
				break;
			case 'we_object_class_cssDel':
				$we_doc->del_css(we_base_request::int('we_cmd', 0, 1));
				break;
		}
		self::includeEditorDefault($we_doc, we_base_WE::$transaction, null);
	}

	public static function processPropertyCmd(string $cmd0): void{
		$we_doc = self::initVars();
		assert($we_doc instanceof we_document_base);
		$reload = null;
		switch($cmd0){
			case 'copyDocumentSelect':
			case 'copyDocument':
				$we_doc->InWebEdition = true;
				if($we_doc->copyDoc(we_base_request::int('we_cmd', 0, 1)) && $we_doc instanceof we_contents_template){
					$reload = new we_base_jsCmd();
					$reload->addCmd('reloadMainEditor');
				}
				break;
			case 'add_navi':
				$we_doc->addNavi(we_base_request::int('we_cmd', 0, 1), we_base_request::string('we_cmd', '', 2), we_base_request::int('we_cmd', 0, 3), we_base_request::int('we_cmd', 0, 4));
				break;
			case 'delete_navi':
				$we_doc->delNavi(we_base_request::_(we_base_request::FILE, 'we_cmd', '', 1));
				break;
			case 'delete_all_navi':
				$we_doc->delAllNavi();
				break;
			case 'add_cat':
				$we_doc->addCat(we_base_request::intList('we_cmd', [], 1));
				break;
			case 'delete_cat':
				$we_doc->delCat(we_base_request::int('we_cmd', 0, 1));
				break;
			case 'delete_all_cats':
				$we_doc->Category = [];
				break;
		}
		self::includeEditorDefault($we_doc, we_base_WE::$transaction, $reload);
	}

	public static function processImageCmd(string $cmd0): void{
		$we_doc = self::initVars();
		assert($we_doc instanceof we_document_image);
		switch($cmd0){
			case 'doImage_convertGIF':
				$we_doc->convert('gif');
				break;
			case 'doImage_convertPNG':
				$we_doc->convert('png');
				break;
			case 'doImage_convertJPEG':
				$we_doc->convert('jpg', we_base_request::int('we_cmd', 0, 1));
				break;
			case 'doImage_crop':
				we_base_file::insertIntoCleanUp(($filename = TEMP_PATH . we_base_file::getUniqueId()));
				copy($we_doc->getElementS('data'), $filename);
				$width = we_base_request::int('we_cmd', 0, 3);
				$height = we_base_request::int('we_cmd', 0, 4);

				try{
					if(($img = imagecrop(we_base_imageEdit::ImageCreateFromFileReplacement($filename), [
						'x' => we_base_request::int('we_cmd', 0, 1),
						'y' => we_base_request::int('we_cmd', 0, 2),
						'width' => $width,
						'height' => $height
						]))){
						we_base_imageEdit::ImageWriteToFileReplacement($img, $filename);
						imagedestroy($img);
					}
				} catch (we_exception_base $e){
					t_e($e->getMessage());
					break;
				}
				$we_doc->setElementS('data', $filename);
				$we_doc->setElementI('width', $width, 'attrib');
				$we_doc->setElementI('origwidth', $width, 'attrib');
				$we_doc->setElementI('height', $height, 'attrib');
				$we_doc->setElementI('origheight', $height, 'attrib');
				$we_doc->DocChanged = true;
				break;
			case 'del_thumb':
				$we_doc->del_thumbnails(we_base_request::int('we_cmd', 0, 1));
				break;
			case 'do_add_thumbnails':
				$we_doc->add_thumbnails(we_base_request::intList('we_cmd', [], 1));
				break;
			case 'resizeImage':
				$we_doc->resizeImage(we_base_request::int('we_cmd', 0, 1), we_base_request::int('we_cmd', 0, 2), we_base_request::int('we_cmd', 0, 3));
				break;
			case 'rotateImage':
				$we_doc->rotateImage(we_base_request::int('we_cmd', 0, 1), we_base_request::int('we_cmd', 0, 2), we_base_request::int('we_cmd', 0, 3), we_base_request::int('we_cmd', 0, 4));
				break;
		}
		self::includeEditorDefault($we_doc, we_base_WE::$transaction, null);
	}

	public static function processEditorCmd(string $cmd0): void{
		$we_doc = self::initVars();
		$jsCmd = null;
		switch($cmd0){
			case 'load_editor':
// set default tab for creating new imageDocuments to "metadata":
				if($we_doc->ContentType === we_base_ContentTypes::IMAGE && $we_doc->ID == 0){
					$_SESSION['weS']['EditPageNr'] = $we_doc->EditPageNr = we_base_constants::WE_EDITPAGE_CONTENT;
				}
				break;
			case 'switch_edit_page':
				$_SESSION['weS']['EditPageNr'] = we_base_request::int('we_cmd', 0, 1);
				$we_doc->EditPageNr = $_SESSION['weS']['EditPageNr'];
				if(we_editor_functions::isSeeMode()){
					$jsCmd = new we_base_jsCmd();
					$jsCmd->addCmd('we_gui_reloadEditFooter', we_base_WE::$transaction);
				}
				break;
			case 'revert_published':
				$we_doc->revert_published();
				break;
		}
		self::includeEditor($we_doc, we_base_WE::$transaction, $jsCmd);
	}

	public static function processEditorDocumentCmd(string $cmd0): void{
		$we_doc = self::initVars();
		assert($we_doc instanceof we_document_base);
		$jsCmd = null;
		switch($cmd0){
			case 'delete_list':
				we_tag_block::removeEntryFromList($we_doc, we_base_request::string('we_cmd', '', 1), we_base_request::int('we_cmd', 0, 2), we_base_request::_(we_base_request::STRING, 'we_cmd', '', 3));
				break;
			case 'insert_entry_at_list':
				we_tag_block::insertEntryAtList($we_doc, we_base_request::string('we_cmd', '', 1), we_base_request::int('we_cmd', 0, 2), we_base_request::int('we_cmd', 1, 3));
				break;
			case 'up_entry_at_list':
				we_tag_block::upEntryAtList($we_doc, we_base_request::string('we_cmd', '', 1), we_base_request::int('we_cmd', 0, 2), we_base_request::int('we_cmd', 1, 4));
				break;
			case 'down_entry_at_list':
				we_tag_block::downEntryAtList($we_doc, we_base_request::string('we_cmd', '', 1), we_base_request::int('we_cmd', 0, 2), we_base_request::int('we_cmd', 1, 4));
				break;
			case 'add_entry_to_list':
				we_tag_block::addEntryToList($we_doc, we_base_request::string('we_cmd', '', 1), we_base_request::int('we_cmd', 1, 2));
				break;
			case 'change_link':
				$we_doc->changeLink(we_base_request::string('we_cmd', '', 1));
				break;
			case 'doctype_changed':
				assert($we_doc instanceof we_document_webEdition);
				$we_doc->changeDoctype(0, true);
				$jsCmd = new we_base_jsCmd();
				$jsCmd->addCmd('we_gui_reloadEditFooter', we_base_WE::$transaction);
				$jsCmd->addCmd('we_gui_reloadEditHeader', we_base_WE::$transaction);
				break;
			case 'template_changed':
				$we_doc->setDocumentControlElements();
				$jsCmd = new we_base_jsCmd();
				$jsCmd->addCmd('we_gui_reloadEditFooter', we_base_WE::$transaction);
				$jsCmd->addCmd('we_gui_reloadEditHeader', we_base_WE::$transaction);
				break;
			case 'remove_image':
				$we_doc->remove_image(we_base_request::string('we_cmd', '', 1));
				break;
			case 'wrap_on_off':
				$_SESSION['weS']['we_wrapcheck'] = we_base_request::bool('we_cmd', false, 1);
				$we_doc->EditPageNr = we_base_constants::WE_EDITPAGE_CONTENT;
				$_SESSION['weS']['EditPageNr'] = we_base_constants::WE_EDITPAGE_CONTENT;
				break;
			case 'delete_link':
				$name = we_base_request::string('we_cmd', '', 1);
				$we_doc->delElement($name);
				break;
		}
		self::includeEditor($we_doc, we_base_WE::$transaction, $jsCmd);
	}

	private static function unPublishInc(string $we_transaction, we_base_jsCmd $jsCmd = null, array $we_responseJS = [
		]): void{
		echo we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'editor_save.js', '', ['id' => 'loadVarEditor_save', 'data-editorSave' => setDynamicVar([
					'we_editor_save' => false,
					'we_transaction' => $we_transaction,
					'isSEEMode' => we_editor_functions::isSeeMode(),
					'we_JavaScript' => [],
					'we_responseText' => self::$we_responseText,
					'we_responseTextType' => self::$we_responseTextType,
					'we_cmd5' => [],
					'we_responseJS' => $we_responseJS,
			])]) . ($jsCmd ? $jsCmd->getCmds() : ''), we_html_element::body());
	}

	public static function saveInc(string $we_transaction, we_contents_root $we_doc, string $we_responseText = '', int $we_responseTextType = 0, array $wejs = [
		], bool $wasSaved = false, bool $saveTemplate = false, array $we_responseJS = [
		], bool $isClose = false, bool $showAlert = false, bool $publish_doc = false): void{
		$reload = [];
		$we_responseJS = empty($we_responseJS) ? [] : $we_responseJS;

		if($wasSaved){
			// DOC was saved, mark open tabs to reload if necessary
			// was saved - not hot anymore

			switch($we_doc->ContentType){
				case we_base_ContentTypes::FOLDER_DOCUMENT:
				case we_base_ContentTypes::FOLDER_TEMPLATE:
				case we_base_ContentTypes::FOLDER_OBJECTFILE:
				case we_base_ContentTypes::FOLDER_VFILE:
					if($we_doc->wasMoved()){
						$reload[$we_doc->Table] = DB_WE::$inst->getAllq('SELECT f.ID FROM ' . $we_doc->Table . ' f INNER JOIN ' . LOCK_TABLE . ' l ON f.ID=l.ID AND l.tbl="' . stripTblPrefix($we_doc->Table) . '" WHERE f.Path LIKE "' . $we_doc->Path . '/%"', true);
					}
					break;

				case we_base_ContentTypes::TEMPLATE: // #538 reload documents based on this template
					$reloadDocsTempls = we_rebuild_base::getTemplAndDocIDsOfTemplate($we_doc->ID, false, false, true, true);

					// reload all documents based on this template
					$reload[FILE_TABLE] = $reloadDocsTempls['documentIDs'];
					//no need to reload the edit tab, since this is not changed & Preview is always regenerated
//			$reload[TEMPLATES_TABLE] = implode(',', $reloadDocsTempls['templateIDs']);

					break;
				case we_base_ContentTypes::OBJECT:
					$reload[OBJECT_FILES_TABLE] = array_map('intval', DB_WE::$inst->getAllq('SELECT of.ID FROM ' . OBJECT_FILES_TABLE . ' `of` INNER JOIN ' . LOCK_TABLE . ' l ON of.ID=l.ID AND l.tbl="' . stripTblPrefix(OBJECT_FILES_TABLE) . '" WHERE of.IsFolder=0 AND of.TableID=' . intval($we_doc->ID), true));
			}
		}
		echo we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'editor_save.js', '', [
				'id' => 'loadVarEditor_save',
				'data-editorSave' => setDynamicVar([
					'we_editor_save' => true,
					'we_transaction' => $we_transaction,
					'isHot' => ($we_responseText && $we_responseTextType === we_base_util::WE_MESSAGE_ERROR),
					'wasSaved' => $wasSaved,
					'wasPublished' => $we_doc->getDocumentState() === we_contents_root::STATE_PUBLISHED,
					'isPublished' => $publish_doc,
					'isSEEMode' => we_editor_functions::isSeeMode(),
					'reloadEditors' => array_filter($reload),
					'ContentType' => $we_doc->ContentType,
					'docID' => $we_doc->ID,
					'EditPageNr' => $we_doc->EditPageNr,
					'saveTmpl' => $saveTemplate,
					'isClose' => $isClose,
					'showAlert' => $showAlert,
					'we_responseText' => $we_responseText,
					'we_responseTextType' => $we_responseTextType,
					'we_JavaScript' => $wejs,
					'we_cmd5' => we_base_request::_(we_base_request::JSON, 'we_cmd', '', 5), // this is we_responseJS through save-template-question
					'we_responseJS' => $we_responseJS,
					'docHasPreview' => in_array(we_base_constants::WE_EDITPAGE_PREVIEW, $we_doc->EditPageNrs, true),
			])]), we_html_element::body());
	}

	public static function templateSave(string $we_transaction): void{
		echo we_html_tools::getHtmlTop('', we_html_element::jsElement('var url=WE().consts.dirs.WEBEDITION_DIR+"we_cmd.php?' . http_build_query(
					['we_cmd' => [
							0 => 'save_document',
							1 => $we_transaction,
							2 => 1,
							5 => we_base_request::_(we_base_request::RAW, 'we_cmd', '', 5), //is json64
							6 => we_base_request::_(we_base_request::RAW, 'we_cmd', '', 6), //is json64
						],
						'we_transaction' => $we_transaction,
						'we_complete_request' => 1
				]) .
				'";
(new (WE().util.jsOverlay)(window, WE().layout.weEditorFrameController.getActiveEditorFrame().EditorFrameWindow,url,"templateSaveQuestion",WE().consts.size.dialog.smaller,WE().consts.size.dialog.tiny)).open();
'), '<body></body>');
	}

	public static function templateSaveQuestion(string $we_transaction, bool $isTemplatesUsedByThisTemplate, int $nrDocsUsedByThisTemplate, array $we_responseJS): void{
		$we_cmd6 = we_base_request::_(we_base_request::JSON, 'we_cmd', '', 6);

		$alerttext = ($isTemplatesUsedByThisTemplate ?
			g_l('alert', '[template_save_warning2]', false, true) :
			sprintf((g_l('alert', ($nrDocsUsedByThisTemplate === 1) ? '[template_save_warning1]' : '[template_save_warning]', false, true)), $nrDocsUsedByThisTemplate)
			);

		echo we_html_tools::getHtmlTop(g_l('global', '[question]'), we_html_element::jsScript(JS_DIR . 'template_save_question.js', '', ['id' => 'loadVarTemplate_save_question',
				'data-editorSave' => setDynamicVar([
					'we_transaction' => $we_transaction,
					'we_responseJS' => $we_responseJS,
					'we_cmd6' => $we_cmd6
			])]), we_html_element::body(['class' => "weEditorBody", 'onload' => "doOnLoad(this)"], we_html_tools::yesNoCancelDialog($alerttext, '<span class="fa-stack fa-lg" style="color:#F2F200;"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i></span>', true, true, true, 'pressed_yes_button()', 'pressed_no_button()', 'pressed_cancel_button()')
			)
		);
	}

	public static function processSave(we_contents_root $we_doc, string $we_transaction, bool $wasSaved, bool $saveTemplate, bool $isClose, bool $showAlert, array $wejs, string $we_responseText = '', int $we_responseTextType = 0): void{
		if(($tmp = $we_doc->getErrMsg())){
			$we_responseText .= '\n\n' . $tmp;
			$we_responseTextType = we_base_util::WE_MESSAGE_ERROR;
		}
		if(($tmp = $we_doc->getWarningMsg())){
			$we_responseText .= '\n\n' . $tmp;
			$we_responseTextType = $we_responseTextType < we_base_util::WE_MESSAGE_WARNING ? we_base_util::WE_MESSAGE_WARNING : $we_responseTextType;
		}
		if(($tmp = $we_doc->getNoticeMsg())){
			$we_responseText .= '\n\n' . $tmp;
			$we_responseTextType = $we_responseTextType < we_base_util::WE_MESSAGE_NOTICE ? we_base_util::WE_MESSAGE_NOTICE : $we_responseTextType;
		}
		if($_SERVER['REQUEST_METHOD'] === 'POST' && !we_base_request::bool('we_complete_request')){
			//will show the message
			$jsCmd = new we_base_jsCmd();
			$jsCmd->addMsg(g_l('weEditor', '[incompleteRequest]'), we_base_util::WE_MESSAGE_ERROR);
			echo $jsCmd->getCmds();
			return;
		}
		$we_doc->saveInSession($we_transaction); // save the changed object in session

		$wejs[] = ['setEditorDocumentId', $we_doc->ID]; // save/ rename a document

		self::saveInc($we_transaction, $we_doc, strtr($we_responseText, ['\n' => we_html_element::br()]), $we_responseTextType, $wejs, $wasSaved, $saveTemplate, we_editor_save::$responseJS, $isClose, $showAlert, self::$publish_doc);
		if(we_modules_info::isActive(we_modules_info::SCHEDULER) && SCHEDULER_TRIGGER !== we_schedule_scheduler::TRIGGER_CRON){
			we_base_request::finishRequest();
			we_schedule_scheduler::trigger_schedule(5);
		}
	}

	//if document is locked - only Preview mode is possible. otherwise show warning.
	public static function documentLocking(we_contents_root $we_doc): void{
		$userID = $we_doc->isLockedByUser();
		if($userID != 0 && $userID != we_users_user::getID() && $we_doc->ID){ // document is locked
			$we_doc->showLockedWarning($userID);
		} elseif($userID != we_users_user::getID() && we_editor_functions::isSeeMode() && $we_doc->EditPageNr != we_base_constants::WE_EDITPAGE_PREVIEW){
// lock document, if in seeMode and EditMode !!, don't lock when already locked
			$we_doc->lockDocument();
		}
	}

	private static function saveTemplate(we_contents_template $we_doc, string $we_transaction, array &$wejs): array{
		$wasNew = $wasSaved = false;
		$doAutomaticRebuild = (we_base_request::bool('we_cmd', false, 8));
		if($doAutomaticRebuild){
// if  we_cmd[8] is set, it means that 'automatic rebuild' was clicked
// so we need to check we_cmd[3] (means save immediately) and we_cmd[4] (means rebuild immediately)
			$_REQUEST['we_cmd'][3] = 1;
			$_REQUEST['we_cmd'][4] = 1;
		}
		if(we_base_request::bool('we_cmd', false, 5)){ //Save in version
			$_REQUEST['we_cmd'][5] = false;
			$we_doc->we_publish();
		}

		$arr = we_rebuild_base::getTemplAndDocIDsOfTemplate($we_doc->ID, true, true);
		$nrDocsUsedByThisTemplate = $arr ? count($arr['documentIDs']) : 0;
		$isTemplatesUsedByThisTemplate = $we_doc->ID && fB('SELECT 1 FROM ' . TEMPLATES_TABLE . ' WHERE MasterTemplateID=' . $we_doc->ID . ' LIMIT 1');
		$somethingNeedsToBeResaved = ($nrDocsUsedByThisTemplate > 0 || $isTemplatesUsedByThisTemplate);

		if(we_base_request::bool('we_cmd', false, 2)){
//this is the second call to save_document (see next else command)
			we_editor_functions::templateSaveQuestion($we_transaction, $isTemplatesUsedByThisTemplate, $nrDocsUsedByThisTemplate, we_editor_save::$responseJS);
			$we_doc->saveInSession($we_transaction); // save the changed object in session
			exit();
		}
		if(!we_base_request::bool('we_cmd', false, 3) && $somethingNeedsToBeResaved){
// this happens when the template is saved and there are documents which use the template and "automatic rebuild" is not checked!
			we_editor_functions::templateSave($we_transaction); // this calls again we_cmd with save_document and sets we_cmd[2]
			$we_doc->saveInSession($we_transaction); // save the changed object in session
			exit();
		}
//this happens when we_cmd[3] is set and not we_cmd[2]
		$oldID = $we_doc->ID;
		if($we_doc->we_save()){
			if($oldID == 0){
				$we_doc->lockDocument();
			}
			$wasSaved = true;
			$wasNew = (intval($we_doc->ID) === 0);

			$wejs[] = ['we_setPath', $we_doc->Path, $we_doc->getText(), intval($we_doc->ID), ($we_doc->Publish_Date !== null ? ($we_doc->Table !== TEMPLATES_TABLE && $we_doc->Mod_Date > $we_doc->Publish_Date ? 'changed' : 'published') : 'notpublished')];
			$wejs[] = ['setEditorDocumentId', $we_doc->ID];
			$wejs[] = $we_doc->getUpdateTreeCmd(true, null); // save/ rename a document
			$we_responseText = sprintf(g_l('weEditor', '[' . $we_doc->ContentType . '][response_save_ok]', true) ?: g_l('weClass', '[response_save_ok]'), $we_doc->Path);
			$we_responseTextType = we_base_util::WE_MESSAGE_NOTICE;
			if(we_base_request::bool('we_cmd', false, 4)){
// this happens when the documents which uses the templates has to be rebuilt. (if user clicks "yes" at template save question or if automatic rebuild was set)
				if($somethingNeedsToBeResaved){
					$wejs[] = ['unsetHot'];
					$wejs[] = ['rebuildTemplates', $we_doc->ID, sprintf($we_responseText, $we_doc->Path)];
					$we_responseText = '';
				}
			}
		} else {
// we got an error while saving the template
			$wejs = [];
			$we_responseText = sprintf(g_l('weEditor', '[' . $we_doc->ContentType . '][response_save_notok]', true) ?: g_l('weClass', '[response_save_notok]'), $we_doc->Path);
			$we_responseTextType = we_base_util::WE_MESSAGE_ERROR;
		}

//FIXME: is this safe??? Code-Injection!
		if(($js = we_base_request::_(we_base_request::JSON, 'we_cmd', '', 6))){
			$wejs[] = $js;
		}
		return [$wasNew, $wasSaved, $we_responseText, $we_responseTextType];
	}

	public static function doUnpublish(): void{
		$we_doc = self::initVars();
		$we_transaction = we_base_WE::$transaction;
		assert($we_doc instanceof we_document_base || $we_doc instanceof we_contents_collection);
		if(!$we_doc->Publish_Date){
			self::$we_responseText = sprintf(g_l('weEditor', '[' . $we_doc->ContentType . '][response_not_published]'), $we_doc->Path);
			self::$we_responseTextType = we_base_util::WE_MESSAGE_ERROR;
			self::unPublishInc($we_transaction);
			return;
		}
		$jsCmd = new we_base_jsCmd();
		if($we_doc->we_unpublish()){
			self::$we_responseText = sprintf(g_l('weEditor', '[' . $we_doc->ContentType . '][response_unpublish_ok]'), $we_doc->Path);
			self::$we_responseTextType = we_base_util::WE_MESSAGE_NOTICE;
			if($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PROPERTIES || $we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_INFO){
				we_editor_save::$responseJS[] = ['switch_edit_page', $we_doc->EditPageNr, $we_transaction]; // wird in Templ eingef?gt
			}
//	When unpublishing a document stay where u are.
//	uncomment the following line to switch to preview page.
			we_editor_save::$responseJS[] = ['we_gui_reloadEditFooter', $we_transaction];
			$jsCmd->addCmd('setEditorDocumentId', $we_doc->ID);
			$we_doc->getUpdateTreeCmd(true, $jsCmd); // save/ rename a document
		} else {
			self::$we_responseText = sprintf(g_l('weEditor', '[' . $we_doc->ContentType . '][response_unpublish_notok]', true) ?: g_l('weClass', '[response_unpublish_notok]'), $we_doc->Path);
			self::$we_responseTextType = we_base_util::WE_MESSAGE_ERROR;
		}
		if($_SERVER['REQUEST_METHOD'] === 'POST' && !we_base_request::bool('we_complete_request')){
			self::$we_responseText = g_l('weEditor', '[incompleteRequest]');
			self::$we_responseTextType = we_base_util::WE_MESSAGE_ERROR;
		} else {
			$we_doc->saveInSession($we_transaction); // save the changed object in session
		}
		self::unPublishInc($we_transaction, $jsCmd, we_editor_save::$responseJS);
	}

	public static function saveDocument(): void{
		//FIXME: make this a better, local var
		global $we_JavaScript;

		$we_doc = self::initVars();
		$we_transaction = we_base_WE::$transaction;
		$makeSameDoc = we_base_request::bool('we_cmd', false, 4);
		$we_JavaScript = [];

		$wasNew = $showAlert = $isClose = $wasSaved = $saveTemplate = false;
		$we_responseTextType = we_base_util::WE_MESSAGE_NOTICE;

		if(!$we_doc->ContentType){
			exit(' ContentType Missing !!! ');
		}
		if(($we_responseText = $we_doc->checkFieldsOnSave())){
			we_editor_functions::processSave($we_doc, $we_transaction, false, false, false, false, [], $we_responseText, we_base_util::WE_MESSAGE_ERROR);
			exit();
		}

		if($we_doc->ID){
			$we_JavaScript[] = ['setEditorDocumentId', $we_doc->ID]; // save/ rename a document
		}
		switch($we_doc->ContentType){
			case we_base_ContentTypes::TEMPLATE:
				$saveTemplate = true;
				assert($we_doc instanceof we_contents_template);
				list($wasNew, $wasSaved, $we_responseText, $we_responseTextType) = we_editor_functions::saveTemplate($we_doc, $we_transaction, $we_JavaScript);
				break;
			case we_base_ContentTypes::PDF:
			case we_base_ContentTypes::APPLICATION:
				assert($we_doc instanceof we_document_binary);
				if((!we_base_permission::hasPerm([we_base_perm::WORK_OTHER_NEW])) && in_array($we_doc->Extension, we_base_ContentTypes::getExtension(we_base_ContentTypes::HTML, true))){
					we_editor_functions::processSave($we_doc, $we_transaction, false, false, false, false, [], sprintf(g_l('weEditor', '[application/*][response_save_wrongExtension]'), $we_doc->Path, $we_doc->Extension), we_base_util::WE_MESSAGE_ERROR);
					exit();
				}
			//falls through
			default:
				$we_responseTextType = we_base_util::WE_MESSAGE_NOTICE;
				$wf_flag = false;
				$wasNew = (intval($we_doc->ID) == 0);
				$wasPubl = (!empty($we_doc->Publish_Date));
				if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) && $we_doc->ContentType != we_base_ContentTypes::OBJECT && $we_doc->ContentType != we_base_ContentTypes::OBJECT_FILE && !we_users_util::in_workspace($we_doc->ParentID, we_users_util::get_ws($we_doc->Table), $we_doc->Table)){
					we_editor_functions::saveInc($we_transaction, $we_doc, g_l('alert', '[' . FILE_TABLE . '][not_im_ws]'), we_base_util::WE_MESSAGE_ERROR, [], false, false, we_editor_save::$responseJS, false, false, self::$publish_doc);
					exit();
				}
				if(!$we_doc->userCanSave()){
					we_editor_functions::saveInc($we_transaction, $we_doc, g_l('alert', '[access_denied]'), we_base_util::WE_MESSAGE_ERROR, [], false, false, we_editor_save::$responseJS, false, false, self::$publish_doc);
					exit();
				}

				$oldID = $we_doc->ID;
				if(!$we_doc->we_save()){
					we_editor_functions::processSave($we_doc, $we_transaction, false, false, false, false, [], sprintf(g_l('weEditor', '[' . $we_doc->ContentType . '][response_save_notok]', true) ?: g_l('weClass', '[response_save_notok]'), $we_doc->Path), we_base_util::WE_MESSAGE_ERROR);
					exit();
				}
				// FIXME: (really) dirty fix for we_content_object => the only content class using we_editor_save
				$we_JavaScript = array_merge($we_JavaScript, we_editor_save::$javaScript);

				if($oldID == 0){
					$we_doc->lockDocument();
				}
				$wasSaved = true;
				switch($we_doc->ContentType){
					case we_base_ContentTypes::OBJECT:
						$we_JavaScript[] = ['updateVTab', OBJECT_FILES_TABLE];
						break;
					case we_base_ContentTypes::COLLECTION:
						$we_JavaScript[] = ['updateVTab', VFILE_TABLE];
						break;
				}
				$err = $we_doc->getErrMsg();
				$we_responseText = sprintf(g_l('weEditor', '[' . $we_doc->ContentType . '][response_save_ok]', true) ?: g_l('weClass', '[response_save_ok]'), $we_doc->Path) . ($err ? ' - ' . $err : '');

				if(we_base_request::bool('we_cmd', false, 5)){
					if(($foo = $we_doc->publInScheduleTable())){
						$we_responseText .= ' - ' . sprintf(g_l('weEditor', '[' . $we_doc->ContentType . '][autoschedule]'), $foo->format(g_l('date', '[format][default]')));
					} elseif($we_doc->we_publish()){
						if(we_modules_info::isActive(we_modules_info::WORKFLOW) && (we_workflow_utility::inWorkflow($we_doc->ID, $we_doc->Table))){
							we_workflow_utility::removeDocFromWorkflow($we_doc->ID, $we_doc->Table, we_users_user::getID(), '');
						}
						// FIXME: fast and dirty fix for wrong messages: we should avoid publishing at all when not needed!
						if(in_array($we_doc->ContentType, [we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::OBJECT_FILE, we_base_ContentTypes::HTML], true)){
							$we_responseText .= ' - ' . sprintf(g_l('weEditor', '[' . $we_doc->ContentType . '][response_publish_ok]'), $we_doc->Path);
						}
// SEEM, here a doc is published
						self::$publish_doc = true;
						switch($we_doc->EditPageNr){
							case we_base_constants::WE_EDITPAGE_PROPERTIES:
							case we_base_constants::WE_EDITPAGE_INFO:
							case we_base_constants::WE_EDITPAGE_PREVIEW:
								if(we_editor_functions::isNormalMode() && (!$makeSameDoc)){
									we_editor_save::$responseJS[] = ['switch_edit_page', $we_doc->EditPageNr, $we_transaction];
									we_editor_save::$responseJS[] = ['we_gui_reloadEditFooter', $we_transaction]; // reload the footer with the buttons
								}
						}
					} else {
						$we_responseText .= ' - ' . sprintf(g_l('weEditor', '[' . $we_doc->ContentType . '][response_publish_notok]'), $we_doc->Path);
						$we_responseTextType = we_base_util::WE_MESSAGE_ERROR;
					}
				} else {
					//in case we only save the document, make updates to schedule table as well
					if(($foo = $we_doc->publInScheduleTable())){
						$we_responseText .= ' - ' . sprintf(g_l('weEditor', '[' . $we_doc->ContentType . '][autoschedule]'), $foo->format(g_l('date', '[format][default]')));
					}
					$tmp = we_base_request::int('we_cmd', 0, 7);
					if(($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_INFO && (!$makeSameDoc)) || $tmp){
						$we_responseText = $tmp ? '' : $we_responseText;
						$we_responseTextType = $tmp ? we_base_util::WE_MESSAGE_ERROR : $we_responseTextType;
						we_editor_save::$responseJS[] = ['switch_edit_page', $we_doc->EditPageNr, $we_transaction];

						switch($tmp){
							case 1:
								$we_JavaScript[] = ['we_workflow_isIn', $we_transaction, $makeSameDoc];
								$wf_flag = true;
								break;
							case 2:
								$we_JavaScript[] = ['we_workflow_pass', $we_transaction];
								$wf_flag = true;
								break;
							case 3:
								$we_JavaScript[] = ['we_workflow_decline', $we_transaction];
								$wf_flag = true;
								break;
							default:
						}
					}
// Bug Fix #2065 -> Reload Preview Page of other documents
					elseif($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PREVIEW){
						switch($we_doc->ContentType){
							case we_base_ContentTypes::APPLICATION:
							case we_base_ContentTypes::PDF:
								$we_JavaScript[] = ['switch_edit_page', $we_doc->EditPageNr, $we_transaction];
						}
					}
				}

				$we_JavaScript[] = $we_doc->getUpdateTreeCmd(!$makeSameDoc, null);
				switch($we_doc->Table){
					case FILE_TABLE:
					case OBJECT_FILES_TABLE:
						$we_JavaScript[] = ['loadIfActive', VFILE_TABLE];
				}

				if($wasNew || (!$wasPubl)){
					if(in_array($we_doc->ContentType, we_base_ContentTypes::FOLDER_TYPES)){
						$we_JavaScript[] = ['switch_edit_page', $we_doc->EditPageNr, $we_transaction];
					}
					$we_JavaScript[] = ['we_gui_reloadEditFooter', $we_transaction];
					$we_JavaScript[] = ['we_gui_reloadEditHeader', $we_transaction];
				}

				switch($we_doc->ContentType){
					case we_base_ContentTypes::HTML:
					case we_base_ContentTypes::WEDOCUMENT:
					case we_base_ContentTypes::OBJECT_FILE:
						switch($we_doc->getDocumentState()){
							case we_contents_root::STATE_UNPUBLISHED:
								$state = 'notpublished';
								break;
							case we_contents_root::STATE_MODIFIED:
								$state = 'changed';
								break;
							default:
								$state = 'published';
						}
						break;
					default:
						$state = 'published';
				}
				$we_JavaScript[] = ['we_setPath', $we_doc->Path, $we_doc->getText(), intval($we_doc->ID), $state];

				if(!we_modules_info::isActive(we_modules_info::SCHEDULER)){
					$we_JavaScript[] = ['setEditorDocumentId', $we_doc->ID];
				}
				if($we_doc->canHaveVariants(true) && $we_doc instanceof we_document_base){
					we_base_variants::setVariantDataForModel($we_doc);
				}

				if(($js = we_base_request::_(we_base_request::JSON, 'we_cmd', '', 6))){
					$we_JavaScript[] = $js;
					$isClose = in_array('closeDocument', $js, true);
				} else if($makeSameDoc && (!$wf_flag)){
					$we_doc->makeSameNew();
					$we_JavaScript[] = ['we_setPath', $we_doc->Path, $we_doc->getText(), intval($we_doc->ID), ($we_doc->Publish_Date !== null ? ($we_doc->Table !== TEMPLATES_TABLE && $we_doc->Mod_Date > $we_doc->Publish_Date ? 'changed' : 'published') : 'notpublished')];
//	switch to propertiy page, when user is allowed to do so.
					switch($_SESSION['weS']['we_mode']){
						case we_base_constants::MODE_SEE:
							$showAlert = true; //	don't show confirm box in editor_save.inc
							we_editor_save::$responseJS[] = ['switch_edit_page', (we_base_permission::hasPerm([we_base_perm::WORK_PROPERTIES_CAN_SEE]) ? we_base_constants::WE_EDITPAGE_PROPERTIES : $we_doc->EditPageNr),
								$we_transaction];
							break;
						case we_base_constants::MODE_NORMAL:
							we_editor_save::$responseJS[] = ['switch_edit_page', $we_doc->EditPageNr, $we_transaction];
							break;
					}
				}
		}

		if($wasNew){ // add to history
			$we_JavaScript[] = ['addHistory', $we_doc->Table, $we_doc->ID, $we_doc->ContentType];
		}

		we_editor_functions::processSave($we_doc, $we_transaction, $wasSaved, $saveTemplate, $isClose, $showAlert, $we_JavaScript, $we_responseText, $we_responseTextType);
	}

	public static function includeEditor(we_contents_root $we_doc, string $we_transaction, we_base_jsCmd $jsCmd = null): void{
// prevent persmissions overriding
		$perms = $_SESSION['perms'];
		if($we_doc instanceof we_document_base && !$we_doc->IsDynamic){
			switch($we_doc->EditPageNr){
				case we_base_constants::WE_EDITPAGE_CONTENT:
					if($we_doc->ContentType !== we_base_ContentTypes::WEDOCUMENT){
						break;
					}
				case we_base_constants::WE_EDITPAGE_PREVIEW:
					switch($we_doc->ContentType){
						case we_base_ContentTypes::WEDOCUMENT:
						case we_base_ContentTypes::HTML:
							self::includeEditorStatic($we_doc, $we_transaction, $jsCmd);
							$_SESSION['perms'] = $perms;
							return;
					}
			}
		}
		self::includeEditorDefault($we_doc, $we_transaction, $jsCmd);
		$_SESSION['perms'] = $perms;
	}

	/*
	 * if the document is a webEdition document, we save it with a temp-name (path of document+extension) and redirect
	 * to the tmp-location. This is done for the content- and preview-editpage.
	 * With html-documents this is only done for preview-editpage.
	 * We need to do this, because, when the pages has for example jsp. content, it will be parsed right!
	 * This is only done when the IsDynamic - PersistantSlot is false.
	 */

	private static function includeEditorStatic(we_contents_root $we_doc, string $we_transaction, we_base_jsCmd $jsCmd = null): void{//this is really unclear what is done here
		$we_include = $we_doc->editor($jsCmd);
		/* if(!$we_include){ // object does not handle html-output, so we need to include a template( return value)
		  exit('Nothing to include ...');
		  } */

		if($we_doc->ContentType === we_base_ContentTypes::WEDOCUMENT){
//remove all already parsed names
			assert($we_doc instanceof we_document_webEdition);
			$we_doc->resetUsedElements();
		}
		//if(is_object($we_include)){
		$docContents = $we_include->show();
		/* } else {
		  ob_start();
		  include((substr(strtolower($we_include), 0, strlen(WE_DOCUMENT_ROOT)) === strtolower(WE_DOCUMENT_ROOT) ?
		  '' : WE_INCLUDES_PATH) .
		  $we_include);
		  $docContents = (string) ob_get_clean() ?? '';
		  } */

		//usedElementNames is set after include
		$we_doc->saveInSession($we_transaction); // save the changed object in session
//  SEEM the file
//  but only, if we are not in the template-editor
		switch($we_doc->ContentType){
			case we_base_ContentTypes::TEMPLATE:
				$contents = $docContents;
				break;
			default:
				$tmpcontents = we_gui_SEEM::parseDocument($docContents);

				$contents = ($jsCmd && !$jsCmd->isEmpty() ? (strpos($tmpcontents, '</head>') ?
					str_replace('</head>', $jsCmd->getCmds() . '</head>', $tmpcontents) :
					$jsCmd->getCmds() . $tmpcontents) :
					$tmpcontents
					);
		}

		switch($we_doc->Extension){
			case '.js':
			case '.css':
			case '.wml':
			case '.xml':
				$tempName = we_base_file::getUniqueId() . '.html';
				break;
			default:
				$tempName = we_base_file::getUniqueId() . $we_doc->Extension;
		}

//
// --> Glossary Replacement
//

		$glossarHTML = ((we_modules_info::isActive(we_modules_info::GLOSSARY) && !we_base_WE::$editmode && $we_doc->InGlossar == 0) ?
			we_glossary_replace::replace($contents, $we_doc->Language) :
			$contents);

		if(we_base_WE::$editmode){
			$matches = [];
			preg_match_all('|<form( name="' . we_base_constants::WE_FORM . '")|i', $glossarHTML, $matches, PREG_PATTERN_ORDER);
			if($matches && !empty($matches[0])){
				//find the number of we-forms
				$all = count($matches[0]);
				$no = count(array_filter($matches[1]));
				if($no > 1){
					//sth very bad must have happend to have 2 we forms in one page
					$warn = $no . ' ' . g_l('parser', '[form][we]');
					t_e($warn, str_replace('.html', '.tmpl', $we_doc->Path));
				}
				if($all - $no){
					$warn = $no . ' ' . g_l('parser', '[form][duplicate]');
					t_e($warn, str_replace('.html', '.tmpl', $we_doc->Path));
				}
			}
		}
		we_base_file::saveTemp($glossarHTML, $tempName, 60);

		header('Location: ' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_base_showTemp&file=' . str_replace(WEBEDITION_DIR, '', TEMP_DIR . $tempName));
	}

	protected static function includeEditorDefault(we_contents_root $we_doc, string $we_transaction, we_base_jsCmd $jsCmd = null): void{
		$we_include = $we_doc->editor($jsCmd);
		echo
		$we_include->show() .
		($jsCmd ? $jsCmd->getCmds() : '');
		$we_doc->saveInSession($we_transaction); // save the changed object in session
	}

	public static function isSeeMode(): bool{
		return isWE() && $_SESSION['weS']['we_mode'] === we_base_constants::MODE_SEE;
	}

	public static function isNormalMode(): bool{
		return isWE() && $_SESSION['weS']['we_mode'] === we_base_constants::MODE_NORMAL;
	}

}
