<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 78bd71ccd4b3 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-05-04 14:43 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Function:    Utility class that implements basic html collection operations
 *
 * Description: Provides functions for creating html tags
 */
class we_html_baseCollection extends we_html_baseElement{
	/** @var we_html_baseCollection[]|we_html_baseElement[]|null[] */
	var $childs = [];

	/**
	 * Constructor
	 *
	 * @param		$tagname								string
	 * @param		$need_end_tag							bool
	 * @param		$attribs								array
	 * @param		$childs_num								int					(optional)
	 *
	 */
	public function __construct(string $tagname, bool $need_end_tag = true, array $attribs = [], int $childs_num = 0){
		parent::__construct($tagname, $need_end_tag, $attribs);
		$this->childs = array_fill(0, $childs_num, null);
	}

	/**
	 * Function adds new element to the collection
	 *
	 * @param		$child									we_html_baseElement
	 *
	 * @return		void
	 */
	function addChild(we_html_baseElement $child): void{
		$this->childs[] = $child;
	}

	/**
	 * Function dels element from the collection
	 * Parameter $childid is a sequence number in the childs array
	 *
	 * @param		$childid									int
	 *
	 * @return		void
	 */
	function delChild(int $childid): void{
		unset($this->childs[$childid]);
		$this->childs = array_values($this->childs);
	}

	/**
	 * Function sets child wit specified childid with given child object
	 *
	 * @param		int $childId
	 * @param		we_html_baseElement $child
	 *
	 * @return		void
	 */
	function setChild(int $childId, we_html_baseElement $child): void{
		$this->childs[$childId] = $child;
	}

	/**
	 * Function gives reference on child object back
	 *
	 * @param		int $childId
	 *
	 * @return		we_html_baseElement|we_html_baseCollection|null
	 */
	function &getChild(int $childId){
		return $this->childs[$childId];
	}

	public function getHtml(): string{
		$childs_content = '';
		foreach($this->childs as $vc){
			$childs_content .= $vc ? $vc->getHtml() : '';
		}
		$this->content = $childs_content;
		return parent::getHtml();
	}

}
