<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 3644b5810887 $
 * $Author: Lukas Imhof $
 * $Date: 2020-05-11 22:05 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//TODO in modulesFrames: set module-settings as class properties instead of looping them through as method parameters!

abstract class we_modules_frame{
	const TREE_DEFAULT_WIDTH = 240;
	const FOOTER = 'edfooter';

	protected $module;

	/** @var we_database_base */
	protected $db;

	/** @var string */
	protected $frameset;

	/** @var we_modules_view */
	protected $View;

	/** @var we_tree_base */
	protected $Tree;

	/** @var bool */
	protected $showTreeHeader = false;

	/** @var bool */
	protected $showTreeFooter = false;

	/** @var int */
	protected $treeWidth = 0;

	/** @var bool */
	protected $hasIconbar = false;
	//FIXME: make this protected again

	/** @var we_base_jsCmd */
	public $jsCmd;

	function __construct(){
		$this->db = new DB_WE();
		$this->frameset = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=' . $this->module;
		$this->jsCmd = new we_base_jsCmd();
	}

	function getHTMLDocument(string $body, string $extraHead = ''): string{
		return we_html_tools::getHtmlTop($this->module, $extraHead . $this->jsCmd->getCmds(), $body);
	}

	public function getHTML(string $what = '', int $mode = 0, int $step = 0): string{
		switch($what){
			case 'frameset':
				return $this->getHTMLFrameset($this->Tree->getJSTreeCode());
			case 'editor':
				return $this->getHTMLEditor();
			case 'edheader':
				return $this->getHTMLEditorHeader();
			case 'edbody':
				return $this->getHTMLEditorBody();
			case self::FOOTER:
				return $this->getHTMLEditorFooter([]);
			case 'cmd':
				return $this->getHTMLCmd();
			case 'tree':
				return $this->getHTMLTree();
			case 'search':
				return $this->getHTMLSearch();
			case 'treeheader':
				return $this->getHTMLTreeHeader();
			default:
				$extraJS = $this->jsCmd->getCmds();
				$ret = ($extraJS ? $this->getHTMLDocument(we_html_element::body(), $extraJS) : '' );
				t_e(__FILE__ . ' unknown reference: ' . $what, ($ret ? 'generated emergency document' : ''));
				return $ret;
		}
	}

	protected function getHTMLFrameset(string $extraHead = '', string $extraUrlParams = ''): string{
		$this->setTreeWidthFromCookie();

		$extraHead = $this->View->getJSTop() .
			//FIXME: throw some of these functions out again and use generic version of main-window functions
			we_html_element::jsScript(JS_DIR . 'modules_tree.js') .
			we_gui_headermenu::css() .
			$extraHead;
		$className = 'we_' . $this->module . '_menu';

		$body = we_html_element::body(['id' => 'weMainBody', 'onload' => 'startTree();'], we_html_element::exIFrame('header', self::getHTMLHeader(
						$className::getMenu())) .
				($this->hasIconbar ? we_html_element::iframe('iconbar', $this->frameset . '&pnt=iconbar' . $extraUrlParams) : '') .
				$this->getHTMLResize($extraUrlParams) .
				we_html_element::iframe('cmd', $this->frameset . '&pnt=cmd' . $extraUrlParams)
		);

		return $this->getHTMLDocument($body, $extraHead);
	}

	/**
	 *
	 * @return string
	 */
	protected static function getHTMLHeader(we_gui_menu $jmenu): string{
		$menu = $jmenu->getHTML();

		return we_html_element::div(['class' => 'menuDiv'], $menu .
				we_html_element::div(['id' => 'moduleMessageConsole'], we_gui_headermenu::createMessageConsole('moduleFrame'))
		);
	}

	private function getHTMLResize(string $extraUrlParams = ''): string{
		$incDecTree = '<div id="baumArrows">
	<div class="baumArrow" id="incBaum" ' . ($this->treeWidth <= 30 ? 'style="background-color: grey"' : '') . ' onclick="top.content.incTree(\'' . $this->module . '\');"><i class="fa fa-plus"></i></div>
	<div class="baumArrow" id="decBaum" ' . ($this->treeWidth <= 30 ? 'style="background-color: grey"' : '') . ' onclick="top.content.decTree(\'' . $this->module . '\');"><i class="fa fa-minus"></i></div>
	<div class="baumArrow" onclick="top.content.toggleTree(\'' . $this->module . '\');"><i id="arrowImg" class="fa fa-lg fa-caret-' . ($this->treeWidth <= 30 ? "right" : "left") . '" ></i></div>
</div>';

		$content = we_html_element::div(['id' => 'moduleContent'], we_html_element::div(['id' => 'lframeDiv', 'style' => 'width: ' . $this->treeWidth . 'px;'], we_html_element::div([
						'id' => 'vtabs'], $incDecTree) .
					$this->getHTMLLeft()
				) .
				we_html_element::div(['id' => 'right', 'style' => 'left: ' . $this->treeWidth . 'px;'], we_html_element::iframe('editor', $this->frameset . '&pnt=editor' . $extraUrlParams)
				)
		);

		return we_html_element::div(['id' => 'resize', 'name' => 'resize', 'class' => ($this->hasIconbar ? 'withIconBar' : ''), 'style' => 'overflow:hidden'], $content);
	}

	protected function getHTMLLeft(): string{
		//we load tree in iFrame, because the complete tree JS is based on document.open() and document.write()
		//it makes not much sense, to rewrite trees before abandoning them anyway
		return we_html_element::div(['id' => 'left', 'name' => 'left'], we_html_element::div(['id' => 'treeheader', 'style' => ($this->showTreeHeader ? 'display:block;' : '')], $this->getHTMLTreeheader()) .
				$this->getHTMLTree() .
				($this->showTreeFooter ? we_html_element::div(['id' => 'treefooter', 'class' => 'editfooter'], $this->getHTMLTreefooter()) :
				''
				)
		);
	}

	protected function getHTMLTree(string $extraHead = ''): string{
		return we_html_element::div([
				'id' => 'tree',
				'class' => ($this->showTreeHeader ? ' withHeader' : '') . ($this->showTreeFooter ? ' withFooter' : '')
				], $extraHead . $this->Tree->getHTMLConstruct()
		);
	}

	protected function getHTMLTreeHeader(): string{
		return '';
		//to be overridden
	}

	protected function getHTMLTreeFooter(): string{
		return '';
		//to be overridden
	}

	protected function getHTMLSearchTreeFooter(): string{
		$hiddens = we_html_element::hiddens([
				'pnt' => 'cmd',
				'cmd' => 'show_search']);

		$table = $hiddens .
			we_html_tools::textInput('keyword', '', 0, ['id' => 'searchInputTree', 'placeholder' => g_l('searchtool', '[placeholder]'), 'onkeyup' => "WE().layout.tree.setSearchButtonIcon(window)"], 'search', '150px') .
			we_html_button::create_button(we_html_button::SEARCH, "javascript:WE().layout.tree.handleSearchMod(window,document.we_form_treefooter.keyword.value);", '', '', false, false, '', false, '', '', 'searchButtonTree');
		//we_html_button::create_button(we_html_button::SEARCH, "javascript:we_cmd('show_search')");

		return we_html_element::div(['id' => 'search', 'style' => 'display:block'], we_html_element::form([
					'name' => 'we_form_treefooter',
					'target' => 'cmd',
					'method' => 'post',
					'onsubmit' => "WE().layout.tree.handleSearchMod(window,document.we_form_treefooter.keyword.value);return false;"
					], $table));
	}

	protected function getHTMLEditor(string $extraUrlParams = '', string $extraHead = ''): string{
		$sid = we_base_request::int('sid', -1);
		$body = we_html_element::body(['class' => 'moduleEditor'], we_html_element::iframe('edheader', $this->frameset . '&pnt=edheader' . ($sid > -1 ? '&sid=' . $sid : '&home=1') . $extraUrlParams, '', 'width: 100%; overflow: hidden', '', 'editorHeader') .
				we_html_element::iframe('edbody', $this->frameset . '&pnt=edbody' . ($sid > -1 ? '&sid=' . $sid : '&home=1') . $extraUrlParams, '', 'border:0;width:100%;height:100%;', '', 'editorBody') .
				we_html_element::iframe(self::FOOTER, $this->frameset . '&pnt=' . self::FOOTER . ($sid > -1 ? '&sid=' . $sid : '&home=1') . $extraUrlParams, '', 'width: 100%; overflow: hidden', '', 'editorButtonFrame')
		);

		return $this->getHTMLDocument($body, $extraHead);
	}

	protected function getHTMLEditorHeader(): string{
		return (we_base_request::bool('home') ?
			$this->getHTMLDocument(we_html_element::body(['class' => 'home'], ''), we_html_element::cssLink(CSS_DIR . 'tools_home.css')) :
			'');
	}

	abstract protected function getHTMLEditorBody(): string;

	/**
	 *
	 * @param array<string,array<string,array{string[],string[]}>> $btn_cmd
	 * @return string
	 */
	protected function getHTMLEditorFooter(array $btn_cmd): string{
		if(we_base_request::bool('home')){
			return $this->getHTMLDocument(we_html_element::body(['style' => 'background-color:#EFF0EF'], ''));
		}

		$buttons = ['main' => '', 'optional' => ''];
		if($btn_cmd){
			foreach($buttons as $key => &$button){
				if(empty($btn_cmd[$key])){
					continue;
				}
				foreach(array_filter($btn_cmd[$key]) as $but => $cur){
					list($right, $cmd) = $cur;
					if(empty($right) || we_base_permission::hasPerm($right)){
						$button .= we_html_button::create_button((string) $but, "javascript:top.content.we_cmd('" . implode("','", $cmd) . "')");
					}
				}
			}
		}
		return $this->getHTMLDocument(we_html_element::body(['id' => 'footerBody'], we_html_element::div(['class' => 'footertable'], we_html_element::div(['class' => 'main'], $buttons['main']) . we_html_element::div([
							'class' => 'optional'], $buttons['optional']))));
	}

	abstract protected function getHTMLCmd(): string;

	function getHTMLSearch(): string{
		return '';
	}

	private function setTreeWidthFromCookie(): void{
		if(!empty($_COOKIE['treewidth_modules']) && ($tw = we_unserialize((string) $_COOKIE['treewidth_modules'], [], true))){
			$this->treeWidth = empty($tw[$this->module]) ? self::TREE_DEFAULT_WIDTH : $tw[$this->module];
		} else {
			$this->treeWidth = self::TREE_DEFAULT_WIDTH;
		}
	}

	protected function formFileChooser(string $IDName = 'ParentID', string $IDValue = '/', string $cmd = '', string $filter = 'all_Types'): string{
		return we_html_tools::formElementTable('', we_html_tools::textInput($IDName, $IDValue, 0, ['readonly' => 'readonly'], 'text', '400px'), '', (we_base_permission::hasPerm([
					we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT]) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server','" . $IDName . "','" . $filter . "',document.we_form.elements['" . $IDName . "'].value,'" . $cmd . "');") : ''));
	}

	/* process vars & commands
	 */

	public function process(): void{
		$this->View->processVariables();
		$this->View->processCommands($this->jsCmd);
	}

	public static function showNotActivatedMsg(string $modName): void{
		$link = '<a href="javascript:top.we_cmd(\'we_gui_openPreferences\');">' . g_l('moduleActivation', '[settings]') . '</a>';
		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('', sprintf(g_l('moduleActivation', '[content]'), $link)))
			->noline());

		echo we_html_tools::getHtmlTop('', '', we_html_element::body(['class' => 'weDialogBody'],
																															 (new we_html_multiIconBox($parts))
					->marginLeft()
					->buttons(we_html_button::create_button(we_html_button::CLOSE,
																						 'javascript:WE().util.dialogTop(window).doClose();'))
					->headline(sprintf(g_l('moduleActivation', '[headline]'), g_l('menu_moduleInformation', '[' . $modName . '][text]')))
					->get()
		));
	}

	public static function showModulesFrameSet(string $mod): void{
		$_REQUEST['mod'] = $mod = ($mod ?: we_base_request::string('mod'));

		//TODO: we should loop through all we_cmd and process them in respective we_module_frames.class only
		$cmd1 = we_base_request::int('we_cmd', -1, 1); //to be used only for IDs or integer constants!
		$sid = in_array($mod, [we_modules_info::CUSTOMER, we_modules_info::THUMB], true) && $cmd1 > -1 ? $cmd1 : we_base_request::int('sid', 0);
		$bid = $mod === we_modules_info::SHOP && $cmd1 > -1 ? $cmd1 : we_base_request::int('bid', 0);

		//FIXME: this is nonsense this is/should be done at we_cmd?!
		$searchInitParams = '';
		if($mod === we_modules_info::SEARCH){
			$searchInitParams = '&' . http_build_query([
					'cmd' => we_base_request::string('cmd', ''),
					'keyword' => we_base_request::string('we_cmd', '', 1),
					'modelid' => we_base_request::int('we_cmd', 0, 3),
					'tab' => we_base_request::int('we_cmd', 0, 4),
			]);
		}

		$we_tabs = new we_gui_tabs();

		foreach(we_modules_info::showModuleInMenu('', 'inModuleMenu') as $menuItem){
			if(we_users_util::canEditModule($menuItem['name'])){
				$we_tabs->addTab($menuItem['text_short'], ($menuItem['icon'] ?: ''), ( $mod === $menuItem['name']), "'" . $menuItem['name'] . "'", ['id' => $menuItem['name']]);
			}
		}
		echo we_html_tools::getHtmlTop('', we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'showMod.js', '', ['id' => 'loadVarShowMod', 'data-moduleData' => setDynamicVar([
					'mod' => $mod,
			])]), we_html_element::body(['id' => 'weMainBody', 'onload' => "weTabs.setFrameSize()", 'onresize' => "weTabs.setFrameSize()"], '<dialog id="alertBox"></dialog>' .
				we_html_element::div(['style' => 'right:0;', 'name' => 'naviDiv', 'id' => 'naviDiv'], we_html_element::div(['id' => 'main'], $we_tabs->getHTML())) .
				we_html_element::iframe('content', WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=' . $mod . $searchInitParams . ($sid ? '&sid=' . $sid : '') . ($bid ? '&bid=' . $bid : ''))
			)
		);
	}

	public static function processModulesCommand(string $mod, string $what): void{
		$mode = we_base_request::int('art', 0);
		$step = we_base_request::int('step', 0);
		$weFrame = null;
		switch($mod){
			case we_modules_info::WORKFLOW:
				$override = ($what === 'log');
			default:
				$protect = (we_users_util::canEditModule($mod) || !empty($override)) ? [] : [false];
				we_html_tools::protect($protect);
		}

		switch($mod){
			case we_modules_info::SEARCH:
				$weFrame = new we_search_frames();
				$weFrame->process();
				break;
			case we_modules_info::SHOP:
				$weFrame = new we_shop_frames();
				$weFrame->processCommands($weFrame->jsCmd);
				break;
			case we_modules_info::CUSTOMER:
				switch($what){
					case 'import':
					case 'export':
						$what = 'frameset';
					case 'eibody':
					case 'eiupload':
						$dialog = new we_dialog_customerEIWizard();
						echo $dialog->getHTML($mode, $step);
						return;
					default:
						$weFrame = new we_customer_frames();
						$weFrame->process();
				}
				break;
			case we_modules_info::USERS:
				$weFrame = new we_users_frames();
				$weFrame->process();
				break;

			case we_modules_info::EXPORT:
				$weFrame = new we_export_frames();
				$weFrame->process();
				break;

			case we_modules_info::GLOSSARY:
				$weFrame = new we_glossary_frames();
				$weFrame->process();
				break;

			case we_modules_info::VOTING:
				$weFrame = new we_voting_frames();
				$weFrame->process();
				break;

			case we_modules_info::DOCTYPE:
				$weFrame = new we_doctype_frames();
				$weFrame->process();
				break;

			case we_modules_info::THUMB:
				$weFrame = new we_thumbnail_frames();
				$weFrame->process();
				break;

			case we_modules_info::CATEGORY:
				$weFrame = new we_category_frames();
				$weFrame->process();
				break;

			case we_modules_info::NAVIGATION:
				switch($what){
					case 'ruleCmd':
					case 'ruleContent':
					case 'ruleFrameset':
						we_html_tools::protect([we_base_perm::NAVIAGTION_EDIT_RULES]);
						$weFrame = new we_navigation_ruleFrames();
						$weFrame->process($weFrame->jsCmd);
						break;
					default:
						$weFrame = new we_navigation_frames();
						$weFrame->process();
						break;
				}
				break;

			case we_modules_info::WORKFLOW:
				$type = we_base_request::string('type', we_workflow_workflow::DOCTYPE_CATEGORY);
				$weFrame = new we_workflow_frames();
				$weFrame->process();
				echo $weFrame->getHTML($what, $mode, 0, $type);
				return;

			case we_modules_info::NEWSLETTER:
				$weFrame = new we_newsletter_frames();
				switch($what){
					case 'edit_file':
						$mode = we_base_request::_(we_base_request::INT, 'art');
						break;
				}

				if(we_base_request::int('inid', -1) < 0){
					switch($what){
						case 'export_csv_mes':
						case 'settings':
						case 'qsend':
						case 'eedit':
						case 'black_list':
						case 'upload_csv':
							break;
						default:
							$weFrame->processVariables();
					}
				}

				switch($what){
					case 'export_csv_mes':
					case 'preview':
					case 'domain_check':
					case 'settings':
					case 'show_log':
					case 'print_lists':
					case 'qsend':
					case 'eedit':
					case 'black_list':
						break;
					default:
						$mode = $mode ?? we_base_request::int('art', 0);
						$weFrame->processCommands($weFrame->jsCmd);
				}

				break;
			default:
				echo 'no module';
				return;
		}

		assert($weFrame instanceof we_modules_frame);
		echo $weFrame->getHTML($what, $mode, $step);
	}

}
