<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: dc4f87e42d5b $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-02-04 21:51 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_navigation_ruleControl{

	/** @var we_navigation_rule */
	private $NavigationRule;

	public function __construct(){
		$this->NavigationRule = new we_navigation_rule();
	}

	function processCommands(we_base_jsCmd $jscmd): void{
		$html = '';

		switch(we_base_request::string('cmd', '')){
			case 'save_navigation_rule' :
				$isNew = $this->NavigationRule->isnew; // navigationID = 0
				$save = true;

				$this->NavigationRule->NavigationName = trim($this->NavigationRule->NavigationName);

				// 2ns check if another element has same name
				$db = new DB_WE();

				if(fB('SELECT 1 FROM ' . NAVIGATION_RULE_TABLE . ' WHERE NavigationName="' . $db->escape($this->NavigationRule->NavigationName) . '" AND ID!=' . intval($this->NavigationRule->ID) . ' LIMIT 1')){
					$jscmd->addMsg(sprintf(g_l('navigation', '[rules][name_exists]'), $this->NavigationRule->NavigationName), we_base_util::WE_MESSAGE_ERROR);
					$save = false;
				}

				if($save && $this->NavigationRule->save()){
					$jscmd->addCmd('setSavedRule', $this->NavigationRule->ID, $this->NavigationRule->NavigationName, $isNew);
					$jscmd->addMsg(sprintf(g_l('navigation', '[rules][saved_successful]'), $this->NavigationRule->NavigationName), we_base_util::WE_MESSAGE_NOTICE);
				}
				break;

			case 'delete_navigation_rule':
				if($this->NavigationRule->delete()){
					$jscmd->addCmd('delRule', $this->NavigationRule->ID, $this->NavigationRule->NavigationName);
				}
				break;

			case 'navigation_edit_rule':
				$this->NavigationRule = new we_navigation_rule();
				$this->NavigationRule->initByID(we_base_request::int('ID'));

				$FolderIDPath = ($this->NavigationRule->FolderID ? id2path($this->NavigationRule->FolderID, FILE_TABLE) : '');
				$ClassIDPath = ($this->NavigationRule->ClassID ? id2path($this->NavigationRule->ClassID, OBJECT_TABLE) : '');
				$NavigationIDPath = htmlspecialchars_decode($this->NavigationRule->NavigationID ? id2path($this->NavigationRule->NavigationID, NAVIGATION_TABLE) : '', ENT_NOQUOTES);

				$jscmd->addCmd('setFormData', [
					'ID' => $this->NavigationRule->ID,
					'NavigationName' => $this->NavigationRule->NavigationName,
					'NavigationID' => $this->NavigationRule->NavigationID,
					'NavigationIDPath' => $NavigationIDPath,
					'FolderID' => $this->NavigationRule->FolderID,
					'FolderIDPath' => $FolderIDPath,
					'SelectionType' => $this->NavigationRule->SelectionType,
					'DoctypeID' => $this->NavigationRule->DoctypeID,
					'ClassID' => $this->NavigationRule->ClassID,
					'ClassIDPath' => $ClassIDPath,
				]);
				// workspaces:
				$workspaceList = [[
				'text' => g_l('navigation', '[no_entry]'),
				'value' => 0
				]];
				$selectWorkspace = '';
				if($this->NavigationRule->ClassID){
					$workspaces = $this->getWorkspacesByClassID($this->NavigationRule->ClassID);

					foreach($workspaces as $key => $value){
						if($key > 0){ // avoid duplicate keys: workspace '/' (0) already set as 'no_entry'
							$workspaceList[] = [
								'text' => $value,
								'value' => $key
							];
						}
					}
					$selectWorkspace = $this->NavigationRule->WorkspaceID;
				}

				$jscmd->addCmd('setWorkspaces', $workspaceList);
				if($selectWorkspace){
					$jscmd->addCmd('selectWorkspace', $this->NavigationRule->WorkspaceID);
				}
				// categories
				if(we_modules_info::isActive(we_modules_info::CATEGORY)){
					$catPaths = ($this->NavigationRule->Categories ? id2pathA(array_filter(we_base_request::filterIntList($this->NavigationRule->Categories)), CATEGORY_TABLE) : [
					]);
					$jscmd->addCmd('setCategories', $catPaths ?: []);
				}
				break;

			case 'get_workspaces':
				if(($classid = we_base_request::int('ClassID'))){
					$workspaces = $this->getWorkspacesByClassID($classid);
					$optionList = [[
					'text' => g_l('navigation', '[no_entry]'),
					'value' => 0
					]];

					foreach($workspaces as $key => $value){
						$optionList[] = [
							'text' => $value,
							'value' => $key
						];
					}
					$jscmd->addCmd('setWorkspaces', $optionList);
				}

				break;
			default:
				return;
		}

		echo we_html_tools::getHtmlTop('', we_html_element::jsScript(WE_JS_MODULES_DIR . 'navigation/navigationRuleControl.js') .
				$jscmd->getCmds(), '<body>' . $html . '</body>');
		exit();
	}

	function getWorkspacesByClassID(int $classId): array{
		$workspaces = [];

		if($classId){
			$workspaces = we_navigation_dynList::getWorkspacesForClass($classId);
			asort($workspaces);
		}
		return $workspaces;
	}

	function processVariables(): void{
		$this->NavigationRule->processVariables();
	}

	/**
	 *
	 * @return we_navigation_rule[]
	 */
	static function getAllNavigationRules(): array{
		//FIXME: add cache?
		$db = new DB_WE();
		$navigationRules = [];

		foreach($db->getAllq('SELECT ID FROM ' . NAVIGATION_RULE_TABLE . ' ORDER BY ID', true) as $id){
			$navigationRules[] = new we_navigation_rule((int) $id);
		}
		return $navigationRules;
	}

}
