<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 428632da509d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-04 00:23 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rebuild_fragment extends we_fragment_base{

	protected function doTask(): void{
		try{
			we_rebuild_base::rebuild($this->data);
		} catch (we_exception_base $e){
			t_e('error rebuilding', $e, $this->data);
		}
	}

	protected function updateTaskPerFragment(): void{
		$type = $this->alldata[$this->currentTask]['ct'];
		switch($type){
			case we_base_ContentTypes::FOLDER_OBJECTFILE:
				$this->taskPerFragment = max(20, $this->taskPerFragment);
				break;
			case we_base_ContentTypes::TEMPLATE:
				$this->taskPerFragment = max(10, $this->taskPerFragment);
				break;
		}
		for($i = 0; $i < $this->taskPerFragment; $i++){
			if(!isset($this->alldata[$i + $this->currentTask]) || $type != $this->alldata[$i + $this->currentTask]['ct']){
				$this->taskPerFragment = max($i, 1);
				return;
			}
		}
	}

	protected function updateProgressBar(): void{
		$this->jsCmd->addCmd('setProgress', [
			'progress' => ((int) ((100 / count($this->alldata)) * ($this->currentTask))),
			'name' => 'pb1',
			'text' => we_base_util::shortenPath($this->data['path'], 60),
			'win' => 'wizbusy'
		]);
	}

	protected function finish(): void{
		//FIXME: this text is not displayed, only logged!
		$responseText = we_base_request::string('responseText', '');
		$this->jsCmd->addMsg($responseText ?: g_l('rebuild', '[finished]'), we_base_util::WE_MESSAGE_NOTICE);
		$this->jsCmd->addCmd('close_dialog');
	}

}
