<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: fd4e08628325 $
 * $Author: Lukas Imhof $
 * $Date: 2020-05-15 22:26 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_search_resultBase{
	/* public properties will be part of json-result */
	/** @var string */
	public $Text = '';

	/** @var string */
	public $Publish_Date = '';

	/** @var string */
	public $Mod_Date = '';

	/** @var bool */
	public $IsUsed = false;

	/** @var int */
	public $TriggerID = 0;

	/** @var string */
	public $media_alt = '';

	/** @var string */
	public $media_title = '';

	/** @var int */
	public $media_filesize = 0;

	/** @var bool */
	public $Show = false;

	/** @var string */
	public $Url = '';

	/** @var int */
	public $WebUserID = 0;

	/** @var string */
	public $elementList = '';

	/** @var string */
	public $elementFields = '';

	/** @var string */
	public $elementGrid = '';

	/** @var string */
	public $elementOverlay = '';

	/* protected properties are needed to create html */

	/** @var string */
	protected $type = '';

	/** @var we_search_search */
	protected $searchclass = null;

	/** @var string */
	protected $Table = '';

	/** @var string */
	protected $docTable = '';

	/** @var int */
	protected $docID = 0;

	/** @var int */
	protected $UID = 0;

	/** @var string */
	protected $Path = '';

	/** @var int */
	protected $ParentID = 0;

	/** @var bool */
	protected $IsFolder = false;

	/** @var int */
	protected $TemplateID = 0;

	/** @var int */
	protected $temp_template_id = 0;

	/** @var string */
	protected $ContentType = '';

	/** @var string */
	protected $SiteTitle = '';

	/** @var string */
	protected $Description = '';

	/** @var int */
	protected $CreatorID = 0;

	/** @var string */
	protected $Creation_Date = '';

	/** @var bool */
	protected $IsPublished = false;

	/** @var bool */
	protected $IsModified = false;

	/** @var string */
	protected $Extension = '';

	/** @var int */
	protected $TableID = 0;

	/** @var array */
	protected $foundInVersions = [];

	/** @var int */
	protected $VersionID = 0;

	/** @var bool */
	protected $VersionActive = false;

	/** @var bool */
	protected $IsProtected = false;

	/** @var string */
	protected $remTable = '';

	/** @var string */
	protected $remCT = '';

	/** @var int */
	protected $remClasse = 0;

	/** @var bool */
	protected $IsSearchable = false;

	/** @var string */
	protected $Workspaces = '';

	/** @var string */
	protected $Language = '';

	/** @var string */
	/*
	  // seems to be unused
	  protected $Nr = '';
	 *
	 */

	/** @var bool */
	/*
	  // seems to be unused
	  protected $IsNr = false;
	 *
	 */

	/** @var int */
	protected static $counter = 0;

	public function __construct(we_search_search $searchclass){
		$this->searchclass = $searchclass;
	}

	/**
	 *
	 * @param string $table
	 */
	public function setTable(string $table): self{
		$this->Table = $table;
		return $this;
	}

	/**
	 *
	 * @param string $docTable
	 */
	public function setDocTable(string $docTable): self{
		$this->docTable = $docTable;
		return $this;
	}

	/**
	 *
	 * @param int $docID
	 */
	public function setDocID(int $docID): self{
		$this->docID = $docID;
		return $this;
	}

	/**
	 *
	 * @param int $uid
	 */
	public function setUID(int $uid): self{
		$this->UID = $uid;
		return $this;
	}

	/**
	 *
	 * @param string $text
	 */
	public function setText(string $text): self{
		$this->Text = $text;
		return $this;
	}

	/**
	 *
	 * @param string $path
	 */
	public function setPath(string $path): self{
		$this->Path = $path;
		return $this;
	}

	/**
	 *
	 * @param int $parentId
	 */
	public function setParentID(int $parentId): self{
		$this->ParentID = $parentId;
		return $this;
	}

	/**
	 *
	 * @param bool $isFolder
	 */
	public function setIsFolder(bool $isFolder): self{
		$this->IsFolder = $isFolder;
		return $this;
	}

	/**
	 *
	 * @param int $templateId
	 */
	public function setTemplateID(int $templateId): self{
		$this->TemplateID = $templateId;
		return $this;
	}

	/**
	 *
	 * @param int $tempTateId
	 */
	public function setTempTemplateId(int $tempTateId): self{
		$this->temp_template_id = $tempTateId;
		return $this;
	}

	/**
	 *
	 * @param string $contentType
	 */
	public function setContentType(string $contentType): self{
		$this->ContentType = $contentType;
		return $this;
	}

	/**
	 *
	 * @param string $siteTitle
	 */
	public function setSiteTitle(string $siteTitle): self{
		$this->SiteTitle = $siteTitle;
		return $this;
	}

	/**
	 *
	 * @param string $description
	 */
	public function setDescription(string $description): self{
		$this->Description = $description;
		return $this;
	}

	/**
	 *
	 * @param int $creatorId
	 */
	public function setCreatorID(int $creatorId): self{
		$this->CreatorID = $creatorId;
		return $this;
	}

	/**
	 *
	 * @param string $creationDate
	 */
	public function setCreationDate(string $creationDate): self{
		$this->Creation_Date = $creationDate;
		return $this;
	}

	/**
	 *
	 * @param string $modDate
	 */
	public function setModDate(string $modDate): self{
		$this->Mod_Date = $modDate;
		return $this;
	}

	/**
	 *
	 * @param string $publishDate
	 */
	public function setPublishDate(string $publishDate): self{
		$this->Publish_Date = $publishDate;
		return $this;
	}

	/**
	 *
	 * @param bool $isPublished
	 */
	public function setIsPublished(bool $isPublished): self{
		$this->IsPublished = $isPublished;
		return $this;
	}

	/**
	 *
	 * @param bool $isModified
	 */
	public function setIsModified(bool $isModified): self{
		$this->IsModified = $isModified;
		return $this;
	}

	/**
	 *
	 * @param string $extension
	 */
	public function setExtension(string $extension): self{
		$this->Extension = $extension;
		return $this;
	}

	/**
	 *
	 * @param int $tableId
	 */
	public function setTableID(int $tableId): self{
		$this->TableID = $tableId;
		return $this;
	}

	/**
	 *
	 * @param string $workspaces
	 */
	public function setWorkspaces(string $workspaces): self{
		$this->Workspaces = $workspaces;
		return $this;
	}

	/**
	 *
	 * @param array $foundInVersion
	 */
	public function setFoundInVersion(array $foundInVersion): self{
		$this->foundInVersions = $foundInVersion;
		return $this;
	}

	/**
	 *
	 * @param int $versionId
	 */
	public function setVersionID(int $versionId): self{
		$this->VersionID = $versionId;
		return $this;
	}

	/**
	 *
	 * @param bool $versionActive
	 */
	public function setVersionActive(bool $versionActive): self{
		$this->VersionActive = $versionActive;
		return $this;
	}

	/**
	 *
	 * @param bool $isProtected
	 */
	public function setIsProtected(bool $isProtected): self{
		$this->IsProtected = $isProtected;
		return $this;
	}

	/**
	 *
	 * @param bool $isUsed
	 */
	public function setIsUsed(bool $isUsed): self{
		$this->IsUsed = $isUsed;
		return $this;
	}

	/**
	 *
	 * @param string $mediaAlt
	 */
	public function setMediaAlt(string $mediaAlt): self{
		$this->media_alt = $mediaAlt;
		return $this;
	}

	/**
	 *
	 * @param string $mediaTitle
	 */
	public function setMediaTitle(string $mediaTitle): self{
		$this->media_title = $mediaTitle;
		return $this;
	}

	/**
	 *
	 * @param int $mediaFilesize
	 */
	public function setMediaFilesize(int $mediaFilesize): self{
		$this->media_filesize = $mediaFilesize;
		return $this;
	}

	/**
	 *
	 * @param string $remTable
	 */
	public function setRemTable(string $remTable): self{
		$this->remTable = $remTable;
		return $this;
	}

	/**
	 *
	 * @param string $remCt
	 */
	public function setRemCT(string $remCt): self{
		$this->remCT = $remCt;
		return $this;
	}

	/**
	 *
	 * @param int $remClass
	 */
	public function setRemClass(int $remClass): self{
		$this->remClasse = $remClass;
		return $this;
	}

	/**
	 *
	 * @param bool $isSearchable
	 */
	public function setIsSearchable(bool $isSearchable): self{
		$this->IsSearchable = $isSearchable;
		return $this;
	}

	/**
	 *
	 * @param bool $show
	 */
	public function setShow(bool $show): self{
		$this->Show = $show;
		return $this;
	}

	/**
	 *
	 * @param string $url
	 */
	public function setUrl(string $url): self{
		$this->Url = $url;
		return $this;
	}

	/**
	 *
	 * @param int $triggerID
	 */
	public function setTriggerID(int $triggerID): self{
		$this->TriggerID = $triggerID;
		return $this;
	}

	/**
	 *
	 * @param string $language
	 */
	public function setLanguage(string $language): self{
		$this->Language = $language;
		return $this;
	}

	/**
	 *
	 * @param int $webUserID
	 */
	public function setWebUserID(int $webUserID): self{
		$this->WebUserID = $webUserID;
		return $this;
	}

	/*
	  // seems to be unused
	  public function setNr(string $nr): self{
	  $this->Nr = $nr;
	  return $this;
	  }
	 *
	 */

	/*
	  // seems to be unused
	  public function setIsNr(bool $isNr): self{
	  $this->IsNr = $isNr;
	  return $this;
	  }
	 *
	 */

	public function processHtml(): void{
		$this->processElementList();
		$this->processElementGrid();
		$this->processElementOverlay();
		self::$counter++;
	}

	protected function processElementList(): void{

	}

	protected function processElementGrid(): void{
		$this->elementGrid = we_html_element::div([
				'class' => 'element grid',
				'onclick' => $this->getJsOpenToEdit(),
				], we_html_element::div(['class' => 'icon',], we_html_element::span(['class' => 'helper']) .
					we_html_element::span(['class' => 'img',
						'onmouseover' => $this->getJsShowOverlay(),
						'onmouseout' => $this->getJsHideOverlay(),
						], $this->getHtmlIconThmubnail(100, 180, we_search_view::VIEW_ICONS)['imageView'])
				) . we_html_element::div([
					'class' => 'text',
					'title' => $this->Text . '(ID: ' . $this->docID . ')',
					], we_base_util::shortenPath($this->Text, 20)
				)
		);
	}

	protected function processElementOverlay(): void{
		$showTemplate = false;
		$showIsProtected = false;
		$showIsUsed = false;
		$showSize = false;
		$showDimension = false;
		$showAltTitle = false;
		$showMeta = false;

		switch($this->ContentType){
			case we_base_ContentTypes::IMAGE:
				$showIsProtected = $this->type === we_search_view::SEARCH_MEDIA;
				$showDimension = true;
				$showAltTitle = $this->type === we_search_view::SEARCH_MEDIA;
			/* fall through */
			case we_base_ContentTypes::AUDIO:
			case we_base_ContentTypes::VIDEO:
			case we_base_ContentTypes::FLASH:
			case we_base_ContentTypes::APPLICATION:
			case we_base_ContentTypes::PDF:
				$showIsUsed = $this->type === we_search_view::SEARCH_MEDIA;
				$showSize = true;
				$showMeta = true;
				break;
			case we_base_ContentTypes::WEDOCUMENT:
				$showTemplate = true;
				break;
		}

		$gridhtmlIcon = $this->getHtmlIconThmubnail(140, 180, we_search_view::VIEW_ICONS);

		$divIcon = we_html_element::div(['style' => 'padding:15px;display: inline-block;'], $gridhtmlIcon['imageView']);
		$divText = we_html_element::div(['style' => 'font-weight:bold;font-size:11px;margin-left:150px;width:200px;padding:0 12px 12px 6px;display:inline-table;'], $this->Text);

		$tableData = (new we_html_table(['style' => 'font-size:10px;margin-left:150px;width:200px;display:inline-table;'], 0, 2))
			->addRow(new we_html_tr([
					new we_html_td(g_l('searchtool', '[idDiv]') . ': '),
					new we_html_td((string) $this->docID),
				]))
			->addRow(new we_html_tr([
				new we_html_td(g_l('searchtool', '[dateityp]') . ': '),
				new we_html_td((we_base_util::shortenPath(g_l('contentTypes', '[' . $this->ContentType . ']'), 22))),
		]));

		if($showTemplate){
			$tableData->addRow(new we_html_tr([
					new we_html_td(g_l('searchtool', '[template]') . ': '),
					new we_html_td((string) $this->TemplateID),
			]));
		}
		if($showSize){
			$sizeText = we_base_file::getHumanFileSize((is_file(($file = WEBEDITION_PATH . '../' . $this->Path)) && ($size = filesize($file))) ? $size : 0);
			$tableData->addRow(new we_html_tr([
					new we_html_td(g_l('searchtool', '[groesse]') . ': '),
					new we_html_td($sizeText),
			]));
		}
		if($showDimension){
			$tableData->addRow(new we_html_tr([
					new we_html_td(g_l('searchtool', '[aufloesung]') . ': '),
					new we_html_td($gridhtmlIcon['sizeX'] . ' x ' . $gridhtmlIcon['sizeY']),
			]));
		}
		if($showAltTitle){
			$tableData->addRow(new we_html_tr([
						new we_html_td(g_l('weClass', '[collection][attr_alt]') . ': '),
						new we_html_td($this->media_alt),
					]))
				->addRow(new we_html_tr([
						new we_html_td(g_l('weClass', '[collection][attr_title]') . ': '),
						new we_html_td($this->media_title),
			]));
		}
		if($showIsProtected){
			$tableData->addRow(new we_html_tr([
					new we_html_td(g_l('searchtool', '[protection]') . ': '),
					new we_html_td(($this->IsProtected ? g_l('global', '[true]') : g_l('global', '[false]'))),
			]));
		}
		if($showIsUsed){
			$tableData->addRow(new we_html_tr([
					new we_html_td(g_l('weClass', '[isUsed]') . ': '),
					new we_html_td(($this->IsUsed ? g_l('global', '[true]') : g_l('global', '[false]'))),
			]));
		}

		$tableData->addRow(new we_html_tr([
					new we_html_td(g_l('searchtool', '[creator]') . ': '),
					new we_html_td($this->CreatorID ? id2path($this->CreatorID, USER_TABLE) : g_l('searchtool', '[nobody]')),
				]))
			->addRow(new we_html_tr([
					new we_html_td(g_l('searchtool', '[created]') . ': '),
					new we_html_td((new DateTime($this->Creation_Date))->format(g_l('searchtool', '[date_format]'))),
				]))
			->addRow(new we_html_tr([
					new we_html_td(g_l('searchtool', '[modified]') . ': '),
					new we_html_td((new DateTime($this->Mod_Date))->format(g_l('searchtool', '[date_format]'))),
		]));

		$divDescription = we_html_element::div(['style' => 'padding:0 0 6px 15px;width:360px;'],
																				 ($this->Description ? '<span style="font-size:10px;">' . g_l('searchtool', '[beschreibung]') . ': ' . we_base_util::shortenPath($this->Description, 150) . '</span>' : '')
		);

		$divMeta = '';
		if($showMeta){
			$metafields = [];
			$divMetaTitle = '';
			$divMetaContent = '';
			$fieldcount = 6;
			foreach(array_keys(we_metadata_metaData::getDefinedMetaDataFields()) as $tagName){
				if($fieldcount-- == 0){
					break;
				}
				if(we_exim_contentProvider::isBinary((int) $this->docID)){
					$metafields[$tagName] = we_base_util::shortenPath(fS('SELECT c.Dat FROM ' . FILE_TABLE . ' f  LEFT JOIN ' . CONTENT_TABLE . ' c ON (c.DID=f.ID) WHERE c.DID=' . intval($this->docID) . ' AND c.nHash=x\'' . md5($tagName) . '\' AND c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '"'), 45);
				}
			}
			if($metafields){
				$divMetaTitle = we_html_element::div(['style' => 'height:20px;overflow:hidden;border-top:1px solid #DDDDDD;margin-top:15px;padding:5px 0 0 15px;'], g_l('searchtool', '[metafelder]') . ':');
				$tableMeta = (new we_html_table([], 0, 2));
				foreach($metafields as $k => $v){
					$tableMeta->addRow(new we_html_tr([
							new we_html_td(we_base_util::shortenPath($k, 90) . ': '),
							new we_html_td(we_base_util::shortenPath($v, 90)),
					]));
				}
				$divMetaContent = we_html_element::div(['style' => 'background-color:#FFF;margin:10px 10px 10px 15px;font-size:10px;'], $tableMeta->getHtml());
			}
			$divMeta = we_html_element::div([], $divMetaTitle . $divMetaContent);
		}

		$divHeadline = we_html_element::div(['style' => 'margin-left:18px;margin-right:18px;height:22px;padding-top:3px;',
				'class' => 'weSearchOverlayHeadlineDivs'
				], $this->SiteTitle);

		$this->elementOverlay = we_html_element::div([
				'class' => 'largeicons middlefont',
				'style' => 'position:absolute;left:-9999px;width:400px;text-align:left;z-index:10000;visibility:visible;border:1px solid #bab9ba; border-radius:20px;background-color:#EDEDED;',
				'id' => 'ImgDetails_' . self::$counter . '_' . $this->type
				], $divHeadline . we_html_element::div(['style' => 'width:100%;border-top:1px solid #DDDDDD;'], $divIcon . $divText . $tableData->getHtml() . $divDescription . $divMeta)
		);
	}

	protected function getHtmlIconThmubnail(int $smallSize = 64, int $bigSize = 140, string $view = we_search_view::VIEW_LIST): array{
		$urlPopup = $url = '';
		if(($this->type === we_search_view::SEARCH_MEDIA || $view === we_search_view::VIEW_ICONS) && $this->ContentType == we_base_ContentTypes::IMAGE &&
					file_exists(WEBEDITION_PATH . '../' . $this->Path) &&
					($imagesize = getimagesize(WEBEDITION_PATH . '../' . $this->Path))){

				$url = WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_base_showThumbnail&id=' . $this->docID . '&path=' . urlencode($this->Path) . '&extension=' . $this->Extension;
				$imageView = '<img src="' . $url . '&size[width]=' . $smallSize . '" style="max-width:' . $smallSize . 'px;max-height:' . $smallSize . '"/></a>';
				$imageViewPopup = '<img src="' . $url . '&size[width]=' . $bigSize . '" style="max-width:' . $smallSize . 'px;max-height:' . $smallSize . '"/></a>';
		} else {
			$imagesize = [0, 0];
			$imageView = $imageViewPopup = '<span class="resultIcon" data-contenttype="' . $this->ContentType . '" data-extension="' . $this->Extension . '"></span>';
		}

		return ['imageView' => $imageView, 'imageViewPopup' => $imageViewPopup, 'sizeX' => $imagesize[0], 'sizeY' => $imagesize[1], 'url' => $url, 'urlPopup' => $urlPopup];
	}

	protected function getJsOpenToEdit(): string{
		return "WE().layout.openToEdit('" . addTblPrefix($this->docTable) . "'," . $this->docID . ",'" . $this->ContentType . "');";
	}

	protected function getJsShowOverlay(): string{
		return "weSearch.showElementOverlay('ImgDetails_" . self::$counter . "_" . $this->type . "',1)";
	}

	protected function getJsHideOverlay(): string{
		return "weSearch.hideElementOverlay('ImgDetails_" . self::$counter . "_" . $this->type . "')";
	}

}
