<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6770d7cd2c78 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-02 19:13 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_shop_vats{
	/** @var array<int,we_shop_vat> */
	private static $allVats = [];

	/** @var array<int,we_shop_vat> */
	private static $vatById = [];

	/** @var we_shop_vat */
	private static $standardShopVat = null;

	public static function getAllShopVATs(): array{
		if(empty(self::$allVats)){
			DB_WE::$inst->query('SELECT id,text,vat,standard,territory,textProvince, categories FROM ' . WE_SHOP_VAT_TABLE . ' ORDER BY vat');
			self::$allVats = [];

			while(DB_WE::$inst->next_record()){
				self::$allVats[DB_WE::$inst->f('id')] = new we_shop_vat((int) DB_WE::$inst->f('id'), DB_WE::$inst->f('text'), (float) DB_WE::$inst->f('vat'), (DB_WE::$inst->f('standard') ? true : false), DB_WE::$inst->f('territory'), DB_WE::$inst->f('textProvince'), DB_WE::$inst->f('categories'));
			}
		}
		sort(self::$allVats);

		return self::$allVats;
	}

	/**
	 *
	 * @param int $id
	 * @return we_shop_vat
	 * @throws we_exception_shop
	 */
	public static function getShopVATById(int $id): we_shop_vat{
		if(!isset(self::$vatById[$id])){
			self::$vatById[$id] = we_shop_vat::getVatById($id);
		}

		return self::$vatById[$id];
	}

	public static function getVatRateForSite(int $id = 0, float $standard = 0): float{
		if($id){
			try{
				$weShopVat = we_shop_vats::getShopVATById($id);
				return $weShopVat->vat;
			} catch (we_exception_shop $e){

			}
		}

		try{
			$weShopVat = we_shop_vats::getStandardShopVat();
			return $weShopVat->vat;
		} catch (we_exception_shop $e){
			return $standard;
		}
	}

	public static function getStandardShopVat(): we_shop_vat{
		if(!self::$standardShopVat){
			$data = getHash('SELECT id,text,vat,standard,territory,textProvince,categories FROM ' . WE_SHOP_VAT_TABLE . ' WHERE standard=1');

			if(!$data){
				throw new we_exception_shop('no standard shop vat found');
			}
			self::$standardShopVat = new we_shop_vat($data['id'], $data['text'], $data['vat'], true, $data['territory'], $data['textProvince'], $data['categories']);
		}

		return self::$standardShopVat;
	}

	public static function saveWeShopVAT(we_shop_vat $weShopVat): int{
		// 1st - change standard for every entry
		if($weShopVat->standard){
			DB_WE::$inst->query('UPDATE ' . WE_SHOP_VAT_TABLE . ' SET standard=0');
		}

		return $weShopVat->save();
	}

	public static function deleteVatById(int $id): bool{
		$vat = new we_shop_vat($id);
		return $vat->delete();
	}

}
