<?php
/* webEdition CMS
 *
 * $Rev: 44db4d7c3342 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 18:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_cache extends we_weTag_tag{
	private static $debugStart = 0.0;

	public static function parse(array $attribs, string $content): string{
		self::attributeMissing($attribs, ['id' => false], __CLASS__);

		$hash = base64_encode(md5($content, true));
		return '<?php if(' . we_tag_tagParser::printTag('cache', ['_hash' => $hash] + $attribs) . '){;?>' .
			$content .
			'<?php ' . we_tag_tagParser::printTag('cache', ['_end' => true, '_hash' => $hash] + $attribs) . ';}?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally
	 * @return bool output for if-tag
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		if(we_base_WE::$editmode){
			return true;
		}
		$id = self::getAttribute('id', $attribs, '', we_base_request::STRING);
		$chash = self::getAttribute('_hash', $attribs, '', we_base_request::STRING);
		$cust = self::getAttribute('customer', $attribs, true, we_base_request::BOOL);
		$ttl = self::getAttribute('ttl', $attribs, 60, we_base_request::INT);
		$debug = self::getAttribute('debug', $attribs, false, we_base_request::BOOL);
		$end = self::getAttribute('_end', $attribs, false, we_base_request::BOOL);
		$key = ($cust && !empty($_SESSION['webuser']['ID']) ? $_SESSION['webuser']['ID'] . '_' . base64_encode(md5(serialize($_SESSION['webuser']), true)) : '0') . '_' . $chash . '_' . $id;

		//end caching, save cached data
		if($end){
			$data = ob_get_clean();
			$hash = base64_encode(md5((string) $data, true));
			we_cache_we::save($key, $hash, $ttl);
			//set new expiration time, if key found, else add the new key
			if(!we_cache_we::touch($hash, $ttl)){
				we_cache_we::save($hash, $data, $ttl);
			}
			echo
			($debug ? '<!-- Cache: Time to Cache Entry (' . $id . ', key:' . $key . ', hash:' . $hash . '): ' . (microtime(true) - self::$debugStart) . ' -->' : '') .
			$data;
			return true;
		}

		//check if cache has entry
		$fetch = ($debug ? microtime(true) : 0);
		try{
			$hash = we_cache_we::load($key);
			$data = we_cache_we::load($hash);
			if($debug){
				echo '<!-- Cache: Fetchtime: ' . (microtime(true) - $fetch) . ' -->';
			}
			if(!$debug){
				echo $data;
				return false;
			}
		} catch (we_exception_cache $e){

		}
		if($debug){
			self::$debugStart = microtime(true);
		}
		ob_start();
		return true;
	}

}
