<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 44db4d7c3342 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 18:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_field extends we_weTag_tag{

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		if(empty(we_tag_listview::$current)){
			return self::parseError(g_l('parser', '[field_not_in_lv]'), __CLASS__);
		}
		//show field in case of e.g. listdir
		if(we_tag_listview::$current instanceof we_listview_pseudo){
			$pseudo = we_tag_listview::$current;
			'@phan-var we_listview_pseudo $pseudo';
			return $pseudo->field;
		}
		self::$doc = we_base_WE::getWeDoc();

		$lvname = we_tag_listview::$current->name;

		$orgName = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
		'@phan-var string $orgName';
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		'@phan-var string $name';
		$href = self::getAttribute('href', $attribs, '', we_base_request::URL);
		$type = (string) self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$alt = self::getAttribute('alt', $attribs, '', we_base_request::STRING);
		$tid = self::getAttribute('tid', $attribs, 0, we_base_request::INT);
		$hyperlink = self::getAttribute('hyperlink', $attribs, false, we_base_request::BOOL);
		$id = self::getAttribute('id', $attribs, '', we_base_request::STRING);
		$striphtml = (bool) self::getAttribute('striphtml', $attribs, false, we_base_request::BOOL);
		$only = self::getAttribute('only', $attribs, '', we_base_request::STRING);
		$usekey = self::getAttribute('usekey', $attribs, false, we_base_request::BOOL);
		$triggerid = self::getAttribute('triggerid', $attribs, 0, we_base_request::INT);
		$showpath = (bool) self::getAttribute('showpath', $attribs, false, we_base_request::BOOL);
		$rootdir = (string) self::getAttribute('rootdir', $attribs, '', we_base_request::FILE);
		$field = (string) self::getAttribute('field', $attribs, '', we_base_request::STRING);
		$max = self::getAttribute('max', $attribs, 0, we_base_request::INT);
		$value = self::getAttribute('value', $attribs, '', we_base_request::RAW);

		$listviewname = self::getAttribute('listviewname', $attribs, $lvname, we_base_request::STRING);
		$src = self::getAttribute('src', $attribs, '', we_base_request::STRING);
		$target = self::getAttribute('target', $attribs, '', we_base_request::STRING);
		$autobr = self::getAttribute('autobr', $attribs, '', we_base_request::BOOL);

		$attribs['title'] = self::getAttribute('title', $attribs, '', we_base_request::STRING);
		$attribs['format'] = self::getAttribute('format', $attribs, '', we_base_request::STRING);
		$attribs['class'] = self::getAttribute('class', $attribs, '', we_base_request::STRING);
		$classid = (int) self::getAttribute('classid', $attribs, 0, we_base_request::INT);
		$seeMode = self::getAttribute('seeMode', $attribs, true, we_base_request::BOOL);
		$customerid = self::getAttribute('customerid', $attribs, 0, we_base_request::INT);
		$attribs['country'] = self::getAttribute('country', $attribs, '', we_base_request::STRING);
		//zusatz typen für verchiedene field-Typen
		$thumbnail = self::getAttribute('thumbnail', $attribs, '', we_base_request::STRING);
		$thumbnailid = self::getAttribute('thumbnailid', $attribs, 0, we_base_request::INT);
// Image
		$attribs['classes'] = self::getAttribute('classes', $attribs, '', we_base_request::STRING);
		//Wwysiwyg
		$attribs['commands'] = self::getAttribute('commands', $attribs, '', we_base_request::STRING);
		$alt = (strtolower($alt) === strtolower(we_listview_base::PATH_FIELD) ? we_listview_base::PATH_FIELD : (strtolower($alt) === strtolower(we_listview_base::TEXT_FIELD) ? we_listview_base::TEXT_FIELD : $alt));
		$name = (string) (strtolower($orgName) === strtolower(we_listview_base::PATH_FIELD) ? we_listview_base::PATH_FIELD : (strtolower($orgName) === strtolower(we_listview_base::TEXT_FIELD) ? we_listview_base::TEXT_FIELD : $name));

		$classid && $attribs['classid'] = $classid;
		$seeMode && $attribs['seeMode'] = $seeMode;
		$customerid && $attribs['customerid'] = $customerid;
		$thumbnail && $attribs['thumbnail'] = $thumbnail;
		$thumbnailid && $attribs['thumbnailid'] = $thumbnailid;

		$href && $attribs['href'] = $href;
		$type && $attribs['type'] = $type;
		$alt && $attribs['alt'] = $alt;
		$tid && $attribs['tid'] = $tid;
		$id && $attribs['id'] = $id;
		$only && $attribs['only'] = $only;
		$usekey && $attribs['usekey'] = $usekey;
		$triggerid && $attribs['triggerid'] = $triggerid;
		$showpath && $attribs['showpath'] = $showpath;
		$rootdir && $attribs['rootdir'] = $rootdir;
		$value && $attribs['value'] = $value;
		$target && $attribs['target'] = $target;
		$src && $attribs['src'] = $src;
		$field && $attribs['field'] = $field;
		unset($attribs['winprops']);

		$out = '';

		//listview of documents, document with a block. Try to access by blockname.
		$name = ($orgName !== $name && we_tag_listview::$current instanceof we_listview_document && we_tag_listview::$current->f($name) ? $name : $orgName);

		$classid = we_tag_listview::$current->getTableID() ?: $classid;

		$isImageDoc = (we_tag_listview::$current->f(we_listview_base::CONTENTTYPE_FIELD) == we_base_ContentTypes::IMAGE);
		$cal = we_tag_listview::$current->getCalendarStruct();
		$isCalendar = $cal && (!empty($cal['calendar']) && we_tag_listview::$current->isCalendarField($type));

		if((!we_tag_listview::$current->f(we_listview_base::ID_FIELD) && !$isCalendar)){
			return '';
		}

		switch($type){
			case 'binary':
				$t = self::$doc->getFieldByVal(we_tag_listview::$current->f($name), $type, $attribs, false, self::$doc->ParentID, self::$doc->Path, $classid, 'listview');
				switch($only){
					case '':
					case 'name':
						$out = (string) $t[0];
						break;
					case 'path':
						$out = (string) $t[1];
						break;
					case 'parentpath':
						$out = (string) $t[2];
						break;
					case 'filename':
						$out = (string) $t[3];
						break;
					case 'extension':
						$out = (string) $t[4];
						break;
					case 'filesize':
						$out = (string) $t[5];
						break;
				}
				$href = ($href ?: $t[1]);
				break;
			case 'link' :
				if(we_tag_listview::$current){
					$out = (string) self::$doc->getFieldByVal(we_tag_listview::$current->f($name), 'link', $attribs, false, self::$doc->ParentID, self::$doc->Path, $classid, 'listview');
					$href = ($href ?: $out);
					break;
				}
			case 'img' :
				if($src){
					$imgAtts = array_merge([
						'alt' => ' ', //  alt must be set
						'src' => $src,
						], array_intersect_key($attribs, array_flip(['alt', 'width', 'height', 'border', 'hspace', 'align', 'vspace']))); //  use some atts form attribs array

					$out = self::htmlTag('img', $imgAtts);
					if(!$out){
						//we have no image, so we don't generate an link
						return '';
					}
					break;
				}
			//intentionally no break
			case 'int' :
			case 'date' :
			case 'float' :
			case 'checkbox' :
			case 'collection':
				$idd = ($isImageDoc && $type === 'img' ) ? we_tag_listview::$current->f(we_listview_base::ID_FIELD) : we_tag_listview::$current->f($name);
				$out = ($idd == 0 ? '' : (string) self::$doc->getFieldByVal($idd, $type, $attribs, false, self::$doc->ParentID, self::$doc->Path, $classid, 'listview'));
				if($type === 'img' && empty($out)){
					return '';
				}
				break;
			case 'day' :
			case 'dayname' :
			case 'dayname_long' :
			case 'dayname_short' :
			case 'month' :
			case 'monthname' :
			case 'monthname_long' :
			case 'monthname_short' :
			case 'year' :
			case 'hour' :
			case 'week' :
				list('code' => $langcode) = self::getOutLang($attribs);
				$out = we_tag_listview::$current->getCalendarField($type, $langcode);
				break;

			case 'multiobject':
				$temp = we_unserialize(we_tag_listview::$current->f($name));
				$out = is_array($temp) ? (!empty($temp['objects']) ? implode(',', $temp['objects']) : implode(',', $temp)) : '';
				break;
			case 'country' :
				list('code' => $langcode) = self::getOutLang($attribs);
				$out = (WE_COUNTRIES_DEFAULT != '' && we_tag_listview::$current->f($name) === '--' ?
					WE_COUNTRIES_DEFAULT :
					we_base_country::getTranslation(we_tag_listview::$current->f($name), we_base_country::TERRITORY, $langcode)
					);
				break;
			case 'language' :
				list('code' => $langcode) = self::getOutLang($attribs);
				$out = we_base_country::getTranslation(we_tag_listview::$current->f($name), we_base_country::LANGUAGE, $langcode);
				break;
			case 'shopVat' :
				$out = self::getShopVatField($type, $attribs, $classid);
				break;
			case 'shopCategory' :
				$out = self::getShopCat($field, $showpath, $rootdir);
				break;
			case 'href' ://#6329: fixed for lv type=document. check later for other types! #6421: field type=href in we:block
				if(we_tag_listview::$current){
					$hrefArr = we_tag_listview::$current->getHrefArray($name);
					$out = $hrefArr ? we_document_base::getHrefByArray($hrefArr) : '';
					$path_parts = pathinfo($out);
					if(!empty(we_tag_listview::$current->hidedirindex) && seoIndexHide($path_parts['basename'])){
						$out = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/';
					}

					break;
				}
			default: // FIXME: treat type="select" as separate case, and clean up the mess with all this little fixes
				if($orgName === we_listview_base::PATH_FIELD && $triggerid && in_array(get_class(we_tag_listview::$current), [//@phan-suppress-current-line PhanPossiblyNullTypeArgumentInternal
						'we_listview_search', 'we_listview_object', 'we_listview_multiobject'], true)){
					$triggerpath = id2path($triggerid);
					$triggerpath_parts = pathinfo($triggerpath);
					$normVal = ($triggerpath_parts['dirname'] != '/' ? $triggerpath_parts['dirname'] : '') . '/' .
						(!empty(we_tag_listview::$current->hidedirindex) && seoIndexHide($triggerpath_parts['basename']) ?
						'' : $triggerpath_parts['filename'] . '/' ) .
						we_tag_listview::$current->f(we_listview_base::URL_FIELD);
				} else {
					$testtype = ($type === 'select' && $usekey) ? 'text' : $type;
					if(we_tag_listview::$current instanceof we_listview_objectBase){
						$attribs['name'] = $orgName;
					}
					$normVal = (string) self::$doc->getFieldByVal(we_tag_listview::$current->f($name), $testtype, $attribs, false, self::$doc->ParentID, self::$doc->Path, $classid, 'listview');
					$normVal = we_wysiwyg_editor::parseGalleryTags($normVal);

					if($orgName === we_listview_base::PATH_FIELD){
						$path_parts = pathinfo($normVal);
						if(!we_base_WE::$mainDoc['doc']->InWebEdition && !empty(we_tag_listview::$current->hidedirindex) && seoIndexHide($path_parts['basename'])){
							$normVal = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/';
						}
					}
				}

				if($name && $name != we_listview_base::HREF_FIELD){
					if($normVal === ''){
						$altVal = (string) self::$doc->getFieldByVal(we_tag_listview::$current->f((string) $alt), $type, $attribs, false, self::$doc->ParentID, self::$doc->Path, $classid, 'listview');
						if($altVal === ''){
							return '';
						}

						if($alt === we_listview_base::PATH_FIELD){
							$path_parts = pathinfo($altVal);
							if(!empty(we_tag_listview::$current->hidedirindex) && seoIndexHide($path_parts['basename'])){
								$altVal = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/';
							}
						}
						$normVal = $altVal;
					}
					$out = self::cutText(($striphtml ? strip_tags($normVal) : $normVal), $max, $striphtml);
				} elseif($value){
					$out = ($striphtml ? strip_tags($value) : $value);
				} else if($striphtml){
					$out = strip_tags($out);
				}

				$out = ($autobr ? nl2br($out) : $out);
		}

		if($hyperlink || $name === we_listview_base::HREF_FIELD){
			$linkAttribs = array_intersect_key($attribs, array_flip(['xml', 'class', 'style']));
			if($target){ //  save atts in array
				$linkAttribs['target'] = $target;
			}

			foreach($attribs as $key => $val){
				'@phan-var string $key';
				if(strpos($key, 'pass_') === 0){
					$linkAttribs[$key] = $val;
				}
			}

			if($href){
				$linkAttribs['href'] = $href;
				return self::htmlTag('a', $linkAttribs, $out, true);
			}
			if($id){
				if($isCalendar){
					$cal = we_tag_listview::$current->getCalendarStruct();
					if(!empty($cal['storage'])){

						foreach($cal['storage'] as $date){
							if((($cal['calendarCount'] > 0 || ($cal['calendar'] === 'day' && $cal['calendarCount'] >= 0)) && $cal['calendarCount'] <= $cal['numofentries']) && (intval($date) >= intval($cal['start_date']) && intval($date) <= intval($cal['end_date']))){
								$show = ($cal['calendar'] === 'year' ? 'month' : 'day');
								$linkAttribs['href'] = id2path($id) . '?' . http_build_query(array_merge(
											we_tag_listview::$current->getListviewRequest($listviewname), array_filter([
										'we_lv_calendar_' . $listviewname => $show,
										'we_lv_datefield_' . $listviewname => $cal['datefield'],
										'we_lv_date_' . $listviewname => ($cal['date'] >= 0 ? date('Y-m-d', $cal['date']) : '')
								])));

								return self::htmlTag('a', $linkAttribs, $out, true);
							}
						}
					}
				} elseif($isImageDoc){
					$linkAttribs['href'] = id2path($id) . '?' . http_build_query(array_merge(
								we_tag_listview::$current->getListviewRequest($listviewname), [
							'we_lv_start_' . $lvname => ((we_tag_listview::$current->count + we_tag_listview::$current->start) - 1),
							'we_lv_pend_' . $lvname => (we_tag_listview::$current->start + we_tag_listview::$current->anz),
							'we_lv_pstart_' . $lvname => (we_tag_listview::$current->start)
					]));

					return self::htmlTag('a', $linkAttribs, $out, true);
				}
			}

			$showlink = false;
			$tailOwnId = '';
			switch(get_class(we_tag_listview::$current)){//@phan-suppress-current-line PhanPossiblyNullTypeArgumentInternal
				case 'we_listview_document':
				case 'we_listview_collectionItems':
					$od = we_tag_listview::$current;
					'@phan-var we_listview_document $od';
					$triggerid = (int) ($triggerid ?: $od->triggerID);
					$tailOwnId = '?we_documentID=' . $od->f(we_listview_base::ID_FIELD);
					break;
				case 'we_listview_search':
					if(we_tag_listview::$current->f('ClassID')){
						return self::getSearchObject($tid, $name, $out, $linkAttribs);
					}
				case '':
				case 'we_listview_variants':
				case 'we_listview_shopItems':
				case 'we_customertag':
				case 'we_listview_customer':
					$showlink = true;
					break;
				case 'we_listview_object':
					$ol = we_tag_listview::$current;
					'@phan-var we_listview_object $ol';
					$triggerid = $ol->useSeoUrls() ? (int) $ol->f('we_triggerid') : ($triggerid ?: (int) $ol->triggerID);
					$showlink = $tid || $triggerid || $ol->f(we_listview_base::PROPPREFIX . 'templates') || $ol->docID;
					$tailOwnId = '?' . WEOBJ_PARAM . '=' . $ol->f('OF_ID');
					break;
				case 'we_listview_multiobject':
					$om = we_tag_listview::$current;
					'@phan-var we_listview_multiobject $om';
					$showlink = $om->f(we_listview_base::PROPPREFIX . 'templates') || $om->docID;
					break;
				case 'we_listview_shopOrder': //listview type="order"
					$showlink = !empty($triggerid);
					break;
				case 'we_listview_category':
					if(we_weTag_util::tag('ifHasChildren')){
						return self::getCategoryData($attribs, $linkAttribs, $name, $out);
					}
				default:
					return (string) $out;
			}

			if(!$showlink){
				return (string) $out;
			}

			$tail = ($tid && (we_tag_listview::$current instanceof we_listview_object) ? '&amp;we_objectTID=' . $tid : '');

			if(self::$doc instanceof we_document_triggerDocument && (self::$doc->InWebEdition)){
				$linkAttribs['href'] = we_tag_listview::$current->f(we_listview_base::LASTPATH_FIELD) . $tail;
			} else {
				$path_parts = pathinfo(we_tag_listview::$current->f(we_listview_base::PATH_FIELD));
				if($triggerid){
					$triggerpath = id2path($triggerid);
					$triggerpath_parts = pathinfo($triggerpath);

					$linkAttribs['href'] = we_tag_listview::$current->useSeoUrls() ? // objectseourls=true
						(empty($triggerpath_parts['dirname']) ? '' : rtrim($triggerpath_parts['dirname'], '/') ) . '/' .
						((!we_base_WE::$mainDoc['doc']->InWebEdition && !empty(we_tag_listview::$current->hidedirindex) && seoIndexHide($triggerpath_parts['basename'])) ? //hidedirindex of triggerID
						we_tag_listview::$current->f(we_listview_base::URL_FIELD) . $tail : //Fix #8708 do not hidedirindex of triggerID
						$triggerpath_parts['filename'] . '/' . we_tag_listview::$current->f(we_listview_base::URL_FIELD) . $tail
						) : //objectseourls=false or not set
						$triggerpath . $tailOwnId . $tail;

					/* End Fix '7771 */
				} else {
					$linkAttribs['href'] = (!empty(we_tag_listview::$current->hidedirindex) && seoIndexHide($path_parts['basename']) ?
						($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' :
						we_tag_listview::$current->f(we_listview_base::PATH_FIELD) . $tail
						);
				}
			}

			return ($name === we_listview_base::HREF_FIELD ? //  return href for this object
				$linkAttribs['href'] :
				self::htmlTag('a', $linkAttribs, $out, true));
		}

		return (string) $out;
	}

	protected static function getShopVatField(string $type, array $attribs, int $classid): string{
		if(!we_modules_info::isActive(we_modules_info::SHOP)){
			return '';
		}
		if(!we_shop_category::isCategoryMode()){
			$normVal = (int) self::$doc->getFieldByVal(we_tag_listview::$current->f(we_shop_shop::VAT_FIELD_NAME), $type, $attribs, false, self::$doc->ParentID, self::$doc->Path, $classid, 'listview'); // war '->getElement', getElemet gibt es aber nicht in LVs, gefunden bei #4648
			return (string) we_shop_vats::getVatRateForSite($normVal);
		}
		$shopVatAttribs = array_merge($attribs, [
			'shopcategoryid' => we_tag_listview::$current->f(we_shop_shop::CATEGORY_FIELD_NAME),
			'wedoccategories' => we_tag_listview::$current->f(we_listview_base::CATEGORY_FIELD)
		]);
		unset($shopVatAttribs['type']);

		return (string) we_weTag_util::tag('shopVat', $shopVatAttribs);
	}

	protected static function getShopCat(string $field, bool $showpath, string $rootdir): string{
		if(!we_modules_info::isActive(we_modules_info::SHOP) && we_tag_listview::$current){
			return '';
		}
		$id = (int) we_tag_listview::$current->f(we_shop_shop::CATEGORY_FIELD_NAME);
		$wedocCategory = we_base_request::filterIntList(we_tag_listview::$current->f(we_listview_base::CATEGORY_FIELD));
		try{
			return (string) we_shop_category::getShopCatFieldByID($id, $wedocCategory, $field, $showpath, $rootdir, true);
		} catch (we_exception_shop $e){
			t_e($e);
		}
		return '';
	}

	protected static function getCategoryData(array $attribs, array $linkAttribs, string $name, string $out): string{
		$parentidname = self::getAttribute('parentidname', $attribs, 'we_parentid', we_base_request::STRING);
		$linkAttribs['href'] = we_http_util::getScriptName() . '?' . $parentidname . '=' . we_tag_listview::$current->f('ID');

		return ($name === we_listview_base::HREF_FIELD ?
			$linkAttribs['href'] :
			self::htmlTag('a', $linkAttribs, $out, true) //  output of link-tag
			);
	}

	protected static function getSearchObject(int $tid, string $name, string $out, array $linkAttribs): string{
		$search = we_tag_listview::$current;
		'@phan-var we_listview_search $search';
		if($tid){
			$data = [
				WEOBJ_PARAM => intval($search->f('OID')),
				'we_objectTID' => $tid
			];

			$tail = '&amp;we_objectTID=' . $tid . '&amp;jwt=' . we_helpers_jwt::encode(we_serialize($data, SERIALIZE_JSON), sha1(SECURITY_ENCRYPTION_KEY));
		} else {
			$tail = '';
		}

		$path_parts = pathinfo(we_http_util::getScriptName());
		$isSeo = $search->useSeoUrls();
		if($isSeo){
			$h = getHash('SELECT Url,TriggerID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . ((int) $search->f('OID')));
			list('TriggerID' => $objecttriggerid, 'Url' => $objecturl ) = $h;
			if($objecttriggerid){
				$path_parts = pathinfo(id2path($objecttriggerid));
			}
		} else {
			$objecturl = '?' . WEOBJ_PARAM . '=' . $search->f('OID') . '&';
		}

		$pidstr = '?pid=' . intval($search->f('WorkspaceID'));
		$linkAttribs['href'] = (!empty($search->hidedirindex) && seoIndexHide($path_parts['basename']) ?
			($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' .
			$objecturl .
			$pidstr :
			($isSeo && $objecturl ?
			($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' . $path_parts['filename'] . '/' . $objecturl :
			we_http_util::getScriptName() . $objecturl
			) . $pidstr);

		$linkAttribs['href'] .= $tail;

		return ($name === we_listview_base::HREF_FIELD ?
			$linkAttribs['href'] :
			self::htmlTag('a', $linkAttribs, $out, true) //  output of link-tag
			);
	}

}
