<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 44db4d7c3342 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 18:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifShopField extends we_weTag_ifTag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		self::attributeMissing($attribs, ['name' => false, 'reference' => false, 'shopname' => false, 'match' => true], __CLASS__);
		$match = self::getAttribute('match', $attribs, '', we_base_request::STRING);
		$operator = self::getAttribute('operator', $attribs, '', we_base_request::RAW);

		$atts = self::removeAttribs($attribs, ['match', 'operator']);
		$atts['type'] = 'print';
		$realvalue = (string) we_weTag_util::tag('shopField', $atts);

		switch($operator){
			default:
			case 'equal':
				return $realvalue === $match;
			case 'less':
				return intval($realvalue) < intval($match);
			case 'less|equal':
				return intval($realvalue) <= intval($match);
			case 'greater':
				return intval($realvalue) > intval($match);
			case 'greater|equal':
				return intval($realvalue) >= intval($match);
			case 'contains':
				return (strpos($realvalue, $match) !== false); //@phan-suppress-current-line PhanPluginSuspiciousParamOrderInternal
		}
	}

}
