<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 44db4d7c3342 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 18:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_votingList extends we_weTag_tag{

	public static function parse(array $attribs, string $content): string{
		self::attributeMissing($attribs, ['name' => false], __CLASS__);

		$attribs['_type'] = 'start';
		return '<?php ' . we_weTag_parser::printTag('votingList', $attribs) . '; ?>' . $content . '<?php ' . we_weTag_parser::printTag('votingList', ['_type' => 'stop']) . ';?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag
	 * @param bool $internal if used by we internally
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): void{
		if(!we_modules_info::isActive(we_modules_info::VOTING)){
			echo self::modulMissingError('Voting', __CLASS__);
			return;
		}
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$groupid = self::getAttribute('groupid', $attribs, 0, we_base_request::INT);
		$rows = self::getAttribute('rows', $attribs, 0, we_base_request::INT);
		$desc = self::getAttribute('desc', $attribs, false, we_base_request::BOOL);
		$order = self::getAttribute('order', $attribs, 'PublishDate', we_base_request::STRING);
		$subgroup = self::getAttribute('subgroup', $attribs, false, we_base_request::BOOL);
		$version = self::getAttribute('version', $attribs, 1, we_base_request::INT);
		$offset = self::getAttribute('offset', $attribs, 0, we_base_request::INT);
		$type = self::getAttribute('_type', $attribs, '', we_base_request::STRING);
		$start = max(we_base_request::int('_we_vl_start_' . $name, 0), 0);

		switch($type){
			case 'start':
				$lv = new we_voting_list($name, $groupid, (int) ($version > 0 ? ($version - 1) : 0), $rows, $offset, $desc, $order, $subgroup, $start);
				we_tag_listview::pre($lv);
				break;
			case 'stop':
				we_tag_listview::post();
				break;
		}
	}

}
