<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 31a8c102f0c9 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-01-03 01:13 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_weTag_attribute{
	/**
	 * @var int
	 */
	protected $Id;

	/**
	 * @var string
	 */
	protected $Name;

	/**
	 * @var boolean
	 */
	protected $Required;

	/**
	 * @var string
	 */
	protected $Module;

	/**
	 * @var string
	 */
	protected $Value;
	protected $Description;
	protected $Deprecated;

	abstract function getCodeForTagWizard(): string;

	/**
	 * @param string $name
	 * @param boolean $required
	 * @param string $module
	 */
	public function __construct(string $name, bool $required = false, string $module = '', string $description = '', bool $deprecated = false){
		static $count = 0;
		$this->Id = ++$count;
		$this->Name = $name;
		$this->Required = $required;
		$this->Module = $module;
		// set value occasionally
		$this->Value = we_base_request::string('attributes', '', $name);
		$this->Description = $description;
		$this->Deprecated = $deprecated;
	}

	/**
	 * @return string
	 */
	protected function getLabelCodeForTagWizard(): string{
		$tmp = [
			'id' => 'label_' . $this->getIdName(),
			'class' => 'defaultfont',
			'for' => $this->getIdName(),
			'style' => ($this->Description ? 'border-bottom-style: dotted;border-bottom-width: 1px;border-spacing: 2px;cursor:help;' : '') . ($this->Deprecated ? 'text-decoration:line-through;' : ''),
			'title' => ($this->Description ?: ''),
		];
		return we_html_element::label($tmp, $this->Name . ($this->Required ? '*' : ''));
	}

	/**
	 * @return string
	 */
	public function getName(): string{
		return $this->Name;
	}

	/**
	 * @return string
	 */
	public function getIdName(): string{
		return 'id' . $this->Id . '_' . $this->Name;
	}

	/**
	 * @return boolean
	 */
	public function IsRequired(): bool{
		return $this->Required;
	}

	public function IsDeprecated(): bool{
		return $this->Deprecated;
	}

	public function getDescription(): string{
		return $this->Description;
	}

	/**
	 * checks if this attribute should be used, checks if needed modules are installed
	 * @return boolean
	 */
	public function useAttribute(): bool{
		return ($this->Module === '' || we_modules_info::isActive($this->Module));
	}

	/**
	 * checks if this option should be used, checks if needed modules are installed
	 * @param we_weTag_option[] $options
	 * @return we_weTag_option[]
	 */
	public static function getUseOptions(array $options): array{
		$useOptions = [];
		foreach($options as $option){
			if($option->useOption()){
				$useOptions[] = $option;
			}
		}
		return $useOptions;
	}

}
