<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 44db4d7c3342 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 18:25 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_weTag_util{
	/** @var array<int,array> */
	private static $profile = [];

	/**
	 * @param string $name
	 * @param array<string,mixed> $attribs
	 * @param string $content
	 * @param bool $internal
	 * @return string
	 */
	public static function tag(string $name, array $attribs = [], string $content = '', bool $internal = false){
		//keep track of editmode
		$edMerk = we_base_WE::$editmode ?? '';
		//FIXME: do we support this????
		//$user = we_weTag_tag::getAttribute('user', $attribs, [], we_base_request::STRING_LIST);
		if(defined('WE_PROFILER')){
			static $last = WE_PROFILER;
			$bt = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 1);
			$cur = microtime(true);
			$prof = [
				'tag' => $name,
				'line' => $bt[0]['line'],
				'file' => $bt[0]['file'],
				'relTime' => round(($cur - $last) * 100, 2) . ' ms',
				'absTime' => round(($cur - WE_PROFILER) * 100, 2) . ' ms',
				'mem' => round(((memory_get_usage() / 1024))) . ' kB',
				''
			];
			$last = $cur;
			self::$profile[] = $prof;
			//annotate tag, if possible
			$attribs['data-time'] = $prof['relTime'];
			$attribs['data-mem'] = $prof['mem'];
			$attribs['data-line'] = $prof['line'];
			$attribs['data-file'] = $prof['file'];
		}
		//make sure comment attribute is never shown
		switch($name){
			case 'setVar':
			case 'xmlnode'://special handling inside tag setVar and xmlnode
				$attribs = we_weTag_tag::removeAttribs($attribs, ['cachelifetime', 'comment', 'user']);
				$nameTo = '';
				$to = 'screen';
				break;
			default:
				$to = we_weTag_tag::getAttribute('to', $attribs, 'screen', we_base_request::STRING);
				$nameTo = we_weTag_tag::getAttribute('nameto', $attribs, $attribs['name'] ?? '', we_base_request::STRING);
				$attribs = we_weTag_tag::removeAttribs($attribs, ['cachelifetime', 'comment', 'to', 'nameto', 'user']);

				/* if to attribute is set, output of the tag is redirected to a variable
				 * this makes only sense if tag output is equal to non-editmode */
				if($to != 'screen'){
					$GLOBALS['we_editmode'] = we_base_WE::$editmode = false;
				}
		}

		//make a copy of the name - this copy is never touched even not inside blocks/listviews etc.
		if(isset($attribs['name'])){
			$attribs['_name_orig'] = $attribs['name'];
			$attribs['name'] = we_weTag_tag::getPostName($attribs['name']);
			if(we_base_WE::$editmode && (we_base_WE::$we_doc instanceof we_document_webEdition)){
				we_base_WE::$we_doc->addUsedElement($name, $attribs['name']);
			}
		}

		$fn = 'we_tag_' . $name;
		try{
			we_weTag_util::includeTagFile($fn);
		} catch (we_exception_file $e){
			return $e->getMessage();
		}

		switch($name){
			case 'setVar':
				we_tag_setVar::tag($attribs);
				//nothing more to do don't waste time
				return '';
			default:
				try{
					$foo = class_exists($fn, false) && method_exists($fn, 'tag') ? $fn::tag($attribs, $content, $internal) : $fn($attribs, $content, $internal);
					$GLOBALS['we_editmode'] = we_base_WE::$editmode = $edMerk;
					return we_weTag_tag::redirectTagOutput((string) $foo, $nameTo, $to);
				} catch (we_exception_parser $e){
					return $e->getMessage();
				}
		}
	}

	public static function includeTagFile(string $filename): void{
		// as default: all tag_functions are in this file.
		if(class_exists($filename, true) && method_exists($filename, 'tag')){
			// do noting
			return;
		}

		throw new we_exception_file(we_weTag_tag::parseError(sprintf(g_l('parser', '[tag_not_known]'), str_replace('we_tag_', '', $filename))));
	}

	public static function we_profiler(bool $start = true): void{
		if($start){
			if(isset($_SERVER['REQUEST_TIME_FLOAT'])){
				define('WE_PROFILER', (float) $_SERVER['REQUEST_TIME_FLOAT']);
			} else {
				define('WE_PROFILER', microtime(true));
			}
			self::$profile = [];
		} else {
			echo implode(',', array_keys(self::$profile[0])) . "<br/>\n";
			foreach(self::$profile as $line){
				echo implode(',', $line) . "<br/>\n";
			}
		}
	}

}
