<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 7d142f7c469d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-06-24 11:26 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_widget_rss extends we_widget_base{
	/** @var string */
	private $rssUri = '';

	/** @var string */
	private $rssCont = '';

	/** @var int */
	private $rssNumItems = 0;

	/** @var string */
	private $rssTb = '';

	/** @var string */
	private $rssTitle = '';

	public function __construct(string $curID = '', array $aProps = []){
		list($this->rssUri, $this->rssCont, $this->rssNumItems, $this->rssTb, $this->rssTitle) = (!empty($aProps[3]) ?
			explode(',', $aProps[3]) : [//use default if data is corrupt
			base64_encode('http://www.webedition.org/de/rss/webedition.xml'),
			'111000',
			0,
			'110000',
			'1'
		]);
	}

	/**
	 * @return array{string,array{string,string}}
	 */
	public function getInsertDiv(int $iCurrId, array $aProps, we_base_jsCmd $jsCmd): array{
		list(/* $bTbLabel */, $bTbTitel/* , $bTbDesc, $bTbLink, $bTbPubDate, $bTbCopyright */) = str_split($this->rssTb);
		$aLabelPrefix = [];

		if($bTbTitel && $this->rssTitle){
			$feed = we_unserialize(we_base_preferences::getUserPref('cockpit_rss')) ?: we_widget_rss::getTopFeeds();
			foreach($feed as $aFeed){
				if($this->rssUri === $aFeed[1]){
					$aLabelPrefix[] = base64_decode($aFeed[0]) ?: '';
					break;
				}
			}
		}
		$sTbPrefix = implode(' - ', $aLabelPrefix);
		$aLang = [$sTbPrefix, ''];

		$oTblDiv = '<div class="rssDiv middlefont" id="m_' . $iCurrId . '_inline" style="width:100%;height:287px ! important; overflow: auto;"></div>';
		$jsCmd->addCmd('loadRSS', [base64_decode($this->rssUri) ?: '', $this->rssCont, $this->rssNumItems, $this->rssTb, $sTbPrefix, 'm_' . $iCurrId]);
		return [$oTblDiv, $aLang];
	}

	/**
	 *
	 * @return array<int,string[]>
	 */
	public static function getTopFeeds(): array{
		$aTopRssFeeds = we_base_country::g_l_a('topFeeds', '');
		'@phan-var array<int,string[]> $aTopRssFeeds';
		foreach($aTopRssFeeds as &$curTopFeed){
			foreach($curTopFeed as &$cur){
				$cur = base64_encode($cur);
			}
			return $aTopRssFeeds;
		}
		return [];
	}

	/**
	 * @return array<string,mixed>
	 */
	public static function getDefaultConfig(): array{
		return [
			'width' => self::WIDTH_SMALL,
			'expanded' => 0,
			'height' => 307,
			'res' => 0,
			'cls' => 'yellow',
			'csv' => base64_encode('http://www.webedition.org/de/feeds/aktuelles.xml') . ',111000,0,110000,1',
			'dlgHeight' => 480,
			'isResizable' => true
		];
	}

	private static function htmlClipElement(array &$dynvar, string $smalltext, string $text, string $content): string{
		static $unique = 0;
		$unique++;
		$dynvar['clip'][$unique] = [
			'state' => 0,
			'text' => $text,
			'textsmall' => $smalltext
		];

		$oClip = new we_html_table(['class' => 'default'], 1, 3);
		$oClip->setCol(0, 0, we_html_button::create_button(we_html_button::DIRRIGHT, 'javascript:clip(' . $unique . ');'), ['width' => '21', 'style' => 'vertical-align:top;text-align:right',
			'id' => "btn_" . $unique]);
		$oClip->setColAttributes(0, 1, ['width' => '10']);
		$oClip->setCol(0, 2, we_html_element::span(['id' => 'clip_' . $unique,
				'style' => "cursor:pointer;",
				'onclick' => "clip(" . $unique . ");"
				], addslashes($smalltext)));

		return $oClip->getHTML() . we_html_element::div(['id' => "div_" . $unique, 'style' => "display:none;"], we_html_element::br() . $content);
	}

	public static function showDialog(): void{
		$dynVar = [];
		$oIptUri = we_html_tools::formElementTable(g_l('cockpit', '[url]'), we_html_tools::textInput("ipt_uri", "", 255, [], "text", '380px'));

		$oSctRss = (new we_html_select(['name' => "sct_rss", 'class' => '', 'onchange' => "onChangeSctRss(this);"]))
			->insertOption(0, "", "");
		$oTblSctRss = we_html_tools::formElementTable(g_l('cockpit', '[rss_top_feeds]'), $oSctRss->getHTML());

		$oRemTopFeeds = we_html_tools::alertAttentionBox(g_l('cockpit', '[rss_edit_rem]'), we_html_tools::TYPE_INFO, 380);
		$oIptNewTitle = we_html_tools::formElementTable(g_l('cockpit', '[title]'), we_html_tools::textInput('ipt_newTitle', "", 255, [], "text", '380px'));
		$oIptNewUri = we_html_tools::formElementTable(g_l('cockpit', '[url]'), we_html_tools::textInput('ipt_newUri', "", 255, [], "text", '380px'));
		$btnAddTopRssFeed = we_html_button::create_button(we_html_button::ADD, "javascript:handleTopRssFeed('add');", '', "", false, false);
		$btnOverwriteTopRssFeed = we_html_button::create_button('overwrite', "javascript:handleTopRssFeed('overwrite');", '', "", false, false);
		$btnDeleteTopRssFeed = we_html_button::create_button(we_html_button::DELETE, "javascript:handleTopRssFeed('delete');", '', "", false, false);

		$oBtnNewFeed = new we_html_table(['class' => 'default'], 1, 5);
		$oBtnNewFeed->setCol(0, 0, $btnAddTopRssFeed);
		$oBtnNewFeed->setColAttributes(0, 1, ['style' => 'width:10px;']);
		$oBtnNewFeed->setCol(0, 2, $btnOverwriteTopRssFeed);
		$oBtnNewFeed->setColAttributes(0, 3, ['style' => 'width:10px;']);
		$oBtnNewFeed->setCol(0, 4, $btnDeleteTopRssFeed);

		$oNewFeed = new we_html_table(['width' => '390', 'class' => 'default'], 3, 1);
		$oNewFeed->setCol(0, 0, $oRemTopFeeds . we_html_element::br() . $oIptNewTitle . we_html_element::br() . $oIptNewUri);
		$oNewFeed->setColAttributes(1, 0, ['style' => 'width:5px;']);
		$oNewFeed->setCol(2, 0, $oBtnNewFeed->getHTML(), ['style' => "text-align:right"]);

		$rssUri = $oIptUri . we_html_element::br() . $oTblSctRss .
			we_html_element::br() .
			self::htmlClipElement($dynVar, g_l('cockpit', '[show_edit_toprssfeeds]'), g_l('cockpit', '[hide_edit_toprssfeeds]'), $oNewFeed->getHTML());

		$oRemRssConf = we_html_tools::alertAttentionBox(g_l('cockpit', '[rss_content_rem]'), we_html_tools::TYPE_INFO, 410);
		$oChbxContTitle = we_html_forms::checkbox('0', false, "chbx_conf", g_l('cockpit', '[title]'), true, "", "", false, "", 0, 0);
		$oChbxContLink = we_html_forms::checkbox('0', false, "chbx_conf", g_l('cockpit', '[link]'), true, "", "", false, "", 0, 0);
		$oChbxContDesc = we_html_forms::checkbox('0', false, "chbx_conf", g_l('cockpit', '[desc]'), true, "", "", false, "", 0, 0);
		$oChbxContEnc = we_html_forms::checkbox('0', false, "chbx_conf", g_l('cockpit', '[content_encoded]'), true, "", "", false, "", 0, 0);
		$oChbxContPubDate = we_html_forms::checkbox('0', false, "chbx_conf", g_l('cockpit', '[pubdate]'), true, "", "", false, "", 0, 0);
		$oChbxContCategory = we_html_forms::checkbox('0', false, "chbx_conf", g_l('cockpit', '[category]'), true, "", "", false, "", 0, 0);
		$oSctNumEntries = (new we_html_select(['name' => "sct_conf", 'class' => '']))
			->insertOption(0, '0', g_l('cockpit', '[no]'));

		for($iCurrEntry = 1; $iCurrEntry <= 50; $iCurrEntry++){
			$oSctNumEntries->insertOption($iCurrEntry, strval($iCurrEntry), strval($iCurrEntry));
			if($iCurrEntry >= 10){
				$iCurrEntry += ($iCurrEntry == 25) ? 24 : 4;
			}
		}

		$oRssContR = new we_html_table(['height' => "100%", 'class' => 'default'], 2, 3);
		$oRssContR->setCol(0, 0, g_l('cockpit', '[limit_entries]'), ['style' => 'vertical-align:middle;']);
		$oRssContR->setColAttributes(0, 1, ['style' => 'width:5px;']);
		$oRssContR->setCol(0, 2, $oSctNumEntries->getHTML(), ['style' => 'vertical-align:middle;']);
		$oRssContR->setCol(1, 0, $oChbxContPubDate . $oChbxContCategory, ['colspan' => '3', 'style' => 'vertical-align:bottom;']);

		$oSelectRssCont = new we_html_table(['class' => 'default'], 1, 2);
		$oSelectRssCont->setCol(0, 0, $oChbxContTitle . $oChbxContLink . $oChbxContDesc . $oChbxContEnc, ['width' => '165']);
		$oSelectRssCont->setCol(0, 1, $oRssContR->getHTML(), ['height' => "100%", 'style' => 'vertical-align:top;']);

		$rssConf = $oRemRssConf . we_html_element::br() . self::htmlClipElement($dynVar, g_l('cockpit', '[show_select_rsscontent]'), g_l('cockpit', '[hide_select_rsscontent]'), $oSelectRssCont->getHTML());

		$oRemLabel = we_html_tools::alertAttentionBox(g_l('cockpit', '[rss_label_rem]'), we_html_tools::TYPE_INFO, 410);
		$oChbxTb = [
			we_html_forms::checkbox('', false, 'chbx_tb', g_l('cockpit', '[label_rssfeed]'), true, '', '', false, '', 0, 0),
			we_html_forms::checkbox('', false, 'chbx_tb', g_l('cockpit', '[title]'), true, '', "onDisableRdoGroup('title');", false, '', 0, 0),
			we_html_forms::checkbox('', false, 'chbx_tb', g_l('cockpit', '[desc]'), true, '', '', false, '', 0, 0),
			we_html_forms::checkbox('', false, 'chbx_tb', g_l('cockpit', '[link_url]'), true, '', '', false, '', 0, 0),
			we_html_forms::checkbox('', false, 'chbx_tb', g_l('cockpit', '[pubdate]'), true, '', "", false, '', 0, 0),
			we_html_forms::checkbox('', false, 'chbx_tb', g_l('cockpit', '[copyright]'), true, '', '', false, '', 0, 0)
		];
		$oRdoTitle = [
			we_html_forms::radiobutton('1', false, 'rdo_title', g_l('cockpit', '[original_of_rssfeed]'), true, '', '', false, '', 0, ''),
			we_html_forms::radiobutton('0', false, 'rdo_title', g_l('cockpit', '[personalized]'), true, '', '', false, '', 0, '')
		];

		$oTitleTb = new we_html_table(['class' => 'default'], 2, 1);
		$oTitleTb->setCol(0, 0, $oRdoTitle[0], ['width' => '165']);
		$oTitleTb->setCol(1, 0, $oRdoTitle[1], ['width' => '165']);

		$oEditTb = new we_html_table(['class' => 'default'], 6, 2);
		$oEditTb->setCol(0, 0, $oChbxTb[0], ['width' => '165']);
		$oEditTb->setCol(1, 0, $oChbxTb[1], ['width' => '165', 'style' => 'vertical-align:top;']);
		$oEditTb->setCol(1, 1, $oTitleTb->getHTML(), ['width' => '165']);
		$oEditTb->setCol(2, 0, $oChbxTb[2], ['width' => '165']);
		$oEditTb->setCol(3, 0, $oChbxTb[3], ['width' => '165']);
		$oEditTb->setCol(4, 0, $oChbxTb[4], ['width' => '165']);
		$oEditTb->setCol(5, 0, $oChbxTb[5], ['width' => '165']);

		$rssLabel = $oRemLabel . we_html_element::br() . self::htmlClipElement($dynVar, g_l('cockpit', '[show_edit_titlebar]'), g_l('cockpit', '[hide_edit_titlebar]'), $oEditTb->getHTML());

		list($jsFile, $oSelCls) = self::getDialogPrefs($dynVar);

		$buttons = we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::SAVE, 'javascript:save();'), we_html_button::create_button(we_html_button::PREVIEW, 'javascript:preview();'), we_html_button::create_button(we_html_button::CLOSE, 'javascript:exit_close();'));

		$sTblWidget = (new we_html_multiIconBox((new we_html_multiIconBoxContent())
				->add(new we_html_multiIconBoxContentLine('', $rssUri))
				->add(new we_html_multiIconBoxContentLine('', $rssConf))
				->add(new we_html_multiIconBoxContentLine('', $rssLabel))
				->add(new we_html_multiIconBoxContentLine('', $oSelCls->getHTML()))
				, 'rssProps'))
			->marginLeft()
			->buttons($buttons)
			->headline(g_l('cockpit', '[rss_feed]'))
			->get();

		echo we_html_tools::getHtmlTop(g_l('cockpit', '[rss_feed]'), $jsFile .
			we_html_element::jsScript(JS_DIR . 'widgets/rss.js'), we_html_element::body(
				['class' => 'weDialogBody', 'onload' => 'init();'], we_html_element::form([], $sTblWidget)
		));
	}

	public function showPreview(): void{
		$jsCmd = new we_base_jsCmd();
		$jsCmd->addCmd('loadRSS', we_base_request::string('we_cmd'));
		echo we_html_tools::getHtmlTop('', $jsCmd->getCmds(), we_html_element::body(['onload' => 'init()']));
	}

	/**
	 *
	 * @param string $sRssUri
	 * @param string $sCfgBinary
	 * @param int $iNumItems
	 * @param string $sTbBinary
	 * @param string $tmpTitle
	 * @return string[]
	 */
	public static function getRSSContent(string $sRssUri, string $sCfgBinary, int $iNumItems, string $sTbBinary, string $tmpTitle): array{

		$bCfgTitle = (bool) $sCfgBinary[0];
		$bCfgLink = (bool) $sCfgBinary[1];
		$bCfgDesc = (bool) $sCfgBinary[2];
		$bCfgContEnc = (bool) $sCfgBinary[3];
		$bCfgPubDate = (bool) $sCfgBinary[4];
		$bCfgCategory = (bool) $sCfgBinary[5];
		switch($iNumItems){
			case 11:
				$iNumItems = 15;
				break;
			case 12:
				$iNumItems = 20;
				break;
			case 13:
				$iNumItems = 25;
				break;
			case 14:
				$iNumItems = 50;
				break;
		}

		$bTbLabel = (bool) $sTbBinary[0];
		$bTbTitel = (bool) $sTbBinary[1];
		$bTbDesc = (bool) $sTbBinary[2];
		$bTbLink = (bool) $sTbBinary[3];
		$bTbPubDate = (bool) $sTbBinary[4];
		$bTbCopyright = (bool) $sTbBinary[5];

		//Bug 6119: Keine Unterstützung für curl in der XML_RSS Klasse
		//daher Umstellung den Inhalt des Feeds selbst zu holen

		$oRssParser = new we_xml_rss(); //this is equal to we_tag_xmlfeed
		/* $cache = WE_CACHE_PATH . 'xmlfeed_' . md5($sRssUri);

		  $do_refresh = (!is_file($cache) || ((filemtime($cache) + 3600) < time())); */

		//if($do_refresh){//FIXME: cache does not work since both classes uses different struct
		/* $ret = */$oRssParser->getFile($sRssUri, 20);
		/* if($ret){
		  $oRssParser->saveCache($cache, (2 * 3600)); //keep file longer, in case of timeouts
		  } else if(is_file($cache)){
		  //timeout + last file exists
		  $oRssParser->loadCache($cache);
		  } else {
		  //in case of an parser error, just return
		  return ['', ''];
		  }
		  } else {
		  $oRssParser->loadCache($cache);
		  } */

		$sRssOut = '';
		$iCurrItem = 0;
		foreach($oRssParser->getItems() as $item){
			assert(is_array($item));
			$bShowTitle = ($bCfgTitle && isset($item['title']));
			$bShowLink = ($bCfgLink && isset($item['link']));
			$bShowDesc = ($bCfgDesc && isset($item['description']));
			$bShowContEnc = ($bCfgContEnc && isset($item['content:encoded']));
			$bShowPubdate = ($bCfgPubDate && isset($item['pubdate']));
			$bShowCategory = ($bCfgCategory && isset($item['category']));

			$sLink = (($bCfgLink && isset($item['link'])) && !$bShowTitle) ? ' &nbsp;' .
				we_html_element::a(['href' => $item['link'], 'target' => '_blank'], g_l('cockpit', '[more]')) : '';
			if($bShowContEnc){
				$contEnc = new we_html_table(['class' => 'default'], 1, 1);
				$contEnc->setCol(0, 0, $item['content:encoded'] . ((!$bCfgDesc) ? $sLink : ''));
			} else {
				$contEnc = null;
			}

			$sRssOut .= ($bShowTitle ?
				($bShowLink ? we_html_element::a(['href' => $item['link'], 'target' => '_blank'], we_html_element::b($item['title'])) :
				we_html_element::b($item['title'])) .
				($bShowPubdate ? ' ' : we_html_element::br()) :
				'') .
				($bShowPubdate ?
				g_l('cockpit', '[published]') . ': ' . (($pub = strtotime($item['pubdate'])) ? date(g_l('date', '[format][default]'), (int) $pub) : '') :
				'') .
				($bShowCategory ?
				($bShowPubdate ? we_html_element::br() : '') .
				g_l('cockpit', '[category]') . ': ' . $item['category'] :
				'') .
				($bShowPubdate || $bShowCategory ?
				we_html_element::br() :
				'') .
				($bShowDesc ?
				$item['description'] . $sLink . we_html_element::br() :
				"") .
				($bShowContEnc ?
				$contEnc->getHTML() :
				(!$bShowDesc ?
				$sLink . we_html_element::br() :
				'')
				) .
				($bShowDesc || $bShowContEnc ?
				we_html_element::br() :
				"");
			if($iNumItems){
				$iCurrItem++;
				if($iCurrItem == $iNumItems){
					break;
				}
			}
		}

		$aTb = [];
		if($bTbLabel){
			$aTb[] = g_l('cockpit', '[rss_feed]');
		}
		$channel = $oRssParser->getChannelInfo();
		if($bTbTitel){
			$aTb[] = $tmpTitle ?: ($channel['title'] ?? '');
		}
		if($bTbDesc){
			$aTb[] = (isset($channel['description'])) ? str_replace(["\n", "\r"], '', $channel['description']) : '';
		}
		if($bTbLink){
			$aTb[] = $channel['link'] ?? '';
		}
		if($bTbPubDate){
			$aTb[] = (isset($channel['pubdate']) && ($pub = strtotime($channel['pubdate']))) ? (date(g_l('date', '[format][default]'), (int) $pub)) : '';
		}
		if($bTbCopyright){
			$aTb[] = $channel['copyright'] ?? '';
		}
		$title = implode(' - ', $aTb);
		return [$title, $sRssOut];
	}

}
