<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 548293d99025 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-09-11 01:15 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_wizard_code{
	/**
	 * Directory where the snippets are located
	 *
	 * @var string
	 */
	const SnippetPath = 'codesnippets';

	/**
	 * get all custom specific snippets
	 *
	 * @return array
	 */
	private static function getSnippets(string $SnippetDir): array{
		$snippets = [];
		return (!is_dir($SnippetDir) ? [] : self::getSnippetsByDir($SnippetDir, 0, $snippets));
	}

	/**
	 * get snippets by directory name
	 *
	 * @return array
	 */
	private static function getSnippetsByDir(string $SnippetDir, int $Depth = 0, array &$Snippets = []): array{
		$Depth++;
		$dir = dir($SnippetDir);
		while(false !== ($entry = $dir->read())){

			// ignore files . and ..
			if($entry === '.' || $entry === '..'){
				continue;
				// ignore these
				// get the snippets by file if extension is xml
			}
			if(!is_dir($SnippetDir . '/' . $entry) && preg_match('-^.*\.json$-', $entry)){
				// get the snippet
				$snippet = new we_wizard_codeSnippet($SnippetDir . '/' . $entry);
				$Snippets[$snippet->folder] = [
					'type' => 'option',
					'name' => $snippet->getName(),
					'value' => $SnippetDir . '/' . $entry
				];

				// enter subdirectory only if depth is smaller than 2
			} elseif(is_dir($SnippetDir . '/' . $entry) && $Depth < 2){
				self::getSnippetsByDir($SnippetDir . '/' . $entry, $Depth, $Snippets);
			}
		}
		$dir->close();

		return $Snippets;
	}

	/**
	 * create the select box to select a snippet
	 *
	 * @param string $type
	 * @return string
	 */
	public static function getSelect(string $type = 'standard'): string{
		switch($type){
			case 'custom':
				$options = self::getSnippets(WE_CUSTOM_PATH . self::SnippetPath);
				break;

			default:
				$options = self::getSnippets(WE_INCLUDES_PATH . self::SnippetPath);
				break;
		}

		$select = '<select id="codesnippet_' . $type . "\" name=\"codesnippet_" . $type . "\"  size=\"7\" style=\"width:250px; height: 100px; display: none;\" ondblclick=\"doAjax(this.value);\" onchange=\"WE().layout.button.enable(document, 'btn_direction_right_applyCode')\">\n";
		$oldFolder = '';
		foreach($options as $folder => $option){
			if($oldFolder != $folder){
				$select .= ($oldFolder === '' ? '' : '</optgroup>') . '<optgroup label="' . $folder . '">';
				$oldFolder = $folder;
			}
			$select .= '<option value="' . $option['value'] . '">' . $option['name'] . '</option>';
		}

		return $select .
			($oldFolder === '' ? '' : '</optgroup>') .
			'</select>';
	}

}
