<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_wysiwyg_ToolbarElement{
	/** @var string */
	var $cmd;

	/** @var we_wysiwyg_editor */
	var $editor;

	/** @var string */
	protected $scope = '';

	/** @var bool */
	protected $isSeparator = false;

	/** @var bool */
	protected $show = false;

	/** @var string */
	protected $group = '';

	const SCOPE_TOOLBAR = 'toolbar';
	const SCOPE_QUICKBAR = 'quickbar';
	const SCOPE_CONTEXTMENU = 'contextmenu';

	function __construct(we_wysiwyg_editor $editor, string $cmd, string $scope = self::SCOPE_TOOLBAR, string $group = ''){
		$this->editor = $editor;
		$this->cmd = $cmd;
		$this->scope = $scope;
		$this->show = $this->hasProp();
		$this->group = $group;
	}

	public function getGroup(): string{
		return $this->group;
	}

	public function isSeparator(): bool{
		return $this->isSeparator;
	}

	public function isShowInToolbar(): bool{
		return $this->scope === self::SCOPE_TOOLBAR && $this->show;
	}

	public function isShowInQuickbar(): bool{
		return $this->scope === self::SCOPE_QUICKBAR && $this->show;
	}

	public function isShowInContextmenu(): bool{
		return $this->scope === self::SCOPE_CONTEXTMENU && $this->show;
	}

	function hasProp(string $cmd = ''): bool{
		$cmd = ($cmd ?: $this->cmd);

		switch($this->scope){
			case self::SCOPE_CONTEXTMENU:
				return stripos($this->editor->getRestrictContextmenu(), ',' . $cmd . ',') !== false;
			default: // classic toolbar
				return !$this->editor->getPropArr() || in_array($cmd, $this->editor->getPropArr());
		}
	}

}
